//
//  JTMAboutUsViewController.swift
//  JobThai
//
//  Created by ForAlL on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMAboutUsViewController: JTMBaseMenuViewController, UITableViewDataSource, UITableViewDelegate, JTMNoticeDelegate {
    
    
    @IBOutlet weak var tableView: UITableView!
    
    var sectionObjectList = NSMutableArray()
    var noticeList = [NoticeModel]()
    var noticeNewCount : Int = 0
    var isLoadNoticeFromService : Bool = false
    
    let TABLE_VIEW_BG_COLOR = UIColor.whiteColor()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.getValueFromPList()
        self.connectNoticeService()
        self.setUpTableView()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_ABOUT_US)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_VIEW, label: LABEL_VIEW_ABOUT_US)
        
        if let selectedIndexPath = self.tableView.indexPathForSelectedRow {
            self.tableView.deselectRowAtIndexPath(selectedIndexPath, animated: false)
        }
        let newCount : Int = (applicationDelegate.appSetting?.getNoticeNewCount())!
        if newCount != self.noticeNewCount{
            self.noticeNewCount = newCount
            self.tableView.reloadData()
        }
    }
    
    //MARK:- Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if !isLoadNoticeFromService {
            connectNoticeService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
    }
    
    override func showAlertConnectionError() {
        //We don't need this controller to alert on connection error
    }
    
    //Mark:- UIBarButton
    override func genUISearchMenuButton() -> UIBarButtonItem? {
        return nil
    }
    
    //MARK: Connect Service & Parse Json for JTMNoticeViewController
    func connectNoticeService(completion: ((ServiceResponseStatus)->Void)? = nil) {
        
        let serviceController = JTMServiceController()
        let url = getServiceNotice()
        
        serviceController.requestService(self, urlString: url, params: nil, isCanLoadParallelToOther: true, isPresentLoadingIndicator: false) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                self.isLoadNoticeFromService = true
                let jsonResult = response!.result.value as? NSMutableDictionary
                self.noticeList.removeAll()
                self.parseJsonToModel(jsonResult!)
            }
            
            completion?(status)
        }
    }
    
    func parseJsonToModel(jsonObject : NSMutableDictionary) {
        let noticeReadIDList = applicationDelegate.appSetting!.getNoticeReadIdList().mutableCopy() as! NSMutableDictionary
        for item in (jsonObject.objectForKey("notices") as? NSArray)!{
            let noticeData : NoticeModel = NoticeModel(notice: item as! NSMutableDictionary)
            if (noticeReadIDList.objectForKey(noticeData.id!) != nil) {
                noticeData.read = true
            }
            else{
                noticeData.read = false
            }
            noticeList.append(noticeData)
        }
        noticeNewCount = noticeList.count - noticeReadIDList.count
        applicationDelegate.appSetting?.saveNoticeNewCount(noticeNewCount)
        self.tableView.reloadData()
    }

    //MARK: ---
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_ABOUT_US")
        self.connectNoticeService()
        self.tableView.reloadData()
    }
    
    func getValueFromPList() {
        if let path = NSBundle.mainBundle().pathForResource("ListAboutUs", ofType: "plist") {
            let listSetting = NSMutableArray(contentsOfFile: path)!
            for sectionObject in listSetting {
                sectionObjectList.addObject(sectionObject as! NSMutableDictionary)
            }
        }
    }
    
    func setUpTableView() {
//        self.tableView.backgroundColor = JTM_LIGHT_GRAY
        self.tableView.backgroundColor = TABLE_VIEW_BG_COLOR
        self.tableView.tableFooterView = UIView(frame: CGRectZero)
//        self.tableView.estimatedRowHeight = 44
        self.tableView.estimatedRowHeight = 50
        self.tableView.rowHeight = UITableViewAutomaticDimension
        self.tableView.scrollEnabled = false
        
        self.tableView.registerNib(UINib(nibName: "JTMAboutUsTableViewCell", bundle: nil), forCellReuseIdentifier: "JTMAboutUsTableViewCell")
    }
    
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        print("Click : \(indexPath.row)")
        if(indexPath.row == 0) {
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_CLICK, label: LABEL_NOTICES)
            pushNoticeViewWithNavigation("JTMNoticeViewController")
        } else if(indexPath.row == 1) {
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_CLICK, label: LABEL_OUR_PRODUCT)
            let ourProducts = sectionObjectList[indexPath.row] as! NSDictionary
            var keyIndex: String
            if(applicationDelegate.appSetting?.currentLanguage == .THAI) {
                keyIndex = "nameTH"
            } else {
                keyIndex = "nameEN"
            }
            openProductUs(ourProducts.objectForKey(keyIndex) as! String)
        } else if(indexPath.row == 2) {
            pushAboutMenuWithNavigation("JTMContactUsViewController")
        } else if(indexPath.row == 3) {
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_CLICK, label: LABEL_VERSION)
            pushAboutMenuWithNavigation("JTMVersionViewController")
        } else if(indexPath.row == 4) {
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_CLICK, label: LABEL_RATE_JOBTHAI_APPLICATION)
            openAppStore();
        } else if(indexPath.row == 5) {
            // Track Google Analytic
            applicationDelegate.trackGoogleAnalyticEvent(CAT_ABOUT_US, action: ACTION_CLICK, label: LABEL_QUICK_GUIDE)
            pushAboutUsMenu("JTMMainQuickGuideViewController")
        }
        
        tableView.deselectRowAtIndexPath(indexPath, animated: true)
        
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return sectionObjectList.count
    }
    
    //    func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
    //        let nib = UINib(nibName: "JTMSettingTableHeaderView", bundle: nil)
    //        let headerView = nib.instantiateWithOwner(self, options: nil)[0] as? JTMSettingTableHeaderView
    //
    //        if (applicationDelegate.appSetting?.currentLanguage == .Thai) {
    //            headerView?.headerTitle.text = self.sectionObjectList[section].objectForKey("nameTH") as? String
    //        } else {
    //            headerView?.headerTitle.text = self.sectionObjectList[section].objectForKey("nameEN") as? String
    //        }
    //
    //        return headerView
    //    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }

    func tableView(tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        return 0.5
    }
    
    func tableView(tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        let footerView = UIView(frame: CGRectMake(0, 0, tableView.frame.width, 1))
        footerView.backgroundColor = JTM_LIGHT_GRAY
//        footerView.backgroundColor = TABLE_VIEW_BG_COLOR
        
        return footerView
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        let rowItem = sectionObjectList[indexPath.row] as! NSDictionary
        let languageKey = (applicationDelegate.appSetting?.currentLanguage == .THAI) ? "nameTH" : "nameEN"
        
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMAboutUsTableViewCell") as! JTMAboutUsTableViewCell
        
        setSelectedBackgroundView(cell)
        
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
        
        let name = rowItem.objectForKey(languageKey) as? String

        cell.title?.text = name
        cell.subTitle?.text = rowItem.objectForKey("description") as? String
        
        if(name == "Version" || name == "เวอร์ชัน") {
             cell.subTitle?.text = version
        }
        
        if(name == "ประกาศ" || name == "Notices") {
            cell.setNoticeLabelShow(noticeNewCount)
        }
        
        return cell
    }
    
    func pushAboutUsMenu(identifierId: String){
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_INTRODUCING)
        
        let quickGuidView = self.storyboard?.instantiateViewControllerWithIdentifier(identifierId)
        self.presentViewController(quickGuidView!, animated: true, completion: nil)
    }
    
    func pushAboutMenuWithNavigation(identifierId: String){
        let quickGuidView = self.storyboard?.instantiateViewControllerWithIdentifier(identifierId)
        self.navigationController?.pushViewController(quickGuidView!, animated: true)
    }

    func pushNoticeViewWithNavigation(identifierId: String){
        let noticeViewController : JTMNoticeViewController = (self.storyboard?.instantiateViewControllerWithIdentifier(identifierId) as? JTMNoticeViewController)!
        noticeViewController.noticeList = noticeList
        noticeViewController.isLoadNoticeFromService = isLoadNoticeFromService
        noticeViewController.delegate = self
        self.navigationController?.pushViewController(noticeViewController, animated: true)
    }
    
    func openProductUs(title: String) {
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_OUR_PRODUCTS)
        
        let webView = self.storyboard?.instantiateViewControllerWithIdentifier("JTMWebViewController") as? JTMWebViewController
        webView?.webTitle =  title
        webView?.urlString = getServiceOurProduct()
        
        self.navigationController?.pushViewController(webView!, animated: true)
    }
    
    func openAppStore() {
        let url  = NSURL(string: APP_STORE_URL_STRING)
        if UIApplication.sharedApplication().canOpenURL(url!) {
            UIApplication.sharedApplication().openURL(url!)
        }
    }
    
    //MARK:- Notice delegate
    func updateNoticeList(noticeController: JTMNoticeViewController) {
        noticeController.showLoading()
        self.connectNoticeService { (status: ServiceResponseStatus) -> () in
            noticeController.hideLoading()
            if (status == .CONNECTION_SUCCESS) {
                noticeController.noticeList = self.noticeList
                noticeController.isLoadNoticeFromService = self.isLoadNoticeFromService
                noticeController.noticeTableView.reloadData()
            } else if (status == .CONNECTION_FAIL) {
                noticeController.onConnectionError(nil)
            }
        }
    }
}
