//
//  JTMApplyNowViewController.swift
//  ApplyNow
//
//  Created by HARIKAN on 4/4/16.
//  Copyright © 2016 HARIKAN. All rights reserved.
//

import UIKit
import DynamicColor
import Alamofire

class JTMApplyNowViewController: JTMBaseViewController, DelegateApplyNow {
    var jobData:PositionModel?
    var companyName: String?
    
    @IBOutlet var applyNowView: JTMApplyNowView!

    override func viewDidLoad() {
        super.viewDidLoad()
        setUpNavigationBarAndButtonBackApplyNow()
        applyNowView.delegate = self
        applyNowView.setValue(companyName!, jobData: jobData!)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_APPLY_NOW)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLY_NOW, action: ACTION_VIEW, label: LABEL_VIEW_APPLY_NOW)
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
    }
    
    func setUpNavigationBarAndButtonBackApplyNow(){
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_APPLY_NOW")
    }
    
    //MARK:- APPLY DELEGATE
    func onClickApplyNowButton(button: UIView, note: String, countString : String, isSendEmail: Bool) {

        if (Int(countString)) <= 200 {
            setTrackGoogleAnalyticApplyNow(note, isSendEmail: isSendEmail)
            callServiceApplyJob(note, isSendEmail: isSendEmail)
        }
        
    }
    
    func callServiceApplyJob(note: String, isSendEmail: Bool) {
        
        let serviceController = JTMServiceController()
        let url = getServiceApplyNow()
        let userData = applicationDelegate.appSetting?.userData
        let email = applicationDelegate.appSetting?.userData?.userName
        let sessionId = applicationDelegate.appSetting?.userData?.sessionId
        let param: [String:AnyObject] = [
            "imei"      : applicationDelegate.getDeviceUUID(),
            "token"     : (userData?.token)!,
            "jobcode"   : (jobData?.runningNumber)!,
            "note"      : note,
            "ccemail"   : isSendEmail.hashValue,
            "emailForCC": email!,
            "sessionId" : sessionId!
        ]
        
        serviceController.requestService(self, urlString: url, params: param) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let resultJSON = response!.result.value as! NSMutableDictionary
                
                if (resultJSON.objectForKey("status") as! String == "ok") {
                    self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_APPLY"),
                                            message: applicationDelegate.getLocalizeStringForKey("ALERT_APPLY_SUCCESS"),
                                            completionHandler: {(AlertAction)->Void in
                                                self.navigationController?.popViewControllerAnimated(true)
                                            }, actionList: nil)
                    NSNotificationCenter.defaultCenter().postNotificationName(JTM_USER_JUST_APPLIED_JOB, object: nil)
                    
                } else {
                    let errorMessage = resultJSON.objectForKey("errmsg") as! String
                    self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_APPLY"),
                                            message: errorMessage,
                                            completionHandler: {(AlertAction)->Void in
                                                self.navigationController?.popViewControllerAnimated(true)
                                            }, actionList: nil)
                }
            }
        }
    }
    
    func setTrackGoogleAnalyticApplyNow(note: String, isSendEmail: Bool) {
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLY_NOW, action: ACTION_CLICK, label: LABEL_APPLY_NOW)
        
        if note != "" {
            applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLY_NOW, action: ACTION_CLICK, label: LABEL_NOTE)
        }
        
        let sendEmailLabel = isSendEmail ? LABEL_FORWARD_EMAIL : LABEL_DONT_FORWARD_EMAIL
        applicationDelegate.trackGoogleAnalyticEvent(CAT_APPLY_NOW, action: ACTION_CLICK, label: sendEmailLabel)
    }
    
//    func alertLogInApplyNow(){
//        let alertControllerLoginApplyNow = UIAlertController(title: "แจ้งเตือน", message: "ยังไม่เข้าสู่ระบบ", preferredStyle: .Alert)
//        alertControllerLoginApplyNow.addAction(UIAlertAction(title: "เข้าสู่ระบบ", style: UIAlertActionStyle.Default, handler: { (UIAlertAction) -> Void in
//            print("log-in")
//        }))
//        alertControllerLoginApplyNow.addAction(UIAlertAction(title: "ตกลง", style: UIAlertActionStyle.Default, handler: nil))
//        self.presentViewController(alertControllerLoginApplyNow, animated: true, completion: nil)
//    }
//
//    func alertApplyNow(alertTitleApplyNow:String, alertTextApplyNow:String, alertDimissApplyNow:String ){
//        let alertControllerApplyNow = UIAlertController(title: alertTitleApplyNow, message: alertTextApplyNow , preferredStyle: .Alert)
//        alertControllerApplyNow.addAction(UIAlertAction(title: alertDimissApplyNow, style: UIAlertActionStyle.Default, handler: nil))
//        self.presentViewController(alertControllerApplyNow, animated: true, completion: nil)
//    }
}
