//
//  JTMBaseSearchViewController.swift
//  JobThai
//
//  Created by kanya on 3/14/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMBaseSearchViewController: JTMBaseViewController, UITableViewDataSource, UITableViewDelegate, UISearchResultsUpdating, UISearchBarDelegate, UISearchControllerDelegate, UITextFieldDelegate, JTMSearchSuggestionControllerDelegate , JTMSearchFooterTableViewCellDelegate , JTMUpdateSaveStateDelegate{
    
    @IBOutlet weak var tableView: UITableView!
//    @IBOutlet weak var tableHeightConstraint: NSLayoutConstraint!

    var parentController: JTMSearchFilterTypeViewController?
    let appSetting = applicationDelegate.appSetting
    var searchSuggestViewController: JTMSearchSuggestViewController?
    var searchController: UISearchController!
    
    var cellSearch: JTMSearchKeyWordTableViewCell?
    var cellFooter : JTMSearchFooterTableViewCell?
    
    let DEFAULT_CELL_SCALE: CGFloat = 1/6 //1/5.5
    let EXTRA_CELL_SCALE: CGFloat = 1.15//1.25 //for cell with area select, cell size is little bigger than other
    let DEFAULT_NUMBER_OF_CELL = 4//3
    let NAVIGATIONBAR_HEIGHT: CGFloat = 40
    let SEARCHBAR_HEIGHT:CGFloat = 44
    let JOBCATE_CELL_INDEX = 0
    
    let CELL_SIZE_BIG    : CGFloat = 135
    let CELL_SIZE_SMALL  : CGFloat = 95
    
    //default if no selected filter
    let DEFAULT_KEYWORD_FOR_NIL_TYPE = "All"
    let DEFAULT_KEYWORD_CATEGORY = applicationDelegate.getLocalizeStringForKey("SUB_TITLE_JOB_CATEGORY")
    let DEFAULT_KEYWORD_LOCATION = applicationDelegate.getLocalizeStringForKey("SUB_TITLE_JOB_LOCATION")
    let DEFAULT_KEYWORD_INDUSTRIAL = applicationDelegate.getLocalizeStringForKey("SUB_TITLE_JOB_INDUSTRIAL_ESTATES")
    let DEFAULT_KEYWORD_TRANSPORTATION = applicationDelegate.getLocalizeStringForKey("SUB_TITLE_JOB_TRANSPORTATION")
    
    let NOTI_RELOAD_JOBCATEGORY_CELL    : String = "NOTI_RELOAD_JOBCATEGORY_CELL"
    let NOTI_RELOAD_KEYWORD_CELL        : String = "NOTI_RELOAD_KEYWORD_CELL"
    let NOTI_RELOAD_DISABILITY_SWITCH   : String = "NOTI_RELOAD_DISABILITY_SWITCH"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
//        cellHeight = (self.view.frame.height - NAVIGATIONBAR_HEIGHT) * DEFAULT_CELL_SCALE
//        bigCellHeight = cellHeight! * EXTRA_CELL_SCALE
        
        loadRequiredFilterListFromService()
        initSearchController()
        onLanguageChange()
        registerNibForTableView()
        initObserver()
        initTableView()
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = !isHasAllRequireFilterListValue()
    }
    
    func isHasAllRequireFilterListValue() -> Bool {
        if (applicationDelegate.appSetting?.getSearchSuggestionListFromUserDefault().count > 0)
            && (applicationDelegate.appSetting?.searchFilterJobCategoryList.count > 0) {
                return true
        }
        
        return false
    }
   
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
    }
    
    //MARK:- Connection Error 
    override func onRetryConnection() {
        super.onRetryConnection()
        
        if applicationDelegate.appSetting?.getSearchSuggestionListFromUserDefault().count == 0 {
            loadListSuggestionFromService()
        }
        
        if (applicationDelegate.appSetting?.searchFilterJobCategoryList.count == 0) {
            loadJobCategoryListFromService()
        }
    }
    
    //Not need controller hide connection error until it finish loading all required filter
    override func onConnectionSuccess() {
        if isHasAllRequireFilterListValue() {
            super.onConnectionSuccess()
        }
    }
    
    //check to ignore error if the filter requested already has value
    override func onConnectionError(errorRequestUrl: String?) {
        if ((errorRequestUrl! == getServiceSearchSuggesttion()) && (applicationDelegate.appSetting?.getSearchSuggestionListFromUserDefault().count > 0))
            || ((errorRequestUrl! == getServiveSearchFilter(.CATEGORY)) && (applicationDelegate.appSetting?.searchFilterJobCategoryList.count > 0)) {
                return
        }
        
        super.onConnectionError(errorRequestUrl)
    }
    
    //MARK:- Manage View On Error :: show connection error view if 1 or more required filter still nil
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if view == connectionErrorView && !isHasAllRequireFilterListValue() {
            setAlphaShowToView(view)
        }
    }
    
    //MARK:- Nib & Observer
    func registerNibForTableView() {
        self.tableView?.registerNib(UINib(nibName: "JTMSeachFilterTableViewCell", bundle: nil),
                                    forCellReuseIdentifier: "JTMSeachFilterTableViewCell")
        
        self.tableView?.registerNib(UINib(nibName: "JTMSearchKeyWordTableViewCell", bundle: nil),
                                    forCellReuseIdentifier: "JTMSearchKeyWordTableViewCell")
        
        self.tableView?.registerNib(UINib(nibName: "JTMSearchFooterTableViewCell", bundle: nil),
                                    forCellReuseIdentifier: "JTMSearchFooterTableViewCell")
    }
    
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self,
                                                         selector: #selector(reloadJobCategoryCell(_:)) ,
                                                         name: NOTI_RELOAD_JOBCATEGORY_CELL,
                                                         object: nil)
        
        NSNotificationCenter.defaultCenter().addObserver(self,
                                                         selector: #selector(reloadKeywordCell(_:)) ,
                                                         name: NOTI_RELOAD_KEYWORD_CELL,
                                                         object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self,
                                                         selector: #selector(reloadDisabilitySwitch(_:)) ,
                                                         name: NOTI_RELOAD_DISABILITY_SWITCH,
                                                         object: nil)
    }
    
    //MARK:- TableView
    func initTableView() {
        tableView.dataSource = self
        tableView.delegate = self
        tableView.tableFooterView = UIView(frame: CGRectZero)
//        tableHeightConstraint.constant = cellHeight! * CGFloat(DEFAULT_NUMBER_OF_CELL)
        tableView.accessibilityIdentifier = ID_SEARCH_FILTER_DETAIL_TABLE_VIEWS
    }
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return DEFAULT_NUMBER_OF_CELL
    }
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return CELL_SIZE_SMALL
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSeachFilterTableViewCell") as! JTMSeachFilterTableViewCell
        
        return configureCellForRowAtIndexPath(cell, indexPath: indexPath)
    }
    
    func configureCellForRowAtIndexPath(cell: JTMSeachFilterTableViewCell, indexPath: NSIndexPath) -> UITableViewCell {
        return cell
    }
    
    func configureJobCategoryCell(cell: JTMSeachFilterTableViewCell) {
        cell.setDescription(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_CATEGORY"),
            subTitle: getKeywordJobCategoryFromSaveState())
    }
    
    func configureCellSearch() -> JTMSearchKeyWordTableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchKeyWordTableViewCell") as? JTMSearchKeyWordTableViewCell
        cell!.selectionStyle = UITableViewCellSelectionStyle.None
        cell!.editSearchKeyWord?.delegate = self
        cell?.setKeywordToEditText((parentController?.searchFilterSaveState?.searchKeyword)!)
        return cell!
    }
    
    func configureCellSearchFooter() -> JTMSearchFooterTableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchFooterTableViewCell") as? JTMSearchFooterTableViewCell
        cell!.setDisabilitySwitchState(getDisabilitySaveState())
        cell!.selectionStyle = UITableViewCellSelectionStyle.None
        cell!.delegate = self
        return cell!
    }
    func setCellSeperatorLineHide(cell:UITableViewCell) {
        cell.separatorInset = UIEdgeInsetsMake(0, CGRectGetWidth(tableView.bounds), 0, 0)
    }

    //MARK:- Search Footer Delegate 
    func footerOnClickSearch() {
    }
    
    func footerOnDisabilityStateChange(disabilitySwitch: UISwitch) {
        parentController?.searchFilterSaveState?.searchDisability = disabilitySwitch.on
        NSNotificationCenter.defaultCenter().postNotificationName(NOTI_RELOAD_DISABILITY_SWITCH, object: self.navigationController)
    }
    
    //MARK:- SearchController
    func initSearchController() {
        searchSuggestViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchSuggestViewController") as? JTMSearchSuggestViewController
        searchSuggestViewController!.delegate = self
        
        searchController = UISearchController(searchResultsController: searchSuggestViewController)
        searchController.dimsBackgroundDuringPresentation = true
        searchController.hidesNavigationBarDuringPresentation = false
        searchController.searchBar.delegate = self
        searchController.searchBar.frame = CGRectMake(0,0,self.view.frame.width,SEARCHBAR_HEIGHT)
        searchController.searchResultsUpdater = self
        searchController.searchBar.returnKeyType = .Done
        searchController.searchBar.enablesReturnKeyAutomatically = false
        searchController.delegate = self
        searchController.searchBar.accessibilityIdentifier = "searchFilterSeachBar"
        searchController.searchBar.accessibilityViewIsModal = true
        searchController.searchBar.translucent = false
        setSearchBarLanguage()
    }
    
    func setSearchBarLanguage() {
        searchController.searchBar.placeholder = applicationDelegate.getLocalizeStringForKey("Search")
        searchController.searchBar.setValue(applicationDelegate.getLocalizeStringForKey("Cancel"), forKey: "_cancelButtonText")
    }
    
    func textFieldShouldBeginEditing(textField: UITextField) -> Bool {
        searchController.searchBar.text = parentController?.searchFilterSaveState?.searchKeyword
        presentViewController(searchController, animated: true, completion: nil)
        searchSuggestViewController?.filterSuggest(searchController.searchBar.text!)
        return false
    }
    
    func updateSearchResultsForSearchController(searchController: UISearchController) {
        searchSuggestViewController?.filterSuggest(searchController.searchBar.text!)
    }
    
    func willDismissSearchController(searchController: UISearchController) {
    }
    
    func searchBarCancelButtonClicked(searchBar: UISearchBar) {
        print("Cancel")
    }
    
    func searchBarSearchButtonClicked(searchBar: UISearchBar) {
        self.searchController.dismissViewControllerAnimated(true, completion: nil)
        parentController?.searchFilterSaveState?.searchKeyword = searchController.searchBar.text!
        cellSearch?.setKeywordToEditText(searchController.searchBar.text!)
        NSNotificationCenter.defaultCenter().postNotificationName(NOTI_RELOAD_KEYWORD_CELL, object: self.navigationController)
    }
    
    //MARK:- Suggestion Delegate
    func didSelectKeyword(keyword: String) {
        self.searchController.dismissViewControllerAnimated(true, completion: nil)
        parentController?.searchFilterSaveState?.searchKeyword = keyword
        cellSearch?.setKeywordToEditText(keyword)
        NSNotificationCenter.defaultCenter().postNotificationName(NOTI_RELOAD_KEYWORD_CELL, object: self.navigationController)
    }
    
    //MARK:- Load Filter List From Service
    func loadRequiredFilterListFromService() {
        if ((applicationDelegate.appSetting?.isNetworkReachable) == false) {
            showAlertNoNetworkIsReachable()
            return
        }
        
//        if applicationDelegate.appSetting?.getSearchSuggestionListFromUserDefault().count == 0 {
//            loadListSuggestionFromService()
//        }
//        
//        if (applicationDelegate.appSetting?.searchFilterJobCategoryList.count == 0) {
//            loadJobCategoryListFromService()
//        }
        
        loadListSuggestionFromService()
        loadJobCategoryListFromService()
    }
    
    func loadJobCategoryListFromService() {
        let lastUpdate = (appSetting?.searchFilterJobCategoryLastUpdate)!
        let url = getServiveSearchFilter(SearchFilterCategoryType.CATEGORY)
        
        loadSearchFilterListFromService(url, lastupdate: lastUpdate) { (jsonObject) -> Void in
            self.appSetting?.searchFilterJobCategoryLastUpdate = (jsonObject.objectForKey("timestamps") as? Int)!
            let itemList = jsonObject.objectForKey("jobcategories") as? NSArray
            
            self.appSetting?.searchFilterJobCategoryList.removeAll()
            for item in itemList! {
                self.appSetting?.searchFilterJobCategoryList.append(SearchFilterCategory(item: item as! NSMutableDictionary))
            }
        }
    }
    
    func loadListSuggestionFromService() {
        var suggestionList = applicationDelegate.appSetting?.getSearchSuggestionListFromUserDefault()
        var lastUpdate = (applicationDelegate.appSetting?.getSearchSuggestionLastUpdateFromUserDefault())!
        let url = getServiceSearchSuggesttion()
        
        
        
        loadSearchFilterListFromService(url, lastupdate: lastUpdate) { (jsonObject) -> Void in
            lastUpdate = (jsonObject.objectForKey("timestamps") as? Int)!
            applicationDelegate.appSetting?.setSearchSuggestionLastUpdateToUserDefault(lastUpdate)
            
            suggestionList = (jsonObject.objectForKey("suggestionlist") as? [String])!
            applicationDelegate.appSetting?.setSearchSuggestionListToUserDefault(suggestionList!)
        }
    }
    
    func loadSearchFilterListFromService(url:String, lastupdate:Int, completionHandler:((jsonObject:NSMutableDictionary)->Void)) {
        let serviceController = JTMServiceController()
        let param = [
            "lastupdate" : lastupdate
        ]
        
        showLoading()
        serviceController.requestService(self, urlString: url, params: param, isCanLoadParallelToOther: true, isPresentLoadingIndicator: false) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                let sync = (jsonObject?.objectForKey("sync") as? Bool)!
                if(sync) {
                    completionHandler(jsonObject: jsonObject!)
                }
            }
            
            if self.isHasAllRequireFilterListValue() {
                self.hideLoading()
            }
        }
    }

    //MARK:- Make Disability Switch
    func getDisabilitySaveState() -> Bool{
        if let state = parentController?.searchFilterSaveState?.searchDisability {
            return state
        } else {
            return false
        }
    }
    
    //MARK:- Keyword From Save State
    func getKeywordJobCategoryFromSaveState() -> String {
        let selectedJobCategory = parentController?.searchFilterSaveState?.selectedJobCategory
        let selectedSubJobCategory = parentController?.searchFilterSaveState?.selectedSubJobCategory
        
        return getKeywordFromSaveState(selectedJobCategory, subFilter: selectedSubJobCategory, defaultKeyword: DEFAULT_KEYWORD_CATEGORY)
    }
    
    func getKeywordFromSaveState(filter: SearchFilterCategory?, subFilter: SearchFilterSubCategory?, defaultKeyword: String , transportationFilter : SearchFilterCategoryTransportation? = nil) -> String {
        if (filter != nil && subFilter != nil && transportationFilter != nil){
            if(transportationFilter!.hasSub!){
                return (transportationFilter?.categoryName)! + " - " + (filter?.name)! + " - " + (subFilter?.name)!
            } else {
                return (transportationFilter?.categoryName)! + " - " + (subFilter?.name)!
            }
            
        } else if (transportationFilter != nil && filter != nil) {
            return (transportationFilter?.categoryName)! + " - " + (filter?.name)!

        } else if (filter != nil && subFilter != nil) {
            return (filter?.name)! + " - " + (subFilter?.name)!
            
        } else if (filter != nil) {
            return (filter?.name)!
            
        } else {
            return defaultKeyword
        }
    }

    //MARK:- Parameter From Save State
    func getJobTypeParamaterFromSaveState() -> String {
        let saveFilter = parentController?.searchFilterSaveState
        return (saveFilter?.selectedJobCategory == nil) ? DEFAULT_KEYWORD_FOR_NIL_TYPE : (saveFilter?.selectedJobCategory!.code)!
    }
    
    func getSubJobTypeParamaterFromSaveState() -> String {
        let saveFilter = parentController?.searchFilterSaveState
        return (saveFilter?.selectedSubJobCategory == nil) ? DEFAULT_KEYWORD_FOR_NIL_TYPE : (saveFilter?.selectedSubJobCategory!.code)!
    }

    func getKeywordParameterFromSaveState() -> String {
        return (parentController?.searchFilterSaveState!.searchKeyword)!
    }

    //MARK: Delegate Update Save State : JobCategory
    func saveStateupdateSelectedJobCategory(saveData: AnyObject?) {
        if(saveData?.isKindOfClass(SearchFilterCategory) == true) {
            let selectedFilterJobCategory = saveData as? SearchFilterCategory
            parentController?.searchFilterSaveState?.selectedJobCategory = selectedFilterJobCategory
            
            if ((selectedFilterJobCategory?.hasSub == false)) {
                parentController?.searchFilterSaveState?.selectedSubJobCategory = nil
            }
        } else if(saveData?.isKindOfClass(SearchFilterSubCategory) == true) {
            let selectedFilterSubCategory = saveData as? SearchFilterSubCategory
            parentController?.searchFilterSaveState?.selectedSubJobCategory = selectedFilterSubCategory
        }
        
        parentController?.isUpdateJobCategory = true
        tableView?.reloadRowsAtIndexPaths([NSIndexPath(forRow: JOBCATE_CELL_INDEX, inSection: 0)], withRowAnimation: .None)
        NSNotificationCenter.defaultCenter().postNotificationName(NOTI_RELOAD_JOBCATEGORY_CELL, object: self.navigationController)
    }
    
    func reloadJobCategoryCell(notification : NSNotification) {
        if isCurrentNavitationType(notification) {
            tableView?.reloadRowsAtIndexPaths([NSIndexPath(forRow: JOBCATE_CELL_INDEX, inSection: 0)], withRowAnimation: .None)
        }
    }
    
    func reloadKeywordCell(notification : NSNotification) {
        guard let keyword = parentController?.searchFilterSaveState?.searchKeyword else {
            return
        }
        
        if isCurrentNavitationType(notification) {
            cellSearch?.setKeywordToEditText(keyword)
        }
    }
    
    func reloadDisabilitySwitch(notification : NSNotification) {
        guard let disabilitySwitchState = parentController?.searchFilterSaveState?.searchDisability else {
            return
        }
        
        if isCurrentNavitationType(notification) {
            cellFooter?.setDisabilitySwitchState(disabilitySwitchState)
        }
    }
    
    func isCurrentNavitationType(notification: NSNotification) -> Bool {
        guard let receiveNavBar = notification.object else {
            return false
        }
        
        let navBar = receiveNavBar as? UINavigationController
        if navBar == parentController?.navigationController {
            return true
        }
        return false
    }
    
    //MARK:- Open Search Result By Param
    func openSearchResult(parameter:[String:AnyObject], isOpenMap : Bool , radiusInMeter : Double) {
        dump(parameter)
        let mainSearchResultStoryboard: UIStoryboard = UIStoryboard(name: "SearchResult",bundle: nil)
        let searchController = mainSearchResultStoryboard.instantiateViewControllerWithIdentifier("JTMMainSearchResultViewController") as? JTMMainSearchResultViewController
        searchController?.params = parameter
        searchController?.isOpenFromSearchMap = isOpenMap
        searchController?.mapRadius = radiusInMeter
        self.navigationController?.pushViewController(searchController!, animated: true)
    }
    
    //MARK:- Open Search Filter By...
    func openSearchFilter(type: SearchFilterCategoryType) {
        let searchFilterCategory : JTMSearchFilterCategoryViewController = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchFilterCategoryViewController") as? JTMSearchFilterCategoryViewController)!
        
        searchFilterCategory.parent = self
        searchFilterCategory.setFilterCategoryType(type)
        searchFilterCategory.saveDelegate = self
        
        let saveFilter = parentController?.searchFilterSaveState
        var isBlankFilter : Bool = true
        
        switch (type) {
        case .LOCATION:
            searchFilterCategory.setTitleName(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_LOCATION"))
            searchFilterCategory.setSearchFilterModel((appSetting?.searchFilterLocationList)!)
            searchFilterCategory.setSelectedFilter(saveFilter?.selectedJobLocation)
            searchFilterCategory.setSelectedSubFilter(saveFilter?.selectedSubJobLocation)

            isBlankFilter = appSetting?.searchFilterLocationList.count == 0
            break
        case .TRANSPORTATION:
           let searchFilterCategoryTransportation = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchFilterCategoryTransportationViewController") as? JTMSearchFilterCategoryTransportationViewController)!
        searchFilterCategoryTransportation.setTitleName(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_TRANSPORTATION"))
           
           searchFilterCategoryTransportation.setSearchFilterModel((appSetting?.searchFilterTransportationList)!)
           
           searchFilterCategoryTransportation.saveFilter = saveFilter!
           searchFilterCategoryTransportation.setSelectFilterTransportModel(saveFilter?.selectedMainTransportation)
           searchFilterCategoryTransportation.setSelectedFilter(saveFilter?.selectedTransportation)
           searchFilterCategoryTransportation.setSelectedSubFilter(saveFilter?.selectedSubTransportation)
           
           searchFilterCategoryTransportation.setFilterCategoryType(type)
           searchFilterCategoryTransportation.parent = self
           searchFilterCategoryTransportation.saveDelegate = self
           if(saveFilter?.selectedMainTransportation == nil) {
            searchFilterCategoryTransportation.indexSelector = -1
           }
           
           if (appSetting?.searchFilterTransportationList.count > 0 && (appSetting?.isNetworkReachable)!) {
            self.navigationController?.pushViewController(searchFilterCategoryTransportation, animated: true)
           }
            break
        case .INDUSTRIALESTATE:
            searchFilterCategory.setTitleName(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_INDUSTRIAL_ESTATES"))
            searchFilterCategory.setSearchFilterModel((appSetting?.searchFilterIndustrialList)!)
            searchFilterCategory.setSelectedFilter(saveFilter?.selectedIndustrial)
            searchFilterCategory.setSelectedSubFilter(saveFilter?.selectedSubIndustrial)
            
            isBlankFilter = appSetting?.searchFilterIndustrialList.count == 0
            break
        default: //.Category
            searchFilterCategory.setTitleName(applicationDelegate.getLocalizeStringForKey("TITLE_JOB_CATEGORY"))
            searchFilterCategory.setSearchFilterModel((appSetting?.searchFilterJobCategoryList)!)
            searchFilterCategory.setSelectedFilter(saveFilter?.selectedJobCategory)
            searchFilterCategory.setSelectedSubFilter(saveFilter?.selectedSubJobCategory)
            searchFilterCategory.saveDelegate = self
            
            isBlankFilter = appSetting?.searchFilterJobCategoryList.count == 0
            break
        }
        
        if(type != .TRANSPORTATION && !isBlankFilter && (appSetting?.isNetworkReachable)!){
            self.navigationController?.pushViewController(searchFilterCategory, animated: true)
        }
    }
}

@objc protocol JTMUpdateSaveStateDelegate {
    optional func saveStateupdateSelectedJobCategory(saveData : AnyObject?)
    optional func saveStateupdateSelectedJobLocation(saveData : AnyObject?)
    optional func saveStateupdateSelectedTransportation(saveData : AnyObject?)
    optional func saveStateupdateSelectedIndustrialEstate(saveData : AnyObject?)
}
