//
//  JTMBrowseJobFirstLevelViewController.swift
//  JobThai
//
//  Created by Isaret Selachun on 20/1/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
//import AlamofireImage
import Alamofire

class JTMBrowseJobFirstLevelViewController: JTMBaseViewController , UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout {
    
    @IBOutlet var collectionView:UICollectionView?
    
    var firstLevelDataArr: [FirstLevelModel]?
    var listProvince: [SecoundLevelModel]?
    
    var jobType : ModeJobType = .NONE
    var currentIndex : Int? = -1
    var currentSection : Int? = -1
    
    // Constant
    let HEIGHT_CELL : Double = 140
    let WIDTH_SPACE : Double = 8
    let NUM_OF_CELL : Double = 4
    let NUM_OF_SPACE : Double = 3
    let HEIGHT_HEADER : CGFloat = 40
    let SECTION_NOT_LOCATION : Int = 1
    let SECTION_LOCATION : Int = 2
    let PADDING_LEFT_AND_RIGHT : Double = 30
    
    override func viewDidLoad() {
        super.viewDidLoad()
        //connectService()
        self.setUpCollectionView()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        setTrackGoogleAnalyticScreen()
        
        clearActive()
    }
    
    func clearActive() {
        let selectedItem = self.collectionView?.indexPathsForSelectedItems()
        if(selectedItem?.count > 0) {
            for indexPath in (selectedItem)! {
                self.deSelectItem(indexPath)
            }
        }
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- On Retry Connection
    override func onRetryConnection() {
        super.onRetryConnection()
        if (firstLevelDataArr == nil || firstLevelDataArr?.count == 0) {
            connectService()
        }
    }
    
    //MARK:- Manage Conenction Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (firstLevelDataArr == nil || firstLevelDataArr?.count == 0) {
            setAlphaShowToView(view)
        }
    }
    
    // Track Google Analytic
    func setTrackGoogleAnalyticScreen() {
        switch(jobType) {
        case .BUSINESS:
            print("Industry")
            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_INDUSTRY)
            break
        case .CATEGORY:
            print("Category")
            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_CATEGORY)
            break
        case .LOCATION:
            print("Province")
            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_PROVINCE)
            break
        case .INDUSTRIAL:
            print("Industrial")
            applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_INDUSTRIAL_ESTATE)
            break
        default:
            break
        }
    }
    
    // Setup Layout and Regist Xib Cell
    func setUpCollectionView(){
        collectionView!.registerNib(UINib(nibName: "JTMFirstLevelCollectionViewCell", bundle: nil), forCellWithReuseIdentifier: "JTMFirstLevelCollectionViewCell")
        collectionView?.registerNib(UINib(nibName: "JTMFirstLevelLocationHeaderViewCell", bundle: nil), forSupplementaryViewOfKind: UICollectionElementKindSectionHeader, withReuseIdentifier: "JTMFirstLevelLocationHeaderViewCell")
        collectionView?.accessibilityIdentifier = ID_BROWSE_JOBS_FIRST_LEVEL_COLLECTION_VIEW
    }
    
    // Sort Data by BlockOrder
    func sortDataByBlockOrder(){
        firstLevelDataArr!.sortInPlace({ $1.blockOrder > $0.blockOrder })
    }
    
    // MARK : Connect Service
    func connectService(){
        
        let serviceController = JTMServiceController()
        let url = getServiceFirstLevel(jobType)
        
        serviceController.requestService(self, urlString: url, params: nil) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.bindJSONToUI(jsonObject!)
            }
        }
    }
    
    // Set Type Mode and ReloadService
    func setModeTypeAndReloadService(mode : ModeJobType, loadService:Bool=true){
        jobType = mode
        if(loadService){
            self.firstLevelDataArr?.removeAll()
            self.collectionView?.reloadData()
            self.performSelector(#selector(connectService), withObject: nil, afterDelay: 0.1)
        }
    }
    
    // Bind Json to UI
    func bindJSONToUI(jsonObject:NSMutableDictionary){
        firstLevelDataArr = [FirstLevelModel]()
        let result = jsonObject.objectForKey("firstLevelList") as! NSArray
        for itemArray in result  {
            let firstItem = FirstLevelModel(item: (itemArray as? NSMutableDictionary)!)
            firstLevelDataArr?.append(firstItem)
        }
        self.sortDataByBlockOrder()
        
        if(jsonObject.objectForKey("provinceList") as? NSArray != nil) {
            self.listProvince = [SecoundLevelModel]()
            let provinceList = jsonObject.objectForKey("provinceList") as! NSArray
            for itemProvince in provinceList {
                let secondLevelObject = SecoundLevelModel(item: itemProvince as! NSMutableDictionary)
                self.listProvince?.append(secondLevelObject)
            }
        }
        
        self.collectionView?.reloadData()
    }
    
    // MARK: UICollectionview DataSource
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return jobType != .LOCATION ? SECTION_NOT_LOCATION : SECTION_LOCATION
    }
    
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if section == 0 {
            return (firstLevelDataArr != nil) ? firstLevelDataArr!.count : 0
            
        } else {
            return (listProvince != nil) ? listProvince!.count : 0
        }
    }

    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        
//        print("currentIndex :: \(currentIndex)")
        print("select indexPath.row :: \(indexPath.row)")
        print("select section :: \(indexPath.section)")
        print("currentIndex :: \(currentIndex)")
//        test()
        
        // Track Google Analytic Event
        if indexPath.section == 0 { 
            applicationDelegate.trackGoogleAnalyticEvent(CAT_BROWSE_JOB_LEVEL_1, action: ACTION_CLICK, label: getLabelNameBrowseJobLevel1(self.title!,browseJobLevel1:replaceNewLineString(firstLevelDataArr![indexPath.row].name!)))
        } else {
            applicationDelegate.trackGoogleAnalyticEvent(CAT_BROWSE_JOB_LEVEL_1, action: ACTION_CLICK, label: getLabelNameBrowseJobLevel1(self.title!,browseJobLevel1:replaceNewLineString(listProvince![indexPath.row].name!)))
        }
        
        
        let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMFirstLevelCollectionViewCell
        if(currentIndex != indexPath.row || currentSection != indexPath.section) {
            cell?.setHighlightCell(true)
            
            //Disable Highlight Old Cell
            if(currentIndex != -1) {
//                let oldCell = collectionView.cellForItemAtIndexPath(NSIndexPath(forRow: currentIndex!, inSection:indexPath.section)) as? JTMFirstLevelCollectionViewCell
                let oldCell = collectionView.cellForItemAtIndexPath(NSIndexPath(forRow: currentIndex!, inSection:currentSection!)) as? JTMFirstLevelCollectionViewCell
                if(cell != nil){
                    oldCell?.setHighlightCell(false)
                }
            }
            currentIndex = indexPath.row
            currentSection = indexPath.section
        }
        
        let browseJobSecondLevelViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMBrowseJobSecondLevelViewController") as! JTMBrowseJobSecondLevelViewController
        
        var firstLevel = FirstLevelModel()
        var secoundLevelMode = [SecoundLevelModel]()
        if indexPath.section == 0 {
            browseJobSecondLevelViewController.title = replaceNewLineString(firstLevelDataArr![indexPath.row].name!)
            secoundLevelMode = [SecoundLevelModel]()
            firstLevel = firstLevelDataArr![indexPath.row]
            secoundLevelMode = firstLevelDataArr![indexPath.row].listSecoundLevelMode!
        }
        
        var param: [String:AnyObject] = [:]
        if(!secoundLevelMode.isEmpty && indexPath.section == 0) {
            browseJobSecondLevelViewController.setModelBrowseJobSecondLevel(secoundLevelMode)
            browseJobSecondLevelViewController.setModeJobtype(firstLevel, modeType: jobType)
            self.navigationController?.pushViewController(browseJobSecondLevelViewController, animated: true)
        } else {
            switch (jobType) {
                case .BUSINESS :
                    param = [
                        PARAMETER_JOB_TYPE_KEY      : DEFAULT_PARAMETER,
                        PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER,
                        PARAMETER_BUSINESS_KEY      : firstLevel.searchCode!
                    ]
                        
                        //PARAMETER_DEFAULT + PARAMETER_BUSINESS + firstLevel.searchCode!
                    break
                case .CATEGORY :
                    param = [
                        PARAMETER_JOB_TYPE_KEY      : firstLevel.searchCode!,
                        PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER
                    ]
                        
                        //PARAMETER_JOB_TYPE +  firstLevel.searchCode! + PARAMETER_SUB_JOB_TYPE + "All"
                    break
                case .LOCATION :
                    param = [
                        PARAMETER_JOB_TYPE_KEY      : DEFAULT_PARAMETER,
                        PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER,
                        PARAMETER_LOCATION_KEY      : indexPath.section == 1 ? listProvince![indexPath.row].code! : firstLevel.searchCode!
                    ]
                        
                        //PARAMETER_DEFAULT + PARAMETER_LOCATION + firstLevel.searchCode!
                    break
                case .INDUSTRIAL :
                    param = [
                        PARAMETER_JOB_TYPE_KEY      : DEFAULT_PARAMETER,
                        PARAMETER_SUB_JOB_TYPE_KEY  : DEFAULT_PARAMETER,
                        PARAMETER_INDUSTRIAL_KEY    : firstLevel.searchCode!
                    ]
                        
                        //PARAMETER_DEFAULT + PARAMETER_INDUSTRIAL + firstLevel.searchCode!
                    break
                default :
                    break
            }
            
            JTMBrowseJobViewController.openBrowseJobViewController(self.storyboard!, navigationController: self.navigationController!, type: typeBrowseJob.SEARCH_RESULT, code: firstLevel.code, searchParams: param)
        }
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        let smallCell : JTMFirstLevelCollectionViewCell = collectionView.dequeueReusableCellWithReuseIdentifier("JTMFirstLevelCollectionViewCell", forIndexPath: indexPath) as! JTMFirstLevelCollectionViewCell
        
        if indexPath.section == 0 {
            smallCell.setCellData(firstLevelDataArr![indexPath.row])
        } else {
            smallCell.setCellData(listProvince![indexPath.row], isLocation: true)
        }

        if(currentIndex == indexPath.row){
            smallCell.setHighlightCell(true)
        } else {
            smallCell.setHighlightCell(false)
        }
        
        return smallCell
    }
    
    func collectionView(collectionView: UICollectionView, layout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
        var widthCell : Double = 0
        widthCell = (Double((self.collectionView?.frame.size.width)!) - (WIDTH_SPACE * NUM_OF_SPACE) - PADDING_LEFT_AND_RIGHT)/NUM_OF_CELL
        return CGSize(width: widthCell, height: HEIGHT_CELL)
    }
    
    func collectionView(collectionView: UICollectionView, didHighlightItemAtIndexPath indexPath: NSIndexPath) {
        let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMFirstLevelCollectionViewCell
        cell?.setHighlightCell(true)
    }
    
    func collectionView(collectionView: UICollectionView, didUnhighlightItemAtIndexPath indexPath: NSIndexPath) {
//        if(currentIndex != indexPath.row){
            let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMFirstLevelCollectionViewCell
            cell?.setHighlightCell(false)
//        }
    }
    
    // Delegate set layout to show
    func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        if kind == UICollectionElementKindSectionHeader {
            if (indexPath.section == 1) {
                let cellHeader = collectionView.dequeueReusableSupplementaryViewOfKind(kind, withReuseIdentifier: "JTMFirstLevelLocationHeaderViewCell", forIndexPath: indexPath)
                
                return cellHeader
            }
        }
        
        return UICollectionReusableView(frame: CGRect(x: 0, y: 0, width: 0, height: 0))
    }
    
    // Delegate for set height header section
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, referenceSizeForHeaderInSection section: Int) -> CGSize {
        if section == 1 {
            return CGSizeMake(self.collectionView!.frame.width, HEIGHT_HEADER)
        }
        return CGSizeMake(0, 0)
    }
    
    func deSelectItem(indexPath: NSIndexPath) {
        let cell = collectionView!.cellForItemAtIndexPath(indexPath) as? JTMFirstLevelCollectionViewCell
        cell?.setHighlightCell(false)
        currentIndex = -1
    }
    
    func test(){
        let selectList = self.collectionView?.indexPathsForSelectedItems()
        print("selectList :: \(selectList)")
    }
    
}


