//
//  JTMBusRouteViewController.swift
//  JobThai
//
//  Created by weerapon on 1/21/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMBusRouteViewController : JTMBaseViewController, UITableViewDelegate, UITableViewDataSource, UISearchBarDelegate, UISearchControllerDelegate , UISearchResultsUpdating{
    
    let SECTION_HEADER_HEIGHT:CGFloat = 42
    
    var busRouteList = [BusRouteModel]()
    var busAllList = [BusModel]()
    
    var filteredArray = [BusModel]()
    var filteredResultBySection = [BusRouteModel]()
    
    var busIndexList = [String]()
    var shouldShowSearchResults = false
    var searchController = UISearchController(searchResultsController: nil)
    
    @IBOutlet var busTableView : UITableView?
    @IBOutlet var searchMessage : UILabel?
    @IBOutlet var subViewSearch : UIView?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.callService()
        initSearchController()
        self.busTableView?.tableFooterView = UIView(frame: CGRectZero)
        self.busTableView?.sectionIndexBackgroundColor = UIColor.clearColor()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("BusRoute viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_BROWSE_JOB_BUS_ROUTE)
        
        self.searchMessage?.text = applicationDelegate.getLocalizeStringForKey("SEARCH_NO_RESULT_MESSAGE")
        self.searchMessage?.hidden = true
        setNoInternetViewToSearchBar()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        self.searchController.dismissViewControllerAnimated(false, completion: nil)
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- Reachability
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        updateNoInternetViewInSearchBarWhenReachabilityChange()
    }
    
    override func manageViewAfterInternetNotReachable() {
        super.manageViewAfterInternetNotReachable()
        updateNoInternetViewInSearchBarWhenReachabilityChange()
    }
    
    //MARK:- On Retry Connectrion
    override func onRetryConnection() {
        super.onRetryConnection()
        if (busRouteList.count == 0) {
            callService()
        }
    }
    
    //MARK:- Manage View On Connection Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (busRouteList.count == 0) {
            setAlphaShowToView(view)
        }
    }
    
    //MARK:- SearchBar
    func setNoInternetViewToSearchBar() {
        if (!isPresentedBefore) {
            searchController.searchBar.addSubview(noInternetTabView!)
        }
    }
    
    func updateNoInternetViewInSearchBarWhenReachabilityChange() {
        if (noInternetTabView != nil) {
            searchController.searchBar.bringSubviewToFront(noInternetTabView!)
        }
    }
    
    //MARK:-
    func initSearchController() {
        self.searchController.dimsBackgroundDuringPresentation = false
        self.searchController.hidesNavigationBarDuringPresentation = false
        self.searchController.searchBar.delegate = self
        self.searchController.searchBar.frame = CGRectMake(0,0,self.view.frame.width,DEFAULT_SEARCH_BAR_HEIGHT)
        self.searchController.searchResultsUpdater = self
        self.setSearchBarLanguage()
        self.subViewSearch?.addSubview(self.searchController.searchBar)
    }
    
    func setSearchBarLanguage() {
        self.searchController.searchBar.placeholder = applicationDelegate.getLocalizeStringForKey("Search")
        self.searchController.searchBar.setValue(applicationDelegate.getLocalizeStringForKey("Cancel"), forKey: "_cancelButtonText")
    }
    
    func callService() {
        
        let serviceController = JTMServiceController()
        let url = getServiceBusRoute()
        
        serviceController.requestService(self, urlString: url, params: nil) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                self.convertJsonToModel(jsonObject!)
                self.busTableView?.reloadData()
            }
        }
    }
    
    func convertJsonToModel(jsonObject : NSMutableDictionary) {
        let busRouteObject = jsonObject.objectForKey("busRoute") as? NSMutableDictionary
        for (key,value) in busRouteObject!{
            let itemBus = BusRouteModel(key: key as! String, listBus: value as! NSArray)
            busRouteList.append(itemBus)
            
            //add bus all List
            busAllList = busAllList + itemBus.busList!
        }
        busIndexList = (jsonObject.objectForKey("indexedList") as? [String])!
        
        // sort
        busRouteList.sortInPlace({Int($0.busKey!) < Int($1.busKey!)})
    }
    
    /*
    * ReMark : Delagate Search Display
    */
    func filterContentForSearchText(searchText: String) {
        var resultCount = 0

        //Filter Data by Section
        filteredResultBySection = [BusRouteModel]()
        
        for busRouteItem : BusRouteModel in busRouteList{
            var busList = [BusModel]()
            var searchBusListResult = [BusModel]()
            let tmpResultBusRoute = BusRouteModel()
            
            busList = busRouteItem.busList!
            searchBusListResult = busList.filter({ (busObject : BusModel) -> Bool in
                let stringMatch = busObject.name?.lowercaseString.containsString(searchText.lowercaseString)
                return stringMatch!
            })
            resultCount += searchBusListResult.count
            searchBusListResult.sortInPlace({Int($0.code!) < Int($1.code!) })
            tmpResultBusRoute.setKeyAndBusList(busRouteItem.busKey!, listBus: searchBusListResult )
            
            filteredResultBySection.append(tmpResultBusRoute)
        }
        
        self.busTableView?.hidden = false
        if(resultCount == 0 && searchText != ""){
            self.searchMessage?.hidden = false
            self.busTableView?.hidden = true
        } else {
            self.searchMessage?.hidden = true
        }
        
        self.busTableView?.reloadData()
    }
    
    func updateSearchResultsForSearchController(searchController: UISearchController) {
        self.filterContentForSearchText(searchController.searchBar.text!)
    }
    
    func searchBarCancelButtonClicked(searchBar: UISearchBar) {
        self.searchController.searchBar.text = ""
        self.busTableView?.reloadData()
    }
    /*
    * ReMark : Delagate TableView and Section
    */
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        if(self.searchController.searchBar.text != ""){
            return self.filteredResultBySection.count
        } else {
            return busRouteList.count
        }
    }
    
    /*
    * delegate จำนวนของ Row ใน Section
    */
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if(self.searchController.searchBar.text != ""){
            return (self.filteredResultBySection[section].busList)!.count
        } else {
            return busRouteList[section].busList!.count
        }
    }
    
    /*
    * delegate Header ที่เป็น View
    */
    func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let nib = UINib(nibName: "JTMBusRouteHeaderViewCell", bundle: nil)
        let headerView = nib.instantiateWithOwner(self, options: nil)[0] as? JTMBusRouteHeaderViewCell
        
        if(self.searchController.searchBar.text != ""){
            let busSection = self.filteredResultBySection[section]
            headerView?.setBusRouteTableHeaderView(busSection.busKey!, imageName: "ic_browsejob_busroute")
            return headerView
        } else {
            headerView?.setBusRouteTableHeaderView(busRouteList[section].busKey!, imageName: "ic_browsejob_busroute")
            return headerView
        }
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        let searchResultController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainSearchResultViewController") as? JTMMainSearchResultViewController
        if(self.searchController.searchBar.text != ""){
            let bus = self.filteredResultBySection[indexPath.section].busList![indexPath.row]
            searchResultController?.params = [
                PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                PARAMETER_MAIN_TRANSPORTATION_KEY   : DEFAULT_PARAMETER_BUS_ROUTE_MAIN_TYPE,
                PARAMETER_BUS_ROUTE_KEY             : bus.code!,
                PARAMETER_RADIUS_KEY                : DEFAULT_PARAMETER_RADIUS
            ]
                
                //PARAMETER_DEFAULT + PARAMETER_BUS_MAIN_TYPE + PARAMETER_BUS_ROUTE + bus.code! + PARAMETER_RADIUS_DEFAULT

        } else {
            let bus = busRouteList[indexPath.section].busList![indexPath.row]
            searchResultController?.params = [
                PARAMETER_JOB_TYPE_KEY              : DEFAULT_PARAMETER,
                PARAMETER_SUB_JOB_TYPE_KEY          : DEFAULT_PARAMETER,
                PARAMETER_MAIN_TRANSPORTATION_KEY   : DEFAULT_PARAMETER_BUS_ROUTE_MAIN_TYPE,
                PARAMETER_BUS_ROUTE_KEY             : bus.code!,
                PARAMETER_RADIUS_KEY                : DEFAULT_PARAMETER_RADIUS
            ]
                
                //PARAMETER_DEFAULT + PARAMETER_BUS_MAIN_TYPE + PARAMETER_BUS_ROUTE + bus.code! + PARAMETER_RADIUS_DEFAULT
        }
        searchResultController?.isOpenFromSearchMap = true
        self.navigationController?.pushViewController(searchResultController!, animated: true)
        
    }
    
    func sectionIndexTitlesForTableView(tableView: UITableView) -> [String]? {
        if(self.searchController.searchBar.text != ""){
            return nil
        }
        else{
            return busIndexList
        }
        
    }
    
    /*
    * delegate ข้อมูล item ของ Cell
    */
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell : UITableViewCell = tableView.dequeueReusableCellWithIdentifier("JTMBusRouteViewCell", forIndexPath: indexPath)
        // Initialization code set Separator
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
        
        if(self.searchController.searchBar.text != ""){
            let bus = self.filteredResultBySection[indexPath.section].busList![indexPath.row]
            cell.textLabel?.text = bus.name
            
            
        } else {
            let bus = busRouteList[indexPath.section].busList![indexPath.row]
            cell.textLabel?.text = bus.name!
        }
        
        cell.selectionStyle = .None
        
        return cell
    }
    
    func tableView(tableView: UITableView, didHighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = JTM_SELECTED_CELL_BORDER_COLOR.colorWithAlphaComponent(0.4)
    }
    
    func tableView(tableView: UITableView, didUnhighlightRowAtIndexPath indexPath: NSIndexPath) {
        let cell = tableView.cellForRowAtIndexPath(indexPath)
        cell?.backgroundColor = UIColor.clearColor()
    }
    
    /* 
    * delegate ความสูงของ section 
    */
    func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if(self.searchController.searchBar.text != ""){
            if( (self.filteredResultBySection[section].busList)!.count > 0){
                return SECTION_HEADER_HEIGHT
            }
            else{
                return 0
            }
        } else {
            return SECTION_HEADER_HEIGHT
        }
        
    }
    
    func scrollViewWillBeginDragging(scrollView: UIScrollView) {
        self.searchController.searchBar.resignFirstResponder()
    }
}
