//
//  JTMButtonWithImageAboveDescriptionView.swift
//  JobThai
//
//  Created by kanya on 4/7/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMButtonWithImageAboveDescriptionView: UIView {

    @IBOutlet weak var iconImageView: UIImageView!
    @IBOutlet weak var titleLabel: UILabel!
    
    var delegate: JTMButtonWithImageAboveDescriptionDelegate?
    
    var selectedColor = JTM_SELECTED_CELL_BORDER_COLOR
    var normalColor = JTM_FOOTER_BG_COLOR //JTM_BROWSEJOB_HOME_BG_COLOR.colorWithAlphaComponent(0.2)
    
    //MARK:- Initial
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupUi()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupUi()
    }
    
    //MARK:- Set UI From Nib
    func setupUi(){
        let view = loadViewFromNib()
        view.frame = bounds
        view.autoresizingMask = [.FlexibleWidth , .FlexibleHeight]
        addSubview(view)
        
        addGesture()
        
        self.backgroundColor = normalColor
        self.titleLabel.textColor = JTM_CORE_ORANGE
    }
    
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMButtonWithImageAboveDescriptionView", bundle: bundle)
        let view = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    //MARK:- Set UI With Data
    func setButtonUIWithData(iconImage: UIImage, title: String) {
        self.iconImageView.image = iconImage
        self.titleLabel.text = title
    }
    
    //MARK:- Tap Gesture
    func addGesture() {
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(handleTap(_:)))
        tapGesture.numberOfTapsRequired = 1
        self.addGestureRecognizer(tapGesture)
    }
    
    func handleTap(tap:UITapGestureRecognizer) {
        print("handleTap")
        self.onClickButton()
        self.onTouchesBegan()
        
        if tap.state == .Began {
            print("Began")
        } else if tap.state == .Cancelled {
            print("Cancelled")
        } else if tap.state == .Changed {
            print("Changed")
        } else if tap.state == .Failed {
            print("Failed")
        } else if tap.state == .Possible {
            print("Possible")
        }  else if tap.state == .Ended {
            print("Ended")
            self.onTouchesCancelled()
        }
    }
    
    func onClickButton() {
        self.delegate?.onClickButton(self)
    }
    
    //MARK:- Touch over & Focus
    
    func onTouchesBegan() {
        print("onTouchesBegan")
        self.backgroundColor = selectedColor
        self.delegate?.onTouchesBegan(self)
    }
    
    func onTouchesCancelled() {
        print("onTouchesCancelled")
        UIView.animateWithDuration(0.1) { () -> Void in
            self.backgroundColor = self.normalColor
            self.delegate?.onTouchesCancelled(self)
        }
    }
    
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        print("touchesBegan")
        self.onTouchesBegan()
    }
    
    override func touchesCancelled(touches: Set<UITouch>?, withEvent event: UIEvent?) {
        print("touchesCancelled")
        self.onTouchesCancelled()
    }
    
    override func touchesEnded(touches: Set<UITouch>, withEvent event: UIEvent?) {
        print("touchesEnded")
        self.onTouchesCancelled()
    }
}

protocol JTMButtonWithImageAboveDescriptionDelegate {
    func onClickButton(button:JTMButtonWithImageAboveDescriptionView)
    func onTouchesBegan(button:JTMButtonWithImageAboveDescriptionView)
    func onTouchesCancelled(button:JTMButtonWithImageAboveDescriptionView)
}
