//
//  JTMChangeEmailViewController.swift
//  JobThai
//
//  Created by weerapon on 2/8/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import IQKeyboardManagerSwift

class JTMChangeEmailViewController: JTMBaseViewController, JTMChangeEmailDelegate {
    
    @IBOutlet weak var changeEmailView : JTMChangeEmailView?
    
    let FOCUS_MODE = ".FOCUS"
    let LOGOUT_MODE = ".LOGOUT"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = applicationDelegate.getLocalizeStringForKey("CHANGE_EMAIL_TITLE")
        changeEmailView?.delegate = self
        
        // Set distance between keyboard and textField
        IQKeyboardManager.sharedManager().keyboardDistanceFromTextField = 20
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_CHANGE_EMAIL)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CHANGE_EMAIL, action: ACTION_VIEW, label: LABEL_VIEW_CHANGE_EMAIL)
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
        isCanUseAfterLogInOnly = true
    }
    
    func onSaveEmail() {
        if (applicationDelegate.appSetting?.isNetworkReachable)! == false {
            return
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CHANGE_EMAIL, action: ACTION_CLICK, label: LABEL_CHANGE_EMAIL)
        
        checkDataInTextField()
    }
    
    func checkDataInTextField() -> Bool {
        var checkError = false
        var errorTitle = ""
        var messageError = ""
        var labelKey : UITextField?
        
        if (self.changeEmailView?.newEmailTextField?.text == "") {
            errorTitle = applicationDelegate.getLocalizeStringForKey("ERROR_NEW_EMAIL")
            checkError = true
            labelKey = (changeEmailView?.newEmailTextField)!
            
        } else if (self.changeEmailView?.confirmEmailTextField?.text == "") {
            errorTitle = applicationDelegate.getLocalizeStringForKey("ERROR_CONFIRM_NEW_EMAIL")
            checkError = true
            labelKey = (changeEmailView?.confirmEmailTextField)!
            
        } else if(self.changeEmailView?.passwordTextField?.text == "") {
            errorTitle = applicationDelegate.getLocalizeStringForKey("ERROR_CURRENT_PASS")
            checkError = true
            labelKey = (changeEmailView?.passwordTextField)!
            
        } else if !(self.changeEmailView?.newEmailTextField?.isValidEmailFormat())! {
            errorTitle = applicationDelegate.getLocalizeStringForKey("ERROR_EMAIL_FORMAT")
            messageError = applicationDelegate.getLocalizeStringForKey("MESSAGE_EMAIL_FORMAT")
            checkError = true
            labelKey = (changeEmailView?.newEmailTextField)!
            
        } else if !isEmailMatch() {
            errorTitle = applicationDelegate.getLocalizeStringForKey("ERROR_EMAIL_NOT_MATCH")
            checkError = true
            labelKey = (changeEmailView?.confirmEmailTextField)!
            
        }
        
        if checkError {
            self.showAlertMessage(errorTitle, alertMessage: messageError, labelKey: labelKey!, type: self.FOCUS_MODE)
            
        } else {
            checkServiceValidEmail()
        }
        
        return checkError
    }
    
    func isEmailMatch() -> Bool {

        let newEmail = self.changeEmailView?.newEmailTextField?.text
        let confirmEmail = self.changeEmailView?.confirmEmailTextField?.text
        
        if newEmail == confirmEmail {
            return true
        }
        
        return false
    }
    
    func checkServiceValidEmail() {
        
        let newEmail = self.changeEmailView?.newEmailTextField?.text
        let confirmEmail = self.changeEmailView?.confirmEmailTextField?.text
        let password = self.changeEmailView?.passwordTextField?.text
        let token = applicationDelegate.appSetting?.userData?.token
        
        var labelKey = (self.changeEmailView?.newEmailTextField)!
        var errCode = ""
        
        let serviceController = JTMServiceController()
        let url = getServiceChangeEmail()
        let param = [
            "newEmail" : newEmail!,
            "confirmEmail" : confirmEmail!,
            "password" : password!,
            "token" : token!,
            "imei" : applicationDelegate.getDeviceUUID()
        ]
        
        serviceController.requestService(self, urlString: url, params: param) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonResult = response!.result.value
                let status = jsonResult!["status"] as! String
                
                if status == "error" {
                    errCode = jsonResult!["errorcode"] as! String
                    
                    // CASE :: Current password is not correct
                    if errCode == INCORRECT_PASSWORD {
                        labelKey = (self.changeEmailView?.passwordTextField)!
                    }
                    
                    let titleMessage = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let errorMessage = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    self.showAlertMessage(titleMessage, alertMessage: errorMessage, labelKey: labelKey, type: self.FOCUS_MODE)
                    
                } else if status == "ok"{
                    // SUCCESS
                    let titleMessage = jsonResult!["titlemsg"] as! String
                    let alertMessage = jsonResult!["errormsg"] as! String
                    self.showAlertMessage(titleMessage, alertMessage: alertMessage, labelKey: labelKey, type: self.LOGOUT_MODE)
                }
            }
        }
    }
    
    func showAlertMessage(titleMessage : String, alertMessage : String, labelKey : UITextField, type : String) {
        
        var message : String!
        message = (alertMessage == "") ? nil : alertMessage
        
        self.showAlertWithData(
            titleMessage,
            message: message,
            completionHandler:  { (UIAlertAction) -> Void in
                switch(type) {
                case self.FOCUS_MODE :
                    labelKey.becomeFirstResponder()
                    break
                case self.LOGOUT_MODE :
                    JTMLoginViewController.connectServiceLogout(self, redirectType: .BACK_TO_HOME, completion: nil)
                    // CLEAR Saved User
                    applicationDelegate.appSetting?.userCredential?.removeSavedUser()
                    break
                default :
                    labelKey.becomeFirstResponder()
                    break
                }
            },
            actionList: nil
        )
        
        
    }
    
}
