//
//  JTMChangePasswordViewController.swift
//  JobThai
//
//  Created by weerapon on 2/8/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import IQKeyboardManagerSwift

class JTMChangePasswordViewController: JTMBaseViewController, JTMChangePasswordDelegate {
    
    @IBOutlet weak var changePasswordView : JTMChangePasswordView?
    
    let FOCUS_MODE = ".FOCUS"
    let LOGOUT_MODE = ".LOGOUT"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        changePasswordView?.delegate = self
        self.title = applicationDelegate.getLocalizeStringForKey("CHANGE_PASSWORD_TITLE")
        
        // Set distance between keyboard and textField
        IQKeyboardManager.sharedManager().keyboardDistanceFromTextField = 20
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_CHANGE_PASSWORD)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CHANGE_PASSWORD, action: ACTION_VIEW, label: LABEL_VIEW_CHANGE_PASSWORD)
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
        isCanUseAfterLogInOnly = true
    }
    
    func onSavePassword() {
        if (applicationDelegate.appSetting?.isNetworkReachable)! == false {
            return
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_CHANGE_PASSWORD, action: ACTION_CLICK, label: LABEL_CHANGE_PASSWORD)
        
        checkDataInTextField()
    }
    
    func checkDataInTextField() -> Bool {
        var checkError = false
        var messageError = ""
        var labelKey : UITextField?
        
        if (self.changePasswordView?.currentPasswordTextField?.text == "") {
            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_CURRENT_PASS")
            checkError = true
            labelKey = (changePasswordView?.currentPasswordTextField)!
            
        } else if (self.changePasswordView?.newPasswordTextField?.text == "") {
            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_NEW_PASS")
            checkError = true
            labelKey = (changePasswordView?.newPasswordTextField)!
            
        } else if(self.changePasswordView?.confirmPasswordTextField?.text == "") {
            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_CONFIRM_NEW_PASS")
            checkError = true
            labelKey = (changePasswordView?.confirmPasswordTextField)!
            
        } else if !isPassMatch() {
            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_NOT_MATCH")
            checkError = true
            labelKey = (changePasswordView?.confirmPasswordTextField)!
            
        }
//        else if !isValidPassword() {
//            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_FORMAT")
//            checkError = true
//            labelKey = (changePasswordView?.newPasswordTextField)!
//            
//        }
        
        // Check these below condition in Service
        //else if self.changePasswordView?.newPasswordTextField?.text?.characters.count < 6 {
//            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_LESS_CHAR")
//            checkError = true
//            labelKey = (changePasswordView?.newPasswordTextField)!
//            
//        } else if self.changePasswordView?.newPasswordTextField?.text?.characters.count > 15 {
//            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_MORE_CHAR")
//            checkError = true
//            labelKey = (changePasswordView?.newPasswordTextField)!
//            
//        } else if isRepeatNum() {
//            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_REPEAT_NUM")
//            checkError = true
//            labelKey = (changePasswordView?.newPasswordTextField)!
//            
//        } else if isSequentialNum() {
//            messageError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_SEQUENTIAL_NUM")
//            checkError = true
//            labelKey = (changePasswordView?.newPasswordTextField)!
//            
//        }
        
        
        if checkError {
            self.showAlertMessage(messageError, alertMessage: "", labelKey: labelKey!, type: self.FOCUS_MODE)
            
        } else {
            checkServiceValidPassword()
            
        }
        return checkError
    }
    
    // MARK :- Check Password Method
//    func isValidPassword() -> Bool  {
//        let password = self.changePasswordView?.newPasswordTextField?.text
//        let regexPattern = "^[0-9a-zA-Z\\_]+$"
//        
//        return NSPredicate(format: "SELF MATCHES %@", regexPattern).evaluateWithObject(password)
//    }
    
    func isPassMatch() -> Bool {
        
        let newPassword = self.changePasswordView?.newPasswordTextField?.text
        let confirmPassword = self.changePasswordView?.confirmPasswordTextField?.text
        
        if newPassword == confirmPassword {
            return true
        }
        
        return false
    }
    
    func checkServiceValidPassword() {
        
        let currentPassword = self.changePasswordView?.currentPasswordTextField?.text
        let newPassword = self.changePasswordView?.newPasswordTextField?.text
        let confirmPassword = self.changePasswordView?.confirmPasswordTextField?.text
        let token = applicationDelegate.appSetting?.userData?.token
        
        var labelKey = (self.changePasswordView?.newPasswordTextField)!
        var errCode = ""
        
        let serviceController = JTMServiceController()
        let url = getServiceChangePassword()
        let param = [
            "currentPassword" : currentPassword!,
            "newPassword" : newPassword!,
            "confirmNewPassword" : confirmPassword!,
            "token" : token!,
            "imei" : applicationDelegate.getDeviceUUID()
        ]
        
        serviceController.requestService(self, urlString: url, params: param) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonResult = response!.result.value
                let status = jsonResult!["status"] as! String
                
                if status == "error" {
                    errCode = jsonResult!["errorcode"] as! String
                    
                    // CASE :: Current password is not correct
                    if errCode == INCORRECT_PASSWORD {
                        labelKey = (self.changePasswordView?.currentPasswordTextField)!
                    }
                    
                    let titleMessage = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let alertMessage = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    self.showAlertMessage(titleMessage, alertMessage: alertMessage, labelKey: labelKey, type: self.FOCUS_MODE)
                    
                } else if status == "ok"{
                    // SUCCESS
                    let titleMessage = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let alertMessage = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    labelKey = (self.changePasswordView?.newPasswordTextField)!
                    self.showAlertMessage(titleMessage, alertMessage: alertMessage, labelKey: labelKey, type: self.LOGOUT_MODE)
                }
            }
        }
    }
    
    func showAlertMessage(titleMessage : String, alertMessage : String, labelKey : UITextField, type : String) {
        
        self.showAlertWithData(
            titleMessage,
            message: alertMessage,
            completionHandler:  { (UIAlertAction) -> Void in
                switch(type) {
                case self.FOCUS_MODE :
                    labelKey.becomeFirstResponder()
                    break
                case self.LOGOUT_MODE :
                    JTMLoginViewController.connectServiceLogout(self, redirectType: .BACK_TO_HOME, completion: nil)
                    break
                default :
                    labelKey.becomeFirstResponder()
                    break
                }
            },
            actionList: nil
        )
        
        
    }
    
    // Check these below condition in Service
//    func isSequentialNum() -> Bool {
//        
////        let regexPattern = "\\d{6,}+"
//        let regexPattern = "^(?=\\d{6,}$)0?1?2?3?4?5?6?7?8?9?"
//        
//        let newPassword = self.changePasswordView?.newPasswordTextField?.text
//        let seperateNumber = newPassword!.componentsSeparatedByCharactersInSet(NSCharacterSet.letterCharacterSet())
//        
//        for num in seperateNumber {
//            if NSPredicate(format: "SELF MATCHES %@", regexPattern).evaluateWithObject(num) {
//                return true
//            }
//        }
//        
//        return false
//    }
//    
//    func isRepeatNum() -> Bool {
//        
//        let regexPattern = "\\d*(\\d)\\1{5,}\\d*"
//        
//        let newPassword = self.changePasswordView?.newPasswordTextField?.text
//        let seperateNumber = newPassword!.componentsSeparatedByCharactersInSet(NSCharacterSet.letterCharacterSet())
//        
//        for num in seperateNumber {
//            if NSPredicate(format: "SELF MATCHES %@", regexPattern).evaluateWithObject(num) {
//                return true
//            }
//        }
//        
//        return false
//    }
}
