//
//  JTMFavoriteJobsViewController.swift
//  JobThai
//
//  Created by kanya on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMFavoriteJobsViewController: JTMBaseMenuViewController, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout, UICollectionViewDelegate, JTMFavoriteJobCollectionViewCellDelegate {

    let FAVORITE_CELL_IDENTIFIER = "JTMFavoriteJobCollectionViewCell"
    let FOOTER_CELL_IDENTIFIER = "JTMSearchResultListFooterCollectionView"
    
    let COLLECTIONVIEW_INSET : CGFloat = 8
    let CELL_MARGIN_TO_COLLECTIONVIEW : CGFloat = 16
    let CELL_HEIGHT :CGFloat = 160
    let FOOTER_CELL_HEIGHT : CGFloat = 44
    let START_BACK_TO_TOP_INDEX : Int = 10
    let BACK_TO_TOP_HIDE_ALPHA : CGFloat = 0
    let BACK_TO_TOP_SHOW_ALPHA : CGFloat = 0.7
    let CELL_PADDING : CGFloat = 10
    let CELL_BETWEEN_PADDING : CGFloat = 0
    
    var isBrowseToJobDetail = false
    var isFavoriteListChanged = false
    var selectedJobCode : Int?
    var isHasLoadMore = false
    var favoriteJobsData : FavoriteJobModel?
    var noResultView : JTMResumeNoResultView?
    
    @IBOutlet weak var positionLabel: UILabel!
    @IBOutlet weak var positionUnitLabel: UILabel!
    @IBOutlet weak var countPositionBackgroundView: UIView!
    @IBOutlet weak var autoRemoveNoticeLabel: UILabel!
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var backToTopButton: UIButton!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initObserver()
        initQuickSearchNavigationItemButton()
        initNoResultView()
        registerNibForCollectionView()
        clearControllerData()
        setAccessibilityIdentifier()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("FavoriteJobs")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_FAVORITE_JOBS)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FAVORITE_JOB, action: ACTION_VIEW, label: LABEL_VIEW_FAV_JOB)
        
        setShowNoResultView(false)
        
//        if (!isBrowseToJobDetail || isFavoriteListChanged) {
//            callService()
//        }
        
        noResultView?.frame = self.view.bounds
        
        isBrowseToJobDetail = false
        isFavoriteListChanged = false
    }
    
    override func viewDidDisappear(animated: Bool) {
        super.viewDidDisappear(animated)
        
        if (isBrowseToJobDetail == false) {
            clearControllerData()
        }
    }
    
    override func setControllerFlags() {        
        isNeedToDimOnConnectionError = true
        isCanUseAfterLogInOnly = true
    }
    
    override func clearControllerData() {
        isLoadingService = false
        selectedJobCode = nil
        isHasLoadMore = false
        favoriteJobsData = nil
        collectionView.reloadData()
        hideAllHeaderLabel()
        setShowBackToTopButton(false)
        setShowNoResultView(false)
    }

    //MARK:- Manage Connection Error
    override func onRetryConnection() {
        super.onRetryConnection()
        if (applicationDelegate.appSetting?.isLogin)!
            && ((favoriteJobsData == nil) || collectionView.visibleCells().contains({$0.isKindOfClass(JTMSearchResultListFooterCollectionView)})) {
            callService()
        }
    }
    
    //MARK:- Manage View On Error
    override func manageViewOnError(view: UIView) {
        super.manageViewOnError(view)
        if (favoriteJobsData == nil) {
            setAlphaShowToView(view)
        }
    }

    //MARK:- No Result View
    func initNoResultView() {
        noResultView = JTMResumeNoResultView(frame: self.view.bounds)
        noResultView!.setNoResultForFavoriteJob()
        self.view.addSubview(noResultView!)
    }
    
    func setShowNoResultView(isShow: Bool) {
        noResultView?.hidden = !isShow
    }
    
    //MARK:- Observer
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(onFavoriteListChanged), name: NOTI_FAVORITE_LIST_CHANGED, object: nil)
    }
    
    func onFavoriteListChanged() {
        if (checkIfControllerIsShowing() == false) {
            clearControllerData()
            isFavoriteListChanged = true
        }
    }

    //MARK:- Language changed observer
    override func onLanguageChange(){
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_FAVORITE_JOB")
        noResultView?.setNoResultForFavoriteJob()
    }
    
    //MARK:- Call Service
    func callService() {
        let serviceController = JTMServiceController()
        let url = getServiceFavoriteJob()
        let indexStart = favoriteJobsData == nil ? 0 : (favoriteJobsData?.favoriteJobList.count)!
        let token = applicationDelegate.appSetting?.userData?.token
        let sessionId = applicationDelegate.appSetting?.userData?.sessionId
        let params = [
            "indexStart" : indexStart,
            "token" : token!,
            "sessionId" : sessionId!
        ]
        
        serviceController.requestService(self, urlString: url, params: params as? [String : AnyObject], isPresentLoadingIndicator: (indexStart == 0)) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let responseJSON = response!.result.value as! NSMutableDictionary
                if (responseJSON.objectForKey("status") as! String == "ok") {
                    self.parseJSONToObject(responseJSON)
                    self.collectionView.reloadData()
                    self.setShowNoResultView(self.favoriteJobsData?.countAllJobs == 0)
                    self.isHasLoadMore = self.favoriteJobsData?.favoriteJobList.count < self.favoriteJobsData?.countAllJobs
                } else {
                    self.onConnectionError(nil)
                }
            }
        }
    }
    
    //MARK:- Controller's Data
    func parseJSONToObject(favoriteJobJSONData: NSMutableDictionary) {
        if (favoriteJobsData == nil) {
            favoriteJobsData = FavoriteJobModel(favoriteJobsJSONData: favoriteJobJSONData)
            self.setHeaderLabel((self.favoriteJobsData?.countAllJobs)!) //Label is all position, so just set on first time only
        } else {
            favoriteJobsData?.appendNewPageToFavoriteJobList(favoriteJobJSONData)
        }
    }
    
    func getNumberOfCellForCollectionView() -> Int {
        if (favoriteJobsData == nil) {
            return 0
        }
        
        let favoriteJobList = favoriteJobsData?.favoriteJobList
        if (isHasLoadMore) {
            return (favoriteJobList?.count)! + 1
        } else {
            return (favoriteJobList?.count)!
        }
    }
    
    //MARK:- CollectionView
    func registerNibForCollectionView() {
        let favoriteCellNib = UINib(nibName: FAVORITE_CELL_IDENTIFIER, bundle: nil)
        collectionView.registerNib(favoriteCellNib, forCellWithReuseIdentifier: FAVORITE_CELL_IDENTIFIER)
        
        let footerCellNib = UINib(nibName: FOOTER_CELL_IDENTIFIER, bundle: nil)
        collectionView.registerNib(footerCellNib, forCellWithReuseIdentifier: FOOTER_CELL_IDENTIFIER)
    }
    
    //MARK:- CollectionView Datasource
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return 1
    }
    
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return getNumberOfCellForCollectionView()
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        if (indexPath.row + 1 == getNumberOfCellForCollectionView() && isHasLoadMore) {
            let loadMoreCell = collectionView.dequeueReusableCellWithReuseIdentifier(FOOTER_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMSearchResultListFooterCollectionView
            loadMoreCell?.accessibilityIdentifier = ID_FAVORITE_LOAD_MORE
            return loadMoreCell!
        } else {
            let jobItem = (favoriteJobsData?.favoriteJobList[indexPath.row])!
            let cell = collectionView.dequeueReusableCellWithReuseIdentifier(FAVORITE_CELL_IDENTIFIER, forIndexPath: indexPath) as? JTMFavoriteJobCollectionViewCell
            cell!.delegate = self
            cell?.setCellData(indexPath.row, jobItem: jobItem)
//            cell?.setBorderWidth(jobItem.jobCode == selectedJobCode)
            return cell!
        }
    }
    
    //Mark:- CollectionView Delegate Flow Layout
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
//        let height = (indexPath.row + 1 == getNumberOfCellForCollectionView() && isHasLoadMore) ? FOOTER_CELL_HEIGHT : CELL_HEIGHT
        let width = collectionView.frame.width
   
//        return CGSize(width: collectionView.frame.width - DEFAULT_RESUME_CELL_MARGIN, height: height)
        
        if (indexPath.row + 1 == getNumberOfCellForCollectionView() && isHasLoadMore) {
            return CGSize(width: width, height: FOOTER_CELL_HEIGHT)
        } else {
            return collectionView.ar_sizeForCellWithIdentifier(FAVORITE_CELL_IDENTIFIER, indexPath: indexPath, fixedWidth: width, configuration: { (cellItem) in
                let jobItem = (self.favoriteJobsData?.favoriteJobList[indexPath.row])!
                if let cell = cellItem as? JTMFavoriteJobCollectionViewCell {
                    cell.delegate = self
                    cell.setCellData(indexPath.row, jobItem: jobItem)
                }
            })
        }
        
    }
    
//    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAtIndex section: Int) -> UIEdgeInsets {
////        return UIEdgeInsets(top: COLLECTIONVIEW_INSET, left: COLLECTIONVIEW_INSET, bottom: COLLECTIONVIEW_INSET, right: COLLECTIONVIEW_INSET)
//        return UIEdgeInsets(top: 0, left: DEFAULT_RESUME_CELL_MARGIN/2, bottom: 0, right: DEFAULT_RESUME_CELL_MARGIN/2)
//    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAtIndex section: Int) -> CGFloat {
//        return COLLECTIONVIEW_INSET
//        return CELL_PADDING
        return CELL_BETWEEN_PADDING
    }
    
    //Mark:- CollectionView Delegate
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        if (indexPath.row + 1 != getNumberOfCellForCollectionView() || !isHasLoadMore) {
            
            // Track Google Analytic Event
            applicationDelegate.trackGoogleAnalyticEvent(CAT_FAVORITE_JOB, action: ACTION_CLICK, label: LABEL_JOB_DETAIL)
            
            let jobItem = (favoriteJobsData?.favoriteJobList[indexPath.row])!
            selectedJobCode = jobItem.jobCode
            collectionView.reloadData()
            
            if (jobItem.onlineStatus == "1") {
                isBrowseToJobDetail = true
                openJobDetail((favoriteJobsData?.favoriteJobList[indexPath.row])!)
            } else {
                showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_CLOSED_JOB"), message: nil, completionHandler: nil, actionList:  nil)
            }
        }
    }
    
    func collectionView(collectionView: UICollectionView, didHighlightItemAtIndexPath indexPath: NSIndexPath) {
        if (indexPath.row + 1 != getNumberOfCellForCollectionView() || !isHasLoadMore) {
            let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMFavoriteJobCollectionViewCell
            cell?.backgroundColor = JTM_HIGHLIGHT_COLOR
        }
    }
    
    func collectionView(collectionView: UICollectionView, didUnhighlightItemAtIndexPath indexPath: NSIndexPath) {
        if (indexPath.row + 1 != getNumberOfCellForCollectionView() || !isHasLoadMore) {
            let cell = collectionView.cellForItemAtIndexPath(indexPath) as? JTMFavoriteJobCollectionViewCell
            
            print("deselected cell at index :: \(indexPath.row)")
            cell?.backgroundColor = UIColor.whiteColor()
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    //Mark:- UIBarButton
    override func genUISearchMenuButton() -> UIBarButtonItem? {
        return nil
    }
    
    //MARK:- Favotire Cell Delegate
    func onClickRemoveFavoriteButton(cell: JTMFavoriteJobCollectionViewCell, index: Int) {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FAVORITE_JOB, action: ACTION_CLICK, label: LABEL_REMOVE_FAV_JOB)
        
        let cancelAction = UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("CANCEL"),
                                        style: UIAlertActionStyle.Cancel,
                                        handler: nil)
        
        self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_CONFIRM_REMOVE_FAVORITE"),
                               message: nil,
                            //message: applicationDelegate.getLocalizeStringForKey("MESSAGE_CONFIRM_REMOVE_FAVORITE"),
                            completionHandler: { (UIAlertAction) -> Void in
                                                    self.callServiceRemoveFavorite(index)
                            },
                            actionList: [cancelAction])
    }
    
    func callServiceRemoveFavorite(index: Int) {
        let jobItem = favoriteJobsData?.favoriteJobList[index]
        applicationDelegate.appSetting?.callServiceRemoveFavoriteForJobCode(self, jobCode: (jobItem?.jobCode)!, completion: { () -> Void in
            self.removeJobFromFavoriteList(index)
            self.setShowNoResultView((self.favoriteJobsData?.countAllJobs)! == 0)
            self.checkToLoadMore()
        })
    }

    func removeJobFromFavoriteList(index: Int) {
        favoriteJobsData?.favoriteJobList.removeAtIndex(index)
        favoriteJobsData?.countAllJobs -= 1
        setHeaderLabel((self.favoriteJobsData?.countAllJobs)!)
        collectionView.reloadData()
    }
   
    //MARK:- ScrollView Delegate
    func scrollViewDidScroll(scrollView: UIScrollView) {
        checkToLoadMore()
        
//        checkToSetBackToTop()
        
        let visibleIndex = collectionView.indexPathsForVisibleItems()
        
        if visibleIndex.count > 0 {
            let arrayVisible = visibleIndex.sort {$0.row < $1.row}
            let index = arrayVisible.last!.row + 1
            
            if(index >= self.START_BACK_TO_TOP_INDEX && scrollView.contentOffset.y != 0) {
                setShowBackToTopButton(true)
            } else {
                setShowBackToTopButton(false)
            }
        }
        
    }
    
    func checkToLoadMore() {
        if (checkIfScrollReachCellAtIndex(getNumberOfCellForCollectionView()) && !isLoadingService && isHasLoadMore) {
            self.callService()
        }
    }
    
    func checkToSetBackToTop() {
//        if (checkIfScrollReachCellAtIndex(START_BACK_TO_TOP_INDEX)) {
//            setShowBackToTopButton(true)
//        } else {
//            setShowBackToTopButton(false)
//        }
    }
    
    func checkIfScrollReachCellAtIndex(index:Int) -> Bool {
        let currentScrollOffset = collectionView.contentOffset.y + collectionView.frame.height
//        let cellOffset = CELL_HEIGHT * CGFloat(index - 1) + COLLECTIONVIEW_INSET * CGFloat(index)
        let cellOffset = CELL_HEIGHT * CGFloat(index - 1) + (DEFAULT_RESUME_CELL_MARGIN/2) * CGFloat(index)
        
        return currentScrollOffset >= cellOffset
    }
    
    //MARK:- Back to top
    func setShowBackToTopButton(isShow: Bool) {
        backToTopButton.alpha = isShow ? BACK_TO_TOP_SHOW_ALPHA : BACK_TO_TOP_HIDE_ALPHA
    }
    
    @IBAction func onClickBackToTop(sender: AnyObject) {
        collectionView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
    }
    
    //MARK:- HeaderView
    func hideAllHeaderLabel() {
        countPositionBackgroundView.hidden = true
        positionLabel.text = ""
        positionUnitLabel.text = ""
        autoRemoveNoticeLabel.text = ""
    }
    
    func setHeaderLabel(position: Int) {
        countPositionBackgroundView.hidden = false
        positionLabel.text = "\(position)"
        positionUnitLabel.text = applicationDelegate.getLocalizeStringForKey("PREFIX_POSITION")
        autoRemoveNoticeLabel.text = applicationDelegate.getLocalizeStringForKey("JOB_WILL_BE_AUTO_REMOVE")
    }
    
    //MARK:- Open job detail
    func openJobDetail(jobItem: FavoriteJobItem) {
        let jobDetailStoryBoard = UIStoryboard(name: "JobDetail", bundle: nil)
        let jobdetail = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMJobDetailViewController") as? JTMJobDetailViewController
        jobdetail?.companyCode = jobItem.companyCode!
        jobdetail?.jobCode = jobItem.jobCode!
        jobdetail?.openPageType = .JOB_INFO_PAGE
        
        self.navigationController?.pushViewController(jobdetail!, animated: true)
    }
    
    //MARK:- Identifier for UITest
    func setAccessibilityIdentifier() {
        collectionView.accessibilityIdentifier = ID_FAVORITE_JOB_COLLECTIONVIEW
        noResultView?.accessibilityIdentifier = ID_FAVORITE_NO_RESULT
    }
}
