//
//  JTMForgotPasswordViewController.swift
//  JobThai
//
//  Created by weerapon on 5/31/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMForgotPasswordView : UIView, UITextFieldDelegate {
    
    var view : UIView?
    var delegate : JTMForgotPasswordDelegate?
    
    @IBOutlet weak var emailForgotTextfield : JTMTextField?
    @IBOutlet weak var sendButton : JTMButton?
    @IBOutlet weak var titleForgotLable : JTMLabel?
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.setUpUi()
    }

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.setUpUi()
    }
    
    // Mark-: View touch set keyboard hide
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        self.emailForgotTextfield!.endEditing(true)
    }
    
    // Mark-: Set up ui
    func setUpUi() {
        let view = self.loadViewFromNib()
        view.frame = self.bounds
        view.autoresizingMask = [.FlexibleWidth, .FlexibleHeight]
        self.addSubview(view)
        
        self.setUIProperties()
        self.setTextFieldPadding(emailForgotTextfield!)
    }
    
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMForgotPasswordView", bundle: bundle)
        let view = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    func setUIProperties() {
        // Set Properties Button Center Horizental and Vertical
        self.sendButton!.titleLabel?.numberOfLines = 0
        self.sendButton!.titleLabel?.textAlignment = .Center
        
        // Set Properties Label textfield
        self.emailForgotTextfield!.text = ""
        self.emailForgotTextfield!.placeholder = applicationDelegate.getLocalizeStringForKey("PLACEHOLDER_FORGOT_PASSWORD")
        
        self.titleForgotLable?.text = applicationDelegate.getLocalizeStringForKey("TITLE_FORGOT_PASSWORD")
        let attrString = NSMutableAttributedString(string: (self.titleForgotLable?.text)!)
        let paragraphStyle = NSMutableParagraphStyle()
        paragraphStyle.lineSpacing = 6
        attrString.addAttribute(NSParagraphStyleAttributeName, value:paragraphStyle, range:NSMakeRange(0, attrString.length))
        self.titleForgotLable?.attributedText = attrString
        
        self.sendButton?.setTitle(applicationDelegate.getLocalizeStringForKey("BUUTON_TEXT_FORGOT"), forState: .Normal)
    }
    
    func setTextFieldPadding(textfield:UITextField) {
        let paddingView = UIView(frame: CGRectMake(0, 0, 3, self.emailForgotTextfield!.frame.height))
        textfield.leftView = paddingView
        textfield.rightView = paddingView
        textfield.leftViewMode = .Always
        textfield.rightViewMode = .UnlessEditing
        textfield.delegate = self
    }
    
    func textFieldShouldClear(textField: UITextField) -> Bool {
        return true
    }
    
    //Mark:- Set focus
    func setfocusOnEmailTextField() {
        self.emailForgotTextfield!.becomeFirstResponder()
    }
    
    //Mark:- Textfield delegate
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        let nextTag: NSInteger = textField.tag + 1
        
        if let nextResponder: UIResponder! = textField.superview!.viewWithTag(nextTag){
            nextResponder.becomeFirstResponder()
        } else {
            textField.resignFirstResponder()
            self.delegate?.onClickSend()
        }
        
        return false
    }
    
    func textFieldDidEndEditing(textField: UITextField) {
        if textField == emailForgotTextfield {
            self.trimSpace(self.emailForgotTextfield!)
        }
    }
    
    func trimSpace(textField : UITextField) {
        textField.text = textField.text!.stringByTrimmingCharactersInSet(
            NSCharacterSet.whitespaceAndNewlineCharacterSet()
        )
    }
    
    @IBAction func onClickForgotPassword() {
        self.endEditing(true)
        self.delegate?.onClickSend()
    }
}

protocol JTMForgotPasswordDelegate {
    func onClickSend()
}
