//
//  JTMForgotPasswordViewController.swift
//  JobThai
//
//  Created by weerapon on 5/31/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMForgotPasswordViewController : JTMBaseViewController, JTMForgotPasswordDelegate {

    @IBOutlet weak var viewForgotPassword : JTMForgotPasswordView?
    
    var parentToBack : UIViewController?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.onLanguageChange()
        self.viewForgotPassword?.delegate = self
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_FORGOT_PASSWORD)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FORGOT_PASSWORD, action: ACTION_VIEW, label: LABEL_VIEW_FORGOT_PASSWORD)
    }
    
    override func onLanguageChange() {
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_FORGOT_PASSWORD")
    }
    
    func checkServiceValidEmail() {
        
        let serviceController = JTMServiceController()
        
        // Get Value From User
        let email = self.viewForgotPassword?.emailForgotTextfield?.text
        let url = getServiceForgotPassword()
        
        let param = [
            "email": email!,
            "imei": applicationDelegate.getDeviceUUID(),
        ]
        
        serviceController.requestService(self, urlString: url, params: param) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonResult = response!.result.value
                let status = jsonResult!["status"] as! String
                if (status == "ok") {
                    let titleMsg = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let errorMsg = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    var backType : ActionForgotPassword = .BACK_HOME
                    if self.parentToBack != nil {
                        backType = .BACK_VIEW_CONTROLLER
                    }
                    self.showAlertMessage(titleMsg, alertMessage: errorMsg, type: backType)
                } else if (status == "error") {
                    let titleMsg = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let errorMsg = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    self.showAlertMessage(titleMsg, alertMessage: errorMsg, type: .FOCUS)
                }
            }
        }
    }
    
    // Mark-: Delegate on click button
    func onClickSend() {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_FORGOT_PASSWORD, action: ACTION_CLICK, label: LABEL_FORGOT_PASS)
        
        if (applicationDelegate.appSetting?.isNetworkReachable)! == false {
            return
        }
        
        self.forgotPassword()
    }
    
    // func check forget password
    func forgotPassword() {
        if (viewForgotPassword?.emailForgotTextfield?.text == "") {
            self.showAlertMessage(applicationDelegate.getLocalizeStringForKey("ALERT_NO_ENTER_EMAIL"), alertMessage:  "", type: .FOCUS)
        } else if(!(self.viewForgotPassword?.emailForgotTextfield?.isValidEmailFormat())!) {
            self.showAlertMessage(applicationDelegate.getLocalizeStringForKey("ALERT_EMAIL_NOT_CORRECT_FORMAT"),
                                    alertMessage: applicationDelegate.getLocalizeStringForKey("MESSAGE_EMAIL_NOT_CORRECT_FORMAT"), type: .FOCUS)
        } else {
            checkServiceValidEmail()
        }
    }
    
    func showAlertMessage(titleMessage : String, alertMessage : String, type : ActionForgotPassword) {
        self.showAlertWithData(
            titleMessage,
            message: alertMessage,
            completionHandler:  { (UIAlertAction) -> Void in
                switch(type) {
                    case .FOCUS :
                        self.viewForgotPassword?.setfocusOnEmailTextField()
                        break
                    case .BACK_HOME :
                        NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
                        break
                    case .BACK_VIEW_CONTROLLER :
                        self.navigationController?.popToViewController(self.parentToBack!, animated: true)
                        break
                }
                
            },
            actionList: nil
        )
    }
}