//
//  JTMFullScreenMapViewController.swift
//  JobThai
//
//  Created by kanya on 4/11/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMFullScreenMapViewController: JTMBaseViewController, TNMapViewDelegate {

    @IBOutlet weak var mapView: TNMapView!
    var annotations         = [RMAnnotation]()
    var isNoMap             : Bool = false
    var delegate : JTMFullScreenMapViewDelegate?
    
    // Property for no mapView
    var companyName         : String!
    var myJobList           : [String]?
    var location            : CLLocationCoordinate2D!
    var myUserInfo          : AnyObject!
    var companyAddress      : String?
    
    var calloutShowType     : mapCalloutType = mapCalloutType.CALLOUT_TYPE_JOBLIST
    
    //CopyRight
    @IBOutlet var copyRightLogoImageView : UIImageView!
    @IBOutlet var copyRightNameImageView : UIImageView!

    var mapLanguage : TNMapLanguage = TNMapLanguageThai
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initialMapView()
        addAnnotationToMapView()
        onLanguageChange()
        addCopyRightToMapView()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        mapView.zoomToFitAllAnnotationWithOutUserLocation()
        showCurrentAnnotationCallout()
    }
    
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("FULLSCREEN_MAP_TITLE")
    }
    
    //MARK:- Reachability change update
    override func onRetryConnection() {
        super.onRetryConnection()
        mapView.reloadMapView()
    }
    
    //MARK: Initial Map
    func initialMapView() {
        setMapTileSourceLanguage()
        mapView.setMapLevel(6)
        mapView.delegate = self
    }
    
    func setMapTileSourceLanguage() {
        mapLanguage = applicationDelegate.appSetting?.currentLanguage == .THAI ? TNMapLanguageThai : TNMapLanguageEnglish
        mapView.setMapTileLanguage(mapLanguage)
        setMapViewCopyRightLanguage(mapLanguage)
    }
    
    func addAnnotationToMapView() {
        if isNoMap {
            print("self.companyName : \(self.companyName)")
            print("myJobList : \(self.myJobList)")
            print("location : \(self.location)")
            print("myUserInfo : \(self.myUserInfo)")
            
            var annotation : RMAnnotation?
            if(calloutShowType == .CALLOUT_TYPE_JOBLIST && myJobList != nil){
                annotation = mapView.createAnnotationWithJobList(self.companyName, jobList : self.myJobList! , coordinate: self.location, userInfo: self.myUserInfo)
            }else{
                if(companyAddress != nil){
                    annotation = mapView.createAnnotationWithAddress(self.companyName, address: self.companyAddress! , coordinate: self.location, userInfo: self.myUserInfo)
                }
            }
            if(annotation != nil){
                self.annotations = [annotation!]
            }
        }
        
        self.annotations = self.mapView.copyAnnotations(self.annotations)!
        mapView.addAnnotationsToMapView(self.annotations)
        print("view: \(UIScreen.mainScreen().bounds.size.width) map: \(mapView.frame.width)")
    }
    
    //MARK:- Map Delegate
    func mapViewTapOnCalloutAccessoryControl(control: UIControl, forAnnotation annotation: RMAnnotation, onMap map: RMMapView) {
        delegate?.didTapCallout(self)
    }
    
    func mapViewSingleTapOnMap(map: RMMapView, at point: CGPoint) {
        
    }
    
    //MARK: Map CopyRight
    func addCopyRightToMapView(){
        setMapViewCopyRightLanguage(TNMapLanguage.init((applicationDelegate.appSetting?.currentLanguage.rawValue)!))
    }
    
    func setMapViewCopyRightLanguage(language : TNMapLanguage){
        if(self.mapView.mapMagicImg == nil){
            mapView.initialCopyRightImage()
        }
        
        copyRightLogoImageView.image = mapView.mapMagicImg
        
        if(language == TNMapLanguageThai){
            copyRightNameImageView.image = mapView.copyRightThImg
        }else{
            copyRightNameImageView.image = mapView.copyRightEnImg
        }
    }
    
    //Show Annotation Callout 
    func showCurrentAnnotationCallout() {
//        self.performSelector(Selector("showCallout"), withObject: nil, afterDelay: 0.2)
        if let annotation = self.annotations.first {
            self.mapView.selectAnnotation(annotation, showAnimate: true)
        }
    }
    
    func showCallout(){
        if let annotation = self.annotations.first {
            self.mapView.selectAnnotation(annotation, showAnimate: true)
        }
    }
}

protocol JTMFullScreenMapViewDelegate {
    func didTapCallout(controller:JTMFullScreenMapViewController)
}
