//
//  JTMJobDetailInfoViewController.swift
//  JobThai
//
//  Created by weerapon on 3/15/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMJobDetailInfoViewController : JTMBaseViewController, UITableViewDataSource, UITableViewDelegate, JTMButtonWithImageAboveDescriptionDelegate, JTMFullScreenMapViewDelegate {
    
    let PAGE_INDEX: IndexJobDetailPage = .JOB_INFO_PAGE
    let HEADER_CELL_IDENTIFIER = "JTMJobDetailInfoHeaderTableViewCell"
    let TEXT_CELL_IDENTIFIER = "JTMJobDetailTextTableViewCell"
    let IMAGE_CELL_IDENTIFIER = "JTMJobDetailImageTableViewCell"
    let MAP_CELL_IDENTIFIER = "JTMJobDetailMapTableViewCell"
    let HOW_TO_APPLY_CELL_IDENTIFIER = "JTMJobDetailHowToApplyTableViewCell"
    
//    var TABLE_FOOTER_HEIGHT: CGFloat = 44
    let PIXEL_OF_HIDDEN_HEADER : CGFloat = 5
    let ANIMATION_DURATION = 0.3
    
    var jobData: PositionModel?
    var infoList = [JobInfoModel]()
    var companyName: String?
    
    var containerViewController : JTMJobDetailViewController?
    var delegateOnScroll : JTMJobDetailMainForSubPageViewDelegate?
    var jobdetailPageViewController : JTMJobDetailPageViewController?
    var checkOnScrolling : Bool = false
    var isWaitingForUpdate : Bool = false
    var isNeedToCallApplyNow : Bool = false
    var isNeedToAddFavorite : Bool = false
    
    var mapInfoData   : NSMutableDictionary?
    var mapAnnotation : RMAnnotation?
    
    @IBOutlet var footerHeight: NSLayoutConstraint!
    @IBOutlet var tableView: UITableView!
    @IBOutlet var jobInfoFooterView: JTMJobDetailInfoFooterView!
    @IBOutlet var jobInfoFooterViewBottomConstraint: NSLayoutConstraint!
    
    override func viewDidLoad() {
        super.viewDidLoad()
    
        initTableView()
        initJobInfoFooterView()
//        setShowJobDetailFooterView(false)
    }
   
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("JobDetailInfo  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_JOB_DETAIL_JOB)
        
        checkOnScrolling = false
        
        if (infoList.count > 0 && isPresentedBefore) {
            tableView?.scrollToRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), atScrollPosition: .Top, animated: false)
        }
        
        setTableFooter()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    //MARK:- Reachability change update
    override func onRetryConnection() {
        super.onRetryConnection()
        if mapInfoData != nil {
            NSNotificationCenter.defaultCenter().postNotificationName(OBSERVER_RELOAD_MAP_CELL, object: mapInfoData)
            tableView.reloadData()
        }
    }
    
    override func setControllerFlags() {
        //Need to set flag before call super cause we need it set as true before initnointernetview
        isPresentedInOtherController = true
    }
    
    override func setPresentedBeforeFlag() {
        super.setPresentedBeforeFlag()
        print("loading? \(self.jobdetailPageViewController?.isLoadingService)")
        //if this controller present first time, it won't reload data on viewwillappear, so it will reload data after viewdidappear
        if (infoList.count > 0) {
            tableView?.reloadData()
//            setShowJobDetailFooterView(true)
            self.containerViewController?.hideLoading()
        }else if (self.jobdetailPageViewController?.isLoadingService == false) {
            self.containerViewController?.hideLoading()
        }
    }
    
    //MARK:- FooterView
    func initJobInfoFooterView() {
        jobInfoFooterView.applyButton.delegate = self
        jobInfoFooterView.addFavoriteButton.delegate = self
        jobInfoFooterView.shareButton.delegate = self
        
//        jobInfoFooterView.backgroundColor = UIColor.clearColor()
    }
    
    //MARK:- UpdateData
    func updateJobData(jobData: PositionModel, companyName: String) {
        self.jobData = jobData
        updateIsFavoriteJob(jobData.isFavorite!)
        
        self.infoList = jobData.jobInfo
        self.companyName = companyName
        
        //Reload Table Data & Footer
        if (isPresentedBefore) {
            tableView.reloadData()
            tableView.scrollToRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), atScrollPosition: .Top, animated: false)
            let isShowFooter = self.infoList.count > 0
            setShowJobDetailFooterView(isShowFooter)
        }
        
        autoCallFunctionWhenFinishedUpdateDataAfterRedirect()
        
        jobInfoFooterView.setApplyNowButtonStatus(jobData.applyButtonDisabledStatus!)
    }
    
    func autoCallFunctionWhenFinishedUpdateDataAfterRedirect() {
        
        /* 
         * Job detail always call service on viewWillAppear and set flag "isWaitingForUpdate" to 'true'
         * so after redirect, this page will wait until calling service finish (updateJobData func is called)
         * and then check if it need to call apply job/ add favorite
        */
        
        if isWaitingForUpdate {
            isWaitingForUpdate = false
            
            if isNeedToCallApplyNow {
                isNeedToCallApplyNow = false
                onClickApplyNowButton()
            } else if isNeedToAddFavorite {
                isNeedToAddFavorite = false
                tryAddFavoriteJobAfterLogIn()
            }
        }
    }

    //MARK:- TableView
    func initTableView() {
        registerNibForJobInfoTableView()
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 44
    }
    
    func setTableFooter() {
        tableView.tableFooterView = UIView(frame: CGRect(x: 0, y: 0, width: tableView.frame.width, height: footerHeight.constant))
    }
    
    func registerNibForJobInfoTableView() {
        let headerNib = UINib(nibName: HEADER_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(headerNib, forCellReuseIdentifier: HEADER_CELL_IDENTIFIER)
        
        let textNib = UINib(nibName: TEXT_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(textNib, forCellReuseIdentifier: TEXT_CELL_IDENTIFIER)
        
        let imageNib = UINib(nibName: IMAGE_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(imageNib, forCellReuseIdentifier: IMAGE_CELL_IDENTIFIER)
        
        let mapNib = UINib(nibName: MAP_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(mapNib, forCellReuseIdentifier: MAP_CELL_IDENTIFIER)
        
        let howToNib = UINib(nibName: HOW_TO_APPLY_CELL_IDENTIFIER, bundle: nil)
        tableView.registerNib(howToNib, forCellReuseIdentifier: HOW_TO_APPLY_CELL_IDENTIFIER)
    }
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return isPresentedBefore ? 1 : 0
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return infoList.count
    }
 
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let infoForCell = infoList[indexPath.row]
        switch (infoForCell.type)! {
            case .TEXT_CELL:
                return configureCellForText(tableView, infoForCell: infoForCell, isDetectForLink: false)
            case .TEXT_DETECT_CELL:
                return configureCellForText(tableView, infoForCell: infoForCell, isDetectForLink: true)
            case .HEADER_CELL:
                return configureCellForHeader(tableView, infoForCell: infoForCell)
            case .IMAGE_CELL:
                return configureCellForImage(tableView, infoForCell: infoForCell)
            case .MAP_CELL:
                let isHasDrawMap = infoList.contains({$0.type == .IMAGE_CELL})
                return configureCellForMap(tableView, infoForCell: infoForCell, isHasDrawMap: isHasDrawMap)
            case .HOW_TO_APPLY_CELL:
                return configureCellForHowToApply(tableView, infoForCell: infoForCell)
            default:
                return UITableViewCell()
        }
    }
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        let infoForCell = infoList[indexPath.row]
        switch (infoForCell.type)! {
        case .IMAGE_CELL:
            return 0.0
        default:
            return UITableViewAutomaticDimension
        }

    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        let infoForCell = infoList[indexPath.row]
        
        switch (infoForCell.type)! {
            case .IMAGE_CELL:
                // Track Google Analytic Event
//                applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_DETAIL, action: ACTION_CLICK, label: LABEL_DRAW_MAP)
                openImageForCellAtIndex(tableView, indexPath: indexPath, infoForCell: infoForCell)
            case .MAP_CELL:
                // Track Google Analytic Event
                applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_DETAIL, action: ACTION_CLICK, label: LABEL_MAP_ONLINE)
                openFullMapView(self.mapAnnotation)
                break
            default:
                break
        }
    }
    
    func openImageForCellAtIndex(tableView: UITableView, indexPath:NSIndexPath, infoForCell: JobInfoModel) {
        let cell = tableView.cellForRowAtIndexPath(indexPath) as! JTMJobDetailImageTableViewCell
        openImageGallery(cell.jobMapImageView, listImageUrl: [NSURL(string: (infoForCell.valueInfo)!)!])
    }
    
    //MARK:- Configure Cell
    func configureCellForHeader(tableView:UITableView, infoForCell:JobInfoModel) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(HEADER_CELL_IDENTIFIER) as? JTMJobDetailInfoHeaderTableViewCell
        cell!.setCellData(infoForCell, isDisabledPersonWelcome: (jobData?.isForDisabled)!, textForDisabled: (jobData?.textForDisadled)!)
        
        return cell!
    }
    
    func configureCellForText(tableView:UITableView, infoForCell:JobInfoModel, isDetectForLink: Bool) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(TEXT_CELL_IDENTIFIER) as? JTMJobDetailTextTableViewCell
        cell!.setCellData(infoForCell, isDetectForLink: isDetectForLink)
        
        return cell!
    }
    
    func configureCellForImage(tableView:UITableView, infoForCell:JobInfoModel) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(IMAGE_CELL_IDENTIFIER) as? JTMJobDetailImageTableViewCell

        // uncomment this line + delete delgate of tableView : heightForRowAtIndexPath + Comment "cell!.hidden = true" for show Draw map
//                cell!.setCellData(infoForCell)
        cell!.hidden = true
        
        return cell!
    }
    
    func configureCellForMap(tableView:UITableView, infoForCell:JobInfoModel, isHasDrawMap: Bool) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(MAP_CELL_IDENTIFIER) as? JTMJobDetailMapTableViewCell
        cell!.setCellData(infoForCell, companyName: companyName!, isHasDrawMap: isHasDrawMap)
        saveMapInfoData(infoForCell, companyName: companyName!, isHasDrawMap: isHasDrawMap)
        self.mapAnnotation = cell!.annotation
        return cell!
    }
    
    func configureCellForHowToApply(tableView:UITableView, infoForCell:JobInfoModel) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier(HOW_TO_APPLY_CELL_IDENTIFIER) as? JTMJobDetailHowToApplyTableViewCell
        cell!.setCellData(infoForCell)
        
        return cell!
    }
    
    //MARK: Save Map Info Data
    func saveMapInfoData(infoForCell : JobInfoModel , companyName : String , isHasDrawMap: Bool) {
        if mapInfoData == nil {
            mapInfoData = NSMutableDictionary()
        } else {
            mapInfoData?.removeAllObjects()
        }
        mapInfoData?.setObject(infoForCell, forKey: MAP_CELL_ANNOTATION_INFO)
        mapInfoData?.setObject(companyName, forKey: MAP_CELL_ANNOTATION_COMPANY_NAME)
        mapInfoData?.setObject(isHasDrawMap, forKey: MAP_CELL_ANNOTATION_IS_HAS_DRAWN_MAP)
    }
    
    //MARK:- ScrollView
    func scrollViewDidScroll(scrollView: UIScrollView) {
        if (scrollView != tableView) {
            return
        }
        
        let offset : CGPoint = scrollView.contentOffset;
        
        if(offset.y > self.PIXEL_OF_HIDDEN_HEADER && !checkOnScrolling){
            self.checkOnScrolling = true
            dispatch_async(dispatch_get_main_queue()) {
                self.delegateOnScroll?.onScrollView(true ,index: self.PAGE_INDEX.rawValue)
            }
        } else if(offset.y < self.PIXEL_OF_HIDDEN_HEADER && (!isCollectionViewCantScrollToBottom() || (isCollectionViewCantScrollToBottom() && scrollView.isScrollUp(scrollView)))) {
            self.checkOnScrolling = false
            dispatch_async(dispatch_get_main_queue()) {
                self.delegateOnScroll?.onScrollView(false ,index: self.PAGE_INDEX.rawValue)
            }
        }
    }
    
    func isCollectionViewCantScrollToBottom() -> Bool {
        let collectionViewHeight = tableView?.contentSize.height
        let companyHeaderHeight = containerViewController?.headerStackView?.frame.height
        let segmentHeight = containerViewController?.segmentContainerView.frame.height
        let isCollectionViewHigherThanViewWithHideCompanyHeader = collectionViewHeight <= DEFAULT_FRAME_HEIGHT - segmentHeight!
        let isCollectionViewShorterThanViewWithShowCompanyHeader = collectionViewHeight >= DEFAULT_FRAME_HEIGHT - segmentHeight! - companyHeaderHeight!
        
        return isCollectionViewHigherThanViewWithHideCompanyHeader && isCollectionViewShorterThanViewWithShowCompanyHeader
    }

    func scrollViewWillBeginDragging(scrollView: UIScrollView) {
        if (scrollView != tableView) {
            return
        }
        
//        setShowJobDetailFooterView(false)
    }
    
    func scrollViewDidEndDragging(scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if (scrollView != tableView) {
            return
        }
        
        if (!decelerate) {
//            setShowJobDetailFooterView(true)
        }
    }
    
    func scrollViewDidEndDecelerating(scrollView: UIScrollView) {
        if (scrollView != tableView) {
            return
        }
        
//        setShowJobDetailFooterView(true)
    }
    
    func setShowJobDetailFooterView(isShow:Bool) {
//        jobInfoFooterViewBottomConstraint.constant = isShow ? 0 : -(TABLE_FOOTER_HEIGHT)
//        
//        UIView.animateWithDuration(ANIMATION_DURATION) { () -> Void in
//            self.view.layoutIfNeeded()
//        }
    }
    
    //Footer Delegate
    func onClickButton(button: JTMButtonWithImageAboveDescriptionView) {
        if (button === jobInfoFooterView.applyButton) {
            onClickApplyNowButton()
        } else if (button === jobInfoFooterView.addFavoriteButton) {
            onClickFavoriteButton()
        } else { //share button
            onClickShareButton()
        }
    }
    
    func onTouchesBegan(button: JTMButtonWithImageAboveDescriptionView) {
        jobInfoFooterView.setTouchesButtonUI(button, isTouch: true)
    }
    
    func onTouchesCancelled(button: JTMButtonWithImageAboveDescriptionView) {
        jobInfoFooterView.setTouchesButtonUI(button, isTouch: false)
    }
    
    
    func onClickApplyNowButton() {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_DETAIL, action: ACTION_CLICK, label: LABEL_APPLY_NOW)
        
        checkUserLoginStatus(.DISMISS_CONTROLLER_NO_ANIMATED,
                            notLogInWarningMessage: applicationDelegate.getLocalizeStringForKey("PLEASE_LOG_IN_TO_APPLY"),
                            completion: { () -> Void in
                                if self.isWaitingForUpdate {
                                    self.isNeedToCallApplyNow = true
                                    return
                                }
                                
                                self.tryToOpenApplyNowViewController()
                            })
    }
    
    func tryToOpenApplyNowViewController() {
        let userData = applicationDelegate.appSetting?.userData
        var applyErrorTitle = ""
        var applyErrorMessage = ""
        
        if (userData?.isBan)! {
            applyErrorTitle = applicationDelegate.getLocalizeStringForKey("ALERT_RESUME_IS_BAN")
            applyErrorMessage = applicationDelegate.getLocalizeStringForKey("MESSAGE_RESUME_IS_BAN")
        } else if (userData?.resumeStatus == .NOT_VERIFIED) {
            applyErrorTitle = applicationDelegate.getLocalizeStringForKey("ALERT_RESUME_NOT_APPROVE")
            applyErrorMessage = applicationDelegate.getLocalizeStringForKey("MESSAGE_RESUME_NOT_APPROVE")
        } else if (self.jobData?.isApplyable)! == false {
            applyErrorTitle = applicationDelegate.getLocalizeStringForKey("ALERT_FAIL_APPLYNOW")
            applyErrorMessage = applicationDelegate.getLocalizeStringForKey("MESSAGE_FAIL_APPLYNOW")
        } else {
            self.showApplyNowViewController()
            return
        }
        
        self.showAlertWithData(applyErrorTitle, message: applyErrorMessage, completionHandler: nil, actionList: nil)
    }
    
    func onClickFavoriteButton() {
        print("Add Favorite")
        
        checkUserLoginStatus(.DISMISS_CONTROLLER,
                            notLogInWarningMessage: applicationDelegate.getLocalizeStringForKey("PLEASE_LOG_IN_TO_ADD_FAV"),
                            completion: { () -> Void in
                                if self.isWaitingForUpdate {
                                    self.isNeedToAddFavorite = true
                                    return
                                }
                                
                                self.checkToAddOrRemoveFavoriteJob()
                            })
    }
    
    func onClickShareButton() {
        print("Share")
        let shareObject = JTMShareSocial()
        shareObject.shareJob(self, jobItem: jobData!, companyName: companyName!)
//        shareObject.ShareJob(self, urlJob: (jobData?.jobLink)!)
    }
    
    //MARK:- Check userlogin
    func checkUserLoginStatus(redirectType: LogInRedirectType, notLogInWarningMessage: String, completion: ()->Void) {
        if ((applicationDelegate.appSetting?.isLogin)!) {
            completion()
        } else {
            let cancelAction = UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("CANCEL"), style: .Cancel, handler: nil)
            
            showAlertWithData(notLogInWarningMessage,
                                message: nil,
                                completionHandler: { (action) -> Void in
                                    self.isWaitingForUpdate = true
                                    self.showLogInViewController(redirectType , parentToBack: self.containerViewController , completion: completion)
                                }, actionList: [cancelAction])
        }
    }
    
//    func showLogInViewController(redirectType: LogInRedirectType, completion: ()->Void) {
//        let mainStoryboard = UIStoryboard(name: "Main", bundle: nil)
//        let loginController = mainStoryboard.instantiateViewControllerWithIdentifier("JTMLoginViewController") as! JTMLoginViewController
//        loginController.redirectType = redirectType
//        loginController.completion = completion
//        self.navigationController?.pushViewController(loginController, animated: true)
//    }
    
    //MARK:- Favorite
    func checkToAddOrRemoveFavoriteJob() {
        
        if ((jobData?.isFavorite)!) {
            
            // Track Google Analytic Event
            applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_DETAIL, action: ACTION_CLICK, label: LABEL_REMOVE_FAV_JOB)
            
            let cancelAction = UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("CANCEL"),
                                            style: UIAlertActionStyle.Cancel,
                                            handler: nil)

            self.showAlertWithData(applicationDelegate.getLocalizeStringForKey("ALERT_CONFIRM_REMOVE_FAVORITE"),
                                   message: nil,
                    //message: applicationDelegate.getLocalizeStringForKey("MESSAGE_CONFIRM_REMOVE_FAVORITE"),
                    completionHandler: { (UIAlertAction) -> Void in
                        self.callServiceRemoveFavoriteJob()
                    },
                    actionList: [cancelAction])
        } else {
            
            // Track Google Analytic Event
            applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_DETAIL, action: ACTION_CLICK, label: LABEL_ADD_FAV)
            
            callServiceAddFavoriteJob()
        }
    }
    
    func tryAddFavoriteJobAfterLogIn() {
        if ((jobData?.isFavorite)! == false) {
            callServiceAddFavoriteJob()
        }
    }
    
    func callServiceAddFavoriteJob() {
        applicationDelegate.appSetting?.callServiceAddFavoriteForJobCode(containerViewController!,
            jobCode: (jobData?.runningNumber)!,
            completion: {() -> Void in
                self.updateIsFavoriteJob(true)
            })
    }
    
    func callServiceRemoveFavoriteJob() {
        applicationDelegate.appSetting?.callServiceRemoveFavoriteForJobCode(self.containerViewController!,
            jobCode: (self.jobData?.runningNumber)!,
            completion: {() -> Void in
                self.updateIsFavoriteJob(false)
            })
    }
    
    func updateIsFavoriteJob (isFavoriteJob: Bool) {
        jobData!.isFavorite = isFavoriteJob
        jobInfoFooterView?.setAddFavoriteButtonStatus(isFavoriteJob)
    }
    
    //MARK:- Apply
    func showApplyNowViewController() {
        print("Apply Now")
        let applyController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMApplyNowViewController") as! JTMApplyNowViewController
        applyController.jobData = jobData
        applyController.companyName = companyName
        self.navigationController?.pushViewController(applyController, animated: true)
    }
    
    //MARK:- Full MapView
    func openFullMapView(annotation: RMAnnotation?) {
        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        let fullMapViewController = storyboard.instantiateViewControllerWithIdentifier("JTMFullScreenMapViewController") as! JTMFullScreenMapViewController
        fullMapViewController.delegate = self
        if(annotation != nil){
            fullMapViewController.annotations = [annotation!]
        }
        
        self.navigationController?.pushViewController(fullMapViewController, animated: true)
    }
    
    //MARK:- Full MapView Delegate
    func didTapCallout(controller: JTMFullScreenMapViewController) {
        controller.navigationController?.popViewControllerAnimated(false)
    }
}
