//
//  JTMJobDetailViewController.swift
//  JobThai
//
//  Created by weerapon on 1/29/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire
import MHVideoPhotoGallery

class JTMJobDetailViewController : JTMBaseViewController, JTMJobdetailPageViewDelegate, JTMJobDetailMainForSubPageViewDelegate, JTMClickAllPositionDelegate {
    
    @IBOutlet weak var companyName : JTMLabel?
    @IBOutlet weak var companyImage : UIImageView?
    @IBOutlet weak var segment : UISegmentedControl?
    @IBOutlet weak var lineHeader : UIView?
    @IBOutlet weak var headerStackView : UIStackView?
    @IBOutlet weak var headerTopConstraint : NSLayoutConstraint?
    @IBOutlet weak var segmentContainerView: UIView!
    
    let TOP_CONSTRAINT : CGFloat = 24
    let START_CONSTRAINT : CGFloat = 0
    let TIME_DELAY : NSTimeInterval = 0.3
    var openPageType : IndexJobDetailPage = .ALL_POSITION_PAGE
    var companyCode : NSString?
    var jobCode : Int?
    var delegateMainJobDetail : JTMJobDetailMainDelegate?
    var jobdetailPageViewController : JTMJobDetailPageViewController?
    var headerHidden : Bool = false
    var titleForSegment = [
        applicationDelegate.getLocalizeStringForKey("SEGMENT_TITLE_INFO"),
        applicationDelegate.getLocalizeStringForKey("SEGMENT_TITLE_COMPANY"),
        applicationDelegate.getLocalizeStringForKey("SEGMENT_TITLE_ALLPOSITION")
    ]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.setHiddenUI()
        self.onLanguageChange()
        self.setUpSegmentTitle()
        self.segment?.selectedSegmentIndex = openPageType.rawValue
        
        //Set for MoveView to under NavigationBar
        self.edgesForExtendedLayout = .Bottom
        
        //Add Quick Search Button to SearchResultView
        self.initQuickSearchNavigationItemButton()
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- On Connection Error
    override func onConnectionError(errorRequestUrl: String?) {
        isHasRetryButtonOnAlertConnectionError = (errorRequestUrl == getServiceJobDetail("\(companyCode!)"))
        super.onConnectionError(errorRequestUrl)
    }
    
    // Mark :: Set Hidden
    func setHiddenUI() {
        self.companyImage?.hidden = true
        self.companyName?.hidden = true
        self.segment?.hidden = true
        self.lineHeader?.hidden = true
    }
    
    // Mark :: Set Show after load data finished
    func setShowUI() {
        self.companyImage?.hidden = false
        self.companyName?.hidden = false
        self.segment?.hidden = false
        self.lineHeader?.hidden = false
    }
    
    // Mark :: Set Up Title Navigation
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_JOBDETAIL")
    }
    
    // Mark :: Set Up Segment Title
    func setUpSegmentTitle(){
        self.setUpSegmentUI()
        self.segment!.removeAllSegments()
        for i in 0 ..< titleForSegment.count {
            self.segment!.insertSegmentWithTitle(titleForSegment[i], atIndex: i, animated: false)
        }
    }
    
    // Mark :: Set Text Color and Segment Selected
    func setUpSegmentUI(){
        let fontSize = UIFont(name: "HelveticaNeue", size: 14)
        let segmentTextAttributes : [NSObject : AnyObject] = [NSForegroundColorAttributeName:JTM_SEGMENT_TEXT_COLOR, NSFontAttributeName: fontSize!]
        self.segment?.tintColor = JTM_SEGMENT_SELECTED
        self.segment?.setTitleTextAttributes(segmentTextAttributes, forState: UIControlState.Normal)
        self.segment?.setTitleTextAttributes(segmentTextAttributes, forState: UIControlState.Selected)
//        self.segment?.setTitleTextAttributes([NSForegroundColorAttributeName:JTM_SEGMENT_TEXT_COLOR], forState: UIControlState.Normal)
//        self.segment?.setTitleTextAttributes([NSForegroundColorAttributeName:JTM_SEGMENT_TEXT_COLOR], forState: UIControlState.Selected)
    }
    
    // Mark :: Override Class for Send Delegate on show Container View
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "JTMJobDetailPageViewController"{
            let jobdetailPageviewController = segue.destinationViewController as? JTMJobDetailPageViewController
            self.delegateMainJobDetail = jobdetailPageviewController
            jobdetailPageviewController?.companyCode = self.companyCode as? String
            jobdetailPageviewController?.jobCode = self.jobCode
            jobdetailPageviewController?.jobDetailMainController = self
            jobdetailPageviewController?.INDEX_PAGE = self.openPageType
            jobdetailPageviewController?.delegateSetMainJobdetail = self
        }
    }
    
    // Mark :: Set Show and Hidden Header
    func handleCompanyHeader(hidden: Bool, index : Int) {
        if hidden {
            self.headerTopConstraint?.constant = self.START_CONSTRAINT - (self.headerStackView?.frame.height)!
        } else {
            self.headerTopConstraint?.constant = self.TOP_CONSTRAINT
        }
        
        UIView.animateWithDuration(self.TIME_DELAY) {
            self.view.layoutIfNeeded()
        }
    }
    
    // Mark :: Set Show and Hidden Header when Slide
    func handleCompanyHeaderSlidePageView() {
        self.headerTopConstraint?.constant = self.TOP_CONSTRAINT
        
        UIView.animateWithDuration(self.TIME_DELAY) {
            self.view.layoutIfNeeded()
        }
    }

    // Mark :: Delegate Set Title And Image After Load Service Finished
    func setTitleJobDetail(companyDetail: CompanyDetailModel) {
        self.setShowUI()
        self.companyName?.text = companyDetail.companyName
        if (companyDetail.companyLogo != "") {
            self.companyImage!.sd_setImageWithURL(NSURL(string: companyDetail.companyLogo!), placeholderImage: UIImage())
            self.companyImage?.hidden = false
        } else {
            self.companyImage?.hidden = true
        }
    }
    
    // Mark :: Delegate Set Current Segment
    func setStatusCurrentSegment(currentIndex: NSInteger) {
        self.segment!.selectedSegmentIndex = currentIndex
        
        if (self.headerTopConstraint?.constant != self.TOP_CONSTRAINT) {
            performSelector(#selector(handleCompanyHeaderSlidePageView), withObject: self, afterDelay: 0.1)
        }
    }
    
    // Mark :: Delegate Set show and hide Company Header
    func onScrollView(hidden: Bool, index : Int) {
        self.handleCompanyHeader(hidden, index: index)
    }
    
    // Mark :: Delegate onClick item from AllPosition
    func onClickItemAllPosition(indexPageView : Int, indexList : Int, jobcode: Int) {
        self.segment!.selectedSegmentIndex = indexPageView
        self.delegateMainJobDetail?.onClickSegment(indexPageView)
        self.delegateMainJobDetail?.updataDataSelectedJob(indexList, jobcode: jobcode)
    }
    
    @IBAction func segmentIndexChange(sender: AnyObject) {
        let index = self.segment!.selectedSegmentIndex
        self.delegateMainJobDetail?.onClickSegment(index)
        self.handleCompanyHeader(false, index: index)
    }
    
    static func openJobDetail(controller: JTMBaseViewController, companyCode : String, jobcode : Int, type : IndexJobDetailPage) {
        let jobDetailStoryBoard = UIStoryboard(name: "JobDetail", bundle: nil)
        let jobDetail = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMJobDetailViewController") as? JTMJobDetailViewController
        
        jobDetail?.companyCode = companyCode
        if type == .JOB_INFO_PAGE {
            jobDetail?.jobCode = jobcode
        }
        jobDetail?.openPageType = type
        
        controller.navigationController?.pushViewController(jobDetail!, animated: true)
    }

}


// Mark :: Delegate send to pageview for manage open pageview
protocol JTMJobDetailMainDelegate {
    func onClickSegment(index : Int)
    func updataDataSelectedJob(indexList : Int, jobcode : Int)
}


// Mark :: Delegate send child pageview for hidden companyHeader
protocol JTMJobDetailMainForSubPageViewDelegate {
    func onScrollView(hidden : Bool, index : Int)
}

// Mark :: Delegate receive position click 
protocol JTMClickAllPositionDelegate {
    func onClickItemAllPosition(indexPageView : Int, indexList : Int, jobcode : Int)
}

