//
//  JTMLoginView.swift
//  JobThai
//
//  Created by kanya on 1/29/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMLoginView: UIView, UITextFieldDelegate {

    //Mark:- Variable
    var delegate: JTMLoginViewDelegate?
//    var checkboxState: Bool = true
    var toggleState : Bool = true
    
    @IBOutlet weak var usernameTextField: JTMTextField!
    @IBOutlet weak var passwordTextField: JTMTextField!
    @IBOutlet weak var forgotPasswordButton: JTMButtonNoBg!
    @IBOutlet weak var rememberMeTitleLabel: JTMLabel!
    @IBOutlet weak var rememberMeSwitch: UISwitch!
    @IBOutlet weak var loginButton: JTMButton!
    @IBOutlet weak var signUpButton: JTMButtonNoBg!
    
    //Mark:- Initial
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.setUpUI()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.setUpUI()
    }
    
    //Mark:- View
    func setUpUI() {
        let view = self.loadViewFromNib()
        view.frame = self.bounds
        view.autoresizingMask = [.FlexibleWidth, .FlexibleHeight]
        self.addSubview(view)
        
        self.usernameTextField.delegate = self
        self.passwordTextField.delegate = self
        self.passwordTextField.font = nil
        self.passwordTextField.font = self.passwordTextField.font?.fontWithSize(13)
        
//        loginButton.titleLabel?.numberOfLines = 0
//        loginButton.titleLabel?.textAlignment = .Center
        self.setUILabel()
        self.setFilledTextField()
    }
    
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMLoginView", bundle: bundle)
        let view = nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    func setUILabel() {
        rememberMeTitleLabel.text = applicationDelegate.getLocalizeStringForKey("REMEMBER_ME_TITLE")
        forgotPasswordButton.setTitle(applicationDelegate.getLocalizeStringForKey("FORGOT_PASSWORD_TITLE"), forState: .Normal)
        loginButton.setTitle(applicationDelegate.getLocalizeStringForKey("LOG_IN_BUTTON_TITLE"), forState: .Normal)
        signUpButton.setTitle(applicationDelegate.getLocalizeStringForKey("SIGN_UP_TITLE"), forState: .Normal)
        
    }
    
    func setFilledTextField() {
        usernameTextField.placeholder = applicationDelegate.getLocalizeStringForKey("USERNAME_PLACEHOLDER_TITLE")
        usernameTextField.text = applicationDelegate.appSetting?.userCredential?.username
        passwordTextField.text = ""
        passwordTextField.placeholder = applicationDelegate.getLocalizeStringForKey("PASSWORD_TITLE")
        self.toggleState = (applicationDelegate.appSetting?.getIsRememberMe())!
        self.setRememberMeSwitchStates(toggleState)
    }
    
    func setRememberMeSwitchStates(state : Bool) {
        self.rememberMeSwitch.on = state
    }
    
    //Mark:- View touch
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        usernameTextField.endEditing(true)
        passwordTextField.endEditing(true)
    }
    
    //Mark:- Button Delegate
    @IBAction func onClickForgotPassword() {
        self.endEditing(true)
        delegate?.onClickForgotPassword()
    }
    
    @IBAction func rememberMeStateChange(sender: UISwitch) {
        toggleState = !toggleState
        applicationDelegate.appSetting?.setIsRememberMe(toggleState)
        self.setRememberMeSwitchStates(toggleState)
    }
    
    @IBAction func onClickLogin() {
        self.endEditing(true)
        delegate?.onclickLogin()
        self.endEditing(true)
    }
    
    @IBAction func onClickSignUp() {
        self.endEditing(true)
        delegate?.onClickSignUp()
    }
    
    
    //Mark:- Textfield delegate
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        let nextTag: NSInteger = textField.tag + 1
        
        if let nextResponder: UIResponder! = textField.superview!.viewWithTag(nextTag){
            nextResponder.becomeFirstResponder()
        }else {
            textField.resignFirstResponder()
            self.onClickLogin()
        }
        
        return false
    }
    
    func textFieldShouldClear(textField: UITextField) -> Bool {
        applicationDelegate.appSetting?.userCredential?.removeSavedUser()
        return true
    }
    
    func textFieldDidEndEditing(textField: UITextField) {
        if textField == usernameTextField {
            self.trimSpace(usernameTextField)
        }
    }
    
    func trimSpace(textField : UITextField) {
        textField.text = textField.text!.stringByTrimmingCharactersInSet(
            NSCharacterSet.whitespaceAndNewlineCharacterSet()
        )
    }
    
    //Mark:- Set focus
    func setfocusOnUsernameTextField() {
        self.usernameTextField.becomeFirstResponder()
    }
    
    func setfocusOnPasswordTextField() {
        self.passwordTextField.becomeFirstResponder()
    }
}

protocol JTMLoginViewDelegate {
    func onClickForgotPassword()
//    func onClickRememberMe()
    func onclickLogin()
    func onClickSignUp()
}
