//
//  JTMMainMenuViewController.swift
//  JobThai
//
//  Created by ForAlL on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import DynamicColor
import ENSwiftSideMenu

class JTMMainMenuViewController: UITableViewController {
//    var selectedMenuItem : Int = 0
    var selectedMenuIdentifier : String = JTM_HOME_VIEWCONTROLLER_IDENTIFIER
    
    var listMainMenu: NSMutableArray?
    var listMainMenuWithLogin: NSMutableArray?
    var listViewControllers: NSMutableDictionary?
    var parentViewHeight: CGFloat!
    
    let NAME_TH_KEY = "NameTH"
    let NAME_EN_KEY = "NameEN"
    let IDENTIFY_KEY = "IdentifyName"
    let ICON_NAME = "IconName"
    let CELL_IDENTIFIER = "JTMMainMenuTableViewCell"
    let TABLE_HEADER = "JTMMainMenuHeaderView"
    let TRACK_LABEL = "trackLabel"
    
    let TABLE_HEADER_HEIGHT : CGFloat = 100
    let TABLE_NO_HEADER_HEIGHT : CGFloat = 0
    let MENU_CELL_PADDING : CGFloat = 0
    
    var appSetting: JTMApplicationSetting?
    
    var parent : UIViewController?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initMainMenu()
        initAppSetting()
        initObserver()
        initTableProperty()
        registerNib()
        
        // Preserve selection between presentations
        self.clearsSelectionOnViewWillAppear = false
        
        setHilightToSelectedRow()
       
        self.view.backgroundColor = JTM_MENU_BG_COLOR
        self.listViewControllers = NSMutableDictionary()
        
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    //MARK:- Initial
    func registerNib() {
        tableView.registerNib(UINib(nibName: CELL_IDENTIFIER, bundle: nil), forCellReuseIdentifier: CELL_IDENTIFIER)
    }
    
    func initTableProperty() {
        // Customize apperance of table view
        tableView.contentInset = UIEdgeInsetsMake(64.0, 0, 0, 0)
        tableView.separatorStyle = .SingleLine
        tableView.separatorColor = JTM_SEGMENT_TEXT_COLOR
        tableView.backgroundColor = UIColor.clearColor()
        
        tableView.scrollsToTop = false
        tableView.bounces = false
        tableView.tableFooterView = UIView(frame: CGRectZero)
        tableView.scrollEnabled = false
        tableView.accessibilityIdentifier = ID_MAIN_MENU_TABLE_VIEWS
    }
    
    func initAppSetting() {
        self.appSetting = applicationDelegate.appSetting! as JTMApplicationSetting;
    }
    
    func initMainMenu() {
    
        if let path = NSBundle.mainBundle().pathForResource("ListMenu", ofType: "plist") {
            listMainMenu = NSMutableArray(contentsOfFile: path)!
        }
        
        if let path = NSBundle.mainBundle().pathForResource("ListMenuWithLogin", ofType: "plist") {
            listMainMenuWithLogin = NSMutableArray(contentsOfFile: path)!
        }
    }
    
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(onLanguageChange), name: "NOTI_LANGUAGE_CHANGE", object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(loginChange(_:)), name: JTM_LOGIN_STATE_DID_CHANGE, object: nil)
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(backToHomeViewController(_:)), name: JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
    }
    
    func backToHomeViewController(notification:NSNotification) {
        //let mainStoryboard: UIStoryboard = UIStoryboard(name: "Main",bundle: nil)
        var destViewController : UIViewController
        
        destViewController = getViewController(JTM_HOME_VIEWCONTROLLER_IDENTIFIER)
        selectedMenuIdentifier = JTM_HOME_VIEWCONTROLLER_IDENTIFIER
        setHilightToSelectedRow()
        
        sideMenuController()?.setContentViewController(destViewController)
        hideSideMenuView()
    }
    
    func insertViewControllerWithName(controller:UIViewController, forName:String){
        self.listViewControllers?.setObject(controller, forKey: forName)
    }
    func getViewController(name:String)->UIViewController{
        var controller:UIViewController? = self.listViewControllers?.objectForKey(name) as? UIViewController
        if(controller == nil){
            let mainStoryboard: UIStoryboard = UIStoryboard(name: "Main",bundle: nil)
            controller = mainStoryboard.instantiateViewControllerWithIdentifier(name)
            insertViewControllerWithName(controller!, forName:name)
            
            //self.listViewControllers?.setObject(controller!, forKey: name)
        }
        return controller!
    }
    
    // MARK: - Table view data source
    //Section
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    override func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return (appSetting?.isLogin)! ? TABLE_HEADER_HEIGHT : TABLE_NO_HEADER_HEIGHT
    }
    
    override func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let headerNib = UINib(nibName: TABLE_HEADER, bundle: nil)
        let headerView = headerNib.instantiateWithOwner(self, options: nil)[0] as? JTMMainMenuHeaderView
        headerView?.setUserDataToView((applicationDelegate.appSetting?.userData)!)
        
//        Add Separator line
//        let bottomBorder = UIView(frame: CGRectMake(MENU_CELL_PADDING, TABLE_HEADER_HEIGHT, tableView.frame.width, 0.70))
//        bottomBorder.backgroundColor = JTM_SEGMENT_TEXT_COLOR
//        
//        headerView?.addSubview(bottomBorder)
    
        return headerView
    }
    
    //Data
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Return the number of rows in the section.
        if ((appSetting?.isLogin) == true) {
            return (listMainMenuWithLogin?.count)!
        }
        
        return (listMainMenu?.count)!
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCellWithIdentifier(CELL_IDENTIFIER, forIndexPath: indexPath) as! JTMMainMenuTableViewCell
        
//        setSelectedBackgroundView(cell)
        cell.setSelectedBackground()
        
        let itemMainMenu: NSDictionary
        
        if ((appSetting?.isLogin) == true) {
           itemMainMenu = listMainMenuWithLogin![indexPath.row] as! NSDictionary
        } else {
            itemMainMenu = listMainMenu![indexPath.row] as! NSDictionary
        }
        
        let titleKeyword = applicationDelegate.appSetting?.currentLanguage == .THAI ? NAME_TH_KEY : NAME_EN_KEY
        let titleString = itemMainMenu[titleKeyword] as? String
        let imageName = itemMainMenu[ICON_NAME] as? String
        
        cell.setCellData(titleString!, imageName: imageName!)
        cell.separatorInset = UIEdgeInsets(top: 0, left: MENU_CELL_PADDING, bottom: 0, right: 0)
        
        return cell
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        let height: CGFloat = (parentViewHeight - TABLE_HEADER_HEIGHT)/CGFloat((listMainMenuWithLogin?.count)!)
        return round(height)
    }
    
    //MARK:- TableView Delegate
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        //Present new view controller
        //let mainStoryboard: UIStoryboard = UIStoryboard(name: "Main",bundle: nil)
        
        
        var destViewController : UIViewController
        
        let itemMainMenu: NSDictionary
        
        if ((appSetting?.isLogin) == true) {
            itemMainMenu = listMainMenuWithLogin![indexPath.row] as! NSDictionary
        } else {
            itemMainMenu = listMainMenu![indexPath.row] as! NSDictionary
        }
        
        setTrackGoogleAnalyticMenuBar(itemMainMenu[TRACK_LABEL] as! String)
        
        let storyBoardId = (itemMainMenu[IDENTIFY_KEY] as? String)!
        
        if(storyBoardId != "") {
            
            if(indexPath.row == getSelectedIndex()) {
                hideSideMenuView()
                
            } else {
                selectedMenuIdentifier = storyBoardId
                destViewController = getViewController((itemMainMenu[IDENTIFY_KEY] as? String)!)
                sideMenuController(self.parent)?.setContentViewController(destViewController)
                
            }
        } else {
            hideSideMenuView()
            alertLogOut()
        }
        
    }
    
//    func setSelectedBackgroundView(cell: UITableViewCell) {
//        let backgroundView = UIView(frame: cell.frame)
//        backgroundView.backgroundColor = JTM_MENU_HIGHLIGHT_COLOR
//        
//        cell.selectedBackgroundView = backgroundView
//    }
    
    //MARK:- Observer
    func onLanguageChange() {
        self.tableView.reloadData()
        setHilightToSelectedRow()
    }
    
    func loginChange(notification:NSNotification) {
        
        self.tableView.reloadData()
        setHilightToSelectedRow()
        hideSideMenuView()
    }
    
    func alertLogOut() {
        let alert = UIAlertController(title: applicationDelegate.getLocalizeStringForKey("ALERT_CONFIRM_LOGOUT"), message: "", preferredStyle: UIAlertControllerStyle.Alert)
        
        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("CANCEL"), style: .Default, handler: { action in
            self.setHilightToSelectedRow()
        }))
        alert.addAction(UIAlertAction(title: applicationDelegate.getLocalizeStringForKey("TEXT_BUTTON_LOGOUT"), style: .Default, handler: { action in
            let destViewController = self.getViewController(JTM_HOME_VIEWCONTROLLER_IDENTIFIER) as! JTMBaseMenuViewController
            JTMLoginViewController.connectServiceLogout(destViewController, redirectType: .BACK_TO_HOME, completion: nil)
            self.setHilightToSelectedRow()
        }))
        
        alert.view.accessibilityIdentifier = ID_ALERT_LOGOUT
        
        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    func setTrackGoogleAnalyticMenuBar(trackLabel : String) {
        
        var label = ""
        
        switch(trackLabel) {
        case "LABEL_APPLIED_JOBS":
            label = LABEL_APPLIED_JOBS
            break
        case "LABEL_FAVORITE_JOBS":
            label = LABEL_FAVORITE_JOBS
            break
        case "LABEL_VIEWED_BY_COMPANY":
            label = LABEL_VIEWED_BY_COMPANY
            break
        case "LABEL_SETTING":
            label = LABEL_SETTING
            break
        case "LABEL_ABOUT_US":
            label = LABEL_ABOUT_US
            break
        case "LABEL_LOGIN":
            label = LABEL_LOGIN
            break
        case "LABEL_LOGOUT":
            label = LABEL_LOGOUT
            break
        default :
            label = LABEL_HOME
            break
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_MENU_BAR, action: ACTION_CLICK, label: label)
    }
    
    //MARK:- Get slide menu item index
    func getSelectedIndex() -> Int {
        let listMenu: NSMutableArray
        
        if ((appSetting?.isLogin) == true) {
            listMenu = listMainMenuWithLogin!
        } else {
            listMenu = listMainMenu!
        }
        
        for menuItem in listMenu {
            let menuItemDictionary = menuItem as! NSDictionary
            if menuItemDictionary[IDENTIFY_KEY] as! String == selectedMenuIdentifier {
                print("AT: \(listMenu.indexOfObject(menuItem))")
                return listMenu.indexOfObject(menuItem)
            }
        }
        print("AT: 000")
        return 0
    }
    
    func setHilightToSelectedRow() {
        tableView.selectRowAtIndexPath(NSIndexPath(forRow: getSelectedIndex(), inSection: 0), animated: false, scrollPosition: .Middle)
    }
}
