//
//  JTMMainSearchResultViewController.swift
//  JobThai
//
//  Created by weerapon on 2/23/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import Foundation
import UIKit

class JTMMainSearchResultViewController: JTMBaseViewController , JTMSeachResultClickSubViewDelegate, JTMSearchResultStatusDelegate {
    
    var delegate : JTMMainSeachResultDelegate?
    var params: [String:AnyObject]? //= ""
    var isOpenFromSearchMap = false
    var currentPageIndex : SearchResultPageIndex = .GRID_VIEW
    var mapRadius : Double = 0
//    let SEARCH_RESULT_LIST = 0
//    let SEARCH_RESULT_GRID = 1
//    let SEARCH_RESULT_MAP = 2
    let SEARCH_RESULT_NULL_DATA = -1
    let TAB_BAR_SHOW_HEIGHT : CGFloat = 45
    let TAB_BAR_HIDE_HEIGHT : CGFloat = 0
    
    @IBOutlet weak var positionLabel : JTMLabel?
    @IBOutlet weak var tabBarHeight: NSLayoutConstraint!
    @IBOutlet weak var prefixLabel : JTMLabel?
    @IBOutlet weak var listButton : UIButton?
    @IBOutlet weak var gridButton : UIButton?
    @IBOutlet weak var mapButton : UIButton?
    
    
    var noResultView:JTMNoResultView?
    
    
    @IBAction func onClickList() {
        onClickButton(.LIST_VIEW)
//        self.currentPageIndex = .LIST_VIEW
//        delegate?.onClickOpenPageView(currentPageIndex.rawValue)//SEARCH_RESULT_LIST)
//        print("onClickList")
//        // Track Google Analytic Event
//        applicationDelegate.trackGoogleAnalyticEvent(CAT_SEARCH_RESULT, action: ACTION_CLICK, label: LABEL_LIST)
    }
    
    @IBAction func onClickGrid() {
        onClickButton(.GRID_VIEW)
//        self.currentPageIndex = .GRID_VIEW
//        delegate?.onClickOpenPageView(currentPageIndex.rawValue)
//        print("onClickGrid")
//        // Track Google Analytic Event
//        applicationDelegate.trackGoogleAnalyticEvent(CAT_SEARCH_RESULT, action: ACTION_CLICK, label: LABEL_GRID)
    }
    
    @IBAction func onClickMap() {
        onClickButton(.MAP_VIEW)
//        self.currentPageIndex = .MAP_VIEW
//        delegate?.onClickOpenPageView(currentPageIndex.rawValue)
//        print("onClickMap")
//        // Track Google Analytic Event
//        applicationDelegate.trackGoogleAnalyticEvent(CAT_SEARCH_RESULT, action: ACTION_CLICK, label: LABEL_MAP)
    }
    
    func onClickButton(ButtonIndex: SearchResultPageIndex) {
        self.currentPageIndex = ButtonIndex
        delegate?.onClickOpenPageView(currentPageIndex.rawValue)
        
        var analyticLabel: String
        switch ButtonIndex {
            case .LIST_VIEW:
                analyticLabel = LABEL_LIST
                break
            case .GRID_VIEW:
                analyticLabel = LABEL_GRID
                break
            case .MAP_VIEW:
                analyticLabel = LABEL_MAP
                break
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_SEARCH_RESULT, action: ACTION_CLICK, label: analyticLabel)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = applicationDelegate.getLocalizeStringForKey("TITLE_SEARCH_RESULT")
        self.tabBarHeight.constant = TAB_BAR_HIDE_HEIGHT
        
        if self.isOpenFromSearchMap {
            self.onClickMap()
            self.setStatusButtonCurrentSeachResult(SearchResultPageIndex.MAP_VIEW.rawValue)
        } else {
            self.onClickGrid()
        }
        
        //Add Quick Search Button to SearchResultView
        self.initQuickSearchNavigationItemButton()
        
        initNoResultView()
    }
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
//        noResultView?.frame = self.view.bounds
    }
    
    func initNoResultView() {
        // Add No Result View
        var noresultFrame = self.view.bounds
        noresultFrame.size.height = noresultFrame.height - NAVIGATION_BAR_AND_STATUS_BAR_HEIGHT
        self.noResultView = JTMNoResultView(frame: noresultFrame)
        self.view.addSubview(self.noResultView!)
        self.noResultView?.hidden = true
    }
    
    override func setControllerFlags() {
        isNeedToDimOnConnectionError = true
    }
    
    //MARK:- Reachability change update
    override func manageViewAfterInternetReachable() {
        super.manageViewAfterInternetReachable()
        setButtonEnableStatus(true)
    }
    
    override func manageViewAfterInternetNotReachable() {
        super.manageViewAfterInternetNotReachable()
        setButtonEnableStatus(false)
    }
    
    override func initConnectionErrorView() {
        super.initConnectionErrorView()
        connectionErrorView?.alpha = VIEW_HIDE_ALPHA
    }
    
    func setButtonEnableStatus(enable: Bool) {
        listButton?.userInteractionEnabled = enable
        gridButton?.userInteractionEnabled = enable
        mapButton?.userInteractionEnabled = enable
    }
    
    // Mark: send deleagate to pageview
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "JTMSearchResultViewControllerSegue"{
            let searchResultViewController = segue.destinationViewController as? JTMSearchResultViewController
            searchResultViewController?.seachResultMainSeachResult = self
            self.delegate = searchResultViewController
            searchResultViewController?.paramSearch = self.params
            searchResultViewController?.mapRadiusArea = self.mapRadius
            searchResultViewController?.isOpenFromSearchMap = self.isOpenFromSearchMap
        }
    }
    
    func hideSearchMainPosition(senderController: UIViewController) {
        if !(self.isPresentingSenderController(senderController)) {
            return
        }
        
        self.positionLabel?.hidden = true
        self.prefixLabel?.hidden = true
    }
    
    func reloadPageOnSortFilterChange() {
        self.tabBarHeight.constant = TAB_BAR_HIDE_HEIGHT
    }
    
    override func alertActionHandler(alert: UIAlertAction) {
        self.navigationController?.popViewControllerAnimated(true)
    }
    
    
    func setSeachMainPosition(senderController: UIViewController, position: Int) {

        if !(self.isPresentingSenderController(senderController)) {
            return
        }
        
        self.tabBarHeight.constant = TAB_BAR_SHOW_HEIGHT
        self.positionLabel?.hidden = false
        self.prefixLabel?.hidden = false
        prefixLabel?.text = applicationDelegate.getLocalizeStringForKey("PREFIX_POSITION")
        
        if senderController.isKindOfClass(JTMSearchResultMapViewController) {
            let searchResultMapView = senderController as? JTMSearchResultMapViewController
            if(searchResultMapView?.searchResultController.mapSearchResultData != nil) {
                self.positionLabel?.text = applicationDelegate.numFormatter.stringFromNumber((searchResultMapView?.searchResultController.mapSearchResultData?.countAllJob)!)
            } else {
                self.positionLabel?.hidden = true
                self.prefixLabel?.hidden = true
            }
        } else {
            self.positionLabel?.hidden = (position == SEARCH_RESULT_NULL_DATA)
            self.prefixLabel?.hidden = (position == SEARCH_RESULT_NULL_DATA)
            self.positionLabel?.text = applicationDelegate.numFormatter.stringFromNumber(position)
        }
    }
    
    func isPresentingSenderController(senderController: UIViewController) -> Bool {
        if (senderController.isKindOfClass(JTMSearchResultListViewController) && self.currentPageIndex == .LIST_VIEW) {
            return true
        } else if (senderController.isKindOfClass(JTMSearchResultGirdViewController) && self.currentPageIndex == .GRID_VIEW) {
            return true
        } else if (senderController.isKindOfClass(JTMSearchResultMapViewController) && self.currentPageIndex == .MAP_VIEW) {
            return true
        }
        
        return false
    }
    
    func onClickCallOut(viewController: UIViewController, item: NSMutableArray) {
        let jobDetailStoryBoard = UIStoryboard(name: "SearchResult", bundle: nil)
        let jobAllPosition = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMAllPositionMapViewController") as? JTMAllPositionMapViewController
        jobAllPosition?.allPositionMapJobList = item
        
        //jobdetail?.companyCode = item.companyCode!
        self.navigationController?.pushViewController(jobAllPosition!, animated: true)
    }
    
    func onClickSubView(viewController: UIViewController, item: JobItem) {
        let jobDetailStoryBoard = UIStoryboard(name: "JobDetail", bundle: nil)
        let jobdetail = jobDetailStoryBoard.instantiateViewControllerWithIdentifier("JTMJobDetailViewController") as? JTMJobDetailViewController
        jobdetail?.companyCode = item.companyCode!
        if(viewController.isKindOfClass(JTMSearchResultListViewController) || viewController.isKindOfClass(JTMSearchResultGirdViewController)){
            jobdetail?.jobCode = item.runningNumber
            jobdetail?.openPageType = .JOB_INFO_PAGE
        }
        else if(viewController.isKindOfClass(JTMSearchResultMapViewController)){
            jobdetail?.openPageType = .ALL_POSITION_PAGE
        }
        self.navigationController?.pushViewController(jobdetail!, animated: true)
    }
    
    func displayNoResult() {
        self.noResultView?.hidden = false
    }
    
    func updateMapButtonEnable(isMapEnable: Bool) {
        self.mapButton?.enabled = isMapEnable
    }
    
    // Mark: set status button
    func setStatusButtonCurrentSeachResult(currentIndex: NSInteger) {
        self.currentPageIndex = SearchResultPageIndex(rawValue: currentIndex)!
        
        switch (currentPageIndex) {
            case .LIST_VIEW:
                self.listButton?.selected = true
                self.gridButton?.selected = false
                self.mapButton?.selected = false
                break
            case .GRID_VIEW:
                self.listButton?.selected = false
                self.gridButton?.selected = true
                self.mapButton?.selected = false
                break
            case .MAP_VIEW:
                self.listButton?.selected = false
                self.gridButton?.selected = false
                self.mapButton?.selected = true
                break
//            default:
//                break
        }
    }
}

protocol JTMMainSeachResultDelegate {
    func onClickOpenPageView(index : Int)
    func parameterSearch(param: [String:AnyObject])
}

protocol JTMSeachResultClickSubViewDelegate {
    func onClickSubView(viewController: UIViewController, item : JobItem)
    func onClickCallOut(viewController: UIViewController, item : NSMutableArray)
    func setSeachMainPosition(senderViewController:UIViewController, position : Int)
    func hideSearchMainPosition(senderViewController: UIViewController)
    func displayNoResult()
    func updateMapButtonEnable(isMapEnable: Bool)
    func setButtonEnableStatus(enable: Bool)
}
