//
//  JTMMaintenaceViewController.swift
//  JobThai
//
//  Created by pinyo on 6/23/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMMaintenaceViewController : JTMBaseViewController {
    
    @IBOutlet var modeImageView     : UIImageView!
    @IBOutlet var modeDescription   : UITextView!
    
    private var mainteneData        : MaintenanceModel?
    
    private var preloadListData = [PreloadModel]()

    var parent : JTMCustomLaunchScreenViewController?
    
    //Try Auto Login
//    var isTryingAutoLogin = false
    
    //Launch Screen Background
    @IBOutlet var launchScreenBackground    : UIView!
    
    override func viewDidLoad() {
        print("maintene did load")
        initBaseObserver()
    }
    
    override func viewDidAppear(animated: Bool) {
        checkMode((self.mainteneData?.type)!)
    }
    
    override func applicationDidActive() {
        if (applicationDelegate.window?.topMostController() == self) {
            checkMode((self.mainteneData?.type)!)
        }
    }
    
    override func updateViewForReachabilityChange() {
//        print("maintene not do update reachability")
    }
    
    override func onConnectionError(errorRequestUrl: String?) {
        //
    }
    
    //MARK: Set Data
    func setMaintenanceData(data : MaintenanceModel) {
        self.mainteneData = data
    }
    
    //MARK: Check Maintenance Mode
    func checkMode(modeType : MaintenanceType) {
        switch modeType {
        case .MODE_MAINTENE :
            isModeMaintenance()
            break
            
        case .MODE_UPDATE_FORCE :
            isModeUpdateForce()
            break
            
        case .MODE_UPDATE_NORMAL :
            isModeUpdateNormal()
            break
            
        case .MODE_NORMAL :
            isModeNormal()
            break
            
        case .MODE_NO_INTERNET :
            isModeConnectionNoInternet()
            break
            
        case .MODE_CONNECTION_FAIL :
            isModeConnectionFail()
            break
        }
    }
    
    //MARK: Maintenance Action
    func isModeMaintenance() {
        setMainteneContent()
        hideLaunchScreenBackground(true)
    }
    
    func isModeUpdateForce() {
        let alertTitle      = self.mainteneData?.alertTitle     ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_UPDATE_FORCE_TITLE")
        let alertMessage    = self.mainteneData?.alertMessage   ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_UPDATE_FORCE_MESSAGE")
        let alertController = getAlertController(alertTitle, txtMessage: alertMessage)
        alertController.addAction(getAlertActionUpdate())

        self.presentViewController(alertController, animated: true, completion: nil)
    }
    
    func isModeUpdateNormal() {
        let alertTitle      = self.mainteneData?.alertTitle     ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_UPDATE_NORMAL_TITLE")
        let alertMessage    = self.mainteneData?.alertMessage   ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_UPDATE_NORMAL_MESSAGE")
        let alertController = getAlertController(alertTitle, txtMessage: alertMessage)
        alertController.addAction(getAlertActionUpdate())
        alertController.addAction(getAlertActionSkip())
        
        self.presentViewController(alertController, animated: true, completion: nil)
    }
    
    func isModeConnectionNoInternet() {
        print("maintene is no internet")
        let alertTitle      = self.mainteneData?.alertTitle     ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_CONNECTION_NO_INTERNET_TITLE")
        let alertMessage    = self.mainteneData?.alertMessage   ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_CONNECTION_NO_INTERNET_MESSAGE")
        presentTryAgainAlertController(alertTitle, meesage: alertMessage)
    }
    
    func isModeConnectionFail() {
        print("maintene is connection fail")
        let alertTitle      = self.mainteneData?.alertTitle     ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_CONNECTION_FAIL_TITLE")
        let alertMessage    = self.mainteneData?.alertMessage   ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_CONNECTION_FAIL_MESSAGE")
        presentTryAgainAlertController(alertTitle, meesage: alertMessage)
    }
    
    func isModeNormal() {
        skipMaintenanceMode()
    }
    
    //MARK: Alert Controller Action
    func getAlertController(txtTitle : String , txtMessage : String) -> UIAlertController {
        let alertController = UIAlertController(title: txtTitle, message: txtMessage, preferredStyle: UIAlertControllerStyle.Alert)
        return alertController
    }
    func getAlertActionUpdate() -> UIAlertAction {
        let actionTitle     = applicationDelegate.getLocalizeStringForKey("MAINTENE_ACTION_UPDATE")
        let updateAction    = UIAlertAction(title: actionTitle, style: UIAlertActionStyle.Default) { (alertAction) -> Void in
            self.openAppStore()
        }
        return updateAction
    }
    
    func getAlertActionSkip() -> UIAlertAction {
        let actionTitle     = applicationDelegate.getLocalizeStringForKey("MAINTENE_ACTION_SKIP")
        let skipAction    = UIAlertAction(title: actionTitle, style: UIAlertActionStyle.Default) { (alertAction) -> Void in
            self.skipMaintenanceMode()
        }
        return skipAction
    }
    
    func getAlertActionTryAgain() -> UIAlertAction{
        let actionTitle     = applicationDelegate.getLocalizeStringForKey("MAINTENE_ACTION_TRY_AGAIN")
        let tryAgainAction  = UIAlertAction(title: actionTitle, style: UIAlertActionStyle.Default) { (alertAction) -> Void in
            if self.parent != nil {
                self.parent!.reloadMaintenance = true
                self.parent!.isInit = false
            }
            self.dismissViewControllerAnimated(false, completion: nil)
        }
        return tryAgainAction
    }
    
    //MARK: Method for Alert Action
    func openAppStore() {
        if mainteneData?.urlAppStore != nil {
            let url = NSURL(string: (mainteneData?.urlAppStore)!)
            if UIApplication.sharedApplication().canOpenURL(url!) {
                UIApplication.sharedApplication().openURL(url!)
            }
        }
    }
    
    func skipMaintenanceMode() {
        tryAutoLogin()
        initialMapEngine()
        checkPreloadService()
    }
    
    func setMainteneContent() {
        self.modeDescription.text = self.mainteneData?.description ?? applicationDelegate.getLocalizeStringForKey("MAINTENE_DESCRIPTION")
        self.modeDescription.textColor = UIColor.whiteColor()
        
        if let mainteneImageUrl = self.mainteneData?.urlImage {
            if UIApplication.sharedApplication().canOpenURL(NSURL(string: mainteneImageUrl)!) {
                self.modeImageView.sd_setImageWithURL(NSURL(string: mainteneImageUrl)!)
            }
        }
    }
    
    //MARK: Present Preload & MainViewController
    func presentPreLoad(view : UIViewController) {
        let preloadViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMPreloadViewController") as? JTMPreloadViewController
        preloadViewController?.setPreloadData(self.preloadListData)
        delay(0.1) {
            view.presentViewController(preloadViewController!, animated: true, completion: nil)
        }
    }
    
    func presentMainView() {
        let mainViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainViewController") as? JTMMainViewController
        applicationDelegate.window?.rootViewController = mainViewController
    }
    
    func presentTryAgainAlertController(title : String , meesage : String) {
        let alertController = getAlertController(title, txtMessage: meesage)
        alertController.addAction(getAlertActionTryAgain())
        self.presentViewController(alertController, animated: true, completion: nil)
    }
    
    // MARK : Connect Service Preload
    func checkPreloadService() {
        connectServicePreload({ () -> Void in
            if(self.checkPreloadData()){
                if self.checkHasSeenQuickGuide() {
                    self.presentQuickGuide()
                } else {
                    self.presentPreLoad(self)
                }
            }else{
                self.presentMainView()
            }
        })
    }
    
    func connectServicePreload(completion : (()->Void)? ){
        let serviceController = JTMServiceController()
        let url = getServicePreload()
        
        serviceController.requestService(self, urlString: url, params: nil, isCanLoadParallelToOther: true) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonObject = response!.result.value as? NSMutableDictionary
                
                if let preloadObject = jsonObject!["preload"] as? NSArray {
                    self.convertPreloadToJson(preloadObject)
                }
            }
            completion?()
        }
    }
    
    func convertPreloadToJson(jsonObject : NSArray) {
        self.preloadListData = self.convertPreload(jsonObject)
    }
    
    func convertPreload(preloadList : NSArray)->[PreloadModel]{
        var preloadArray = [PreloadModel]()
        
        for itemPreload in preloadList {
            let preloadItem = PreloadModel(item: itemPreload as! NSMutableDictionary)
            preloadArray.append(preloadItem)
        }
        return preloadArray
    }
    
    func checkPreloadData() -> Bool {
        if(preloadListData.count > 0) {
            return true
        }
        return false
    }
    
    //MARK: Try Auto Login
    func tryAutoLogin() {
        let userCredential = applicationDelegate.appSetting?.userCredential
        let isLogin = (applicationDelegate.appSetting?.isLogin)!
        
        if (userCredential?.token != "" && !isLogin && !applicationDelegate.isTryingAutoLogin) {
            
            applicationDelegate.isTryingAutoLogin = true
            
//            JTMLoginViewController.connectServiceLogIn(self,
//                username: (userCredential?.username)!,
//                password: (userCredential?.password)!,
//                redirectType: .AUTO_LOGIN) {
//                    () -> Void in
//                    //                    controller!.pushToMainViewController()
//                    self.isTryingAutoLogin = false
//            }
            
            print(userCredential?.token)
            print(userCredential?.sessionId)
            let param = JTMLoginViewController.getAutoLogInParameter((userCredential?.token)!, sessionId: (userCredential?.sessionId)!)
            
            JTMLoginViewController.connectServiceLogIn(self, parameter: param, redirectType: .AUTO_LOGIN, completion: { () -> Void in
//                controller!.pushToMainViewController()
                NSNotificationCenter.defaultCenter().postNotificationName(JTM_FINISHED_AUTO_LOG_IN, object: nil)
                applicationDelegate.isTryingAutoLogin = false
            })
            
        } else {
            //            controller?.pushToMainViewController()
        }
    }
    
    //MARK: Manage Launch Screen Background
    func hideLaunchScreenBackground(hidden : Bool) {
        launchScreenBackground.hidden   = hidden
    }
    
    //MARK: Manage Map Tile
    func initialMapEngine() {
        //Initial MapView Engine
        TNMapView.initialMapViewEngine { () -> (Void) in
            self.callServiceTileMap()
        }
    }
    
    func callServiceTileMap() {
        let serviceController = JTMServiceController()
        let url = getServiceMapTileSource()
        
        serviceController.requestService(JTMBaseViewController(), urlString: url, params: nil, isCanLoadParallelToOther: false, isPresentLoadingIndicator: false)
            { (response, status) -> Void in
                if (status == .CONNECTION_SUCCESS) {
                    let jsonObject = response!.result.value as? NSMutableDictionary
                    self.parseTileMapJson(jsonObject)
                }
        }
    }
    
    func parseTileMapJson(jsonObject : NSMutableDictionary?) {
        let urlTileSource = jsonObject?.objectForKey("urlTileSource") as? String
        let startMapRange = jsonObject?.objectForKey("startRange") as? Int
        let stopMapRange  = jsonObject?.objectForKey("stopRange") as? Int
        let maxZoom       = jsonObject?.objectForKey("maxZoom") as? Float
        //        let minZoom       = jsonObject?.objectForKey("minZoom") as? Float
        
        if urlTileSource != nil && startMapRange != nil && stopMapRange != nil && maxZoom != nil {
            checkMapTileSourceUrl(urlTileSource, startRange: startMapRange!, stopRange: stopMapRange!, maxZoom: maxZoom!)
        }
    }
    
    func checkMapTileSourceUrl(newUrl : String? , startRange : Int , stopRange : Int , maxZoom : Float , minZoom : Float = 1) {
        let saveUrl = applicationDelegate.appSetting?.getMapTileSourceUrl()
        let (rangeStart , rangeStop) = (applicationDelegate.appSetting?.getMapTileRange())!
        if saveUrl != newUrl || rangeStart != startRange || rangeStop != stopRange {
            TNMapView.clearMapTileCache()
            applicationDelegate.appSetting?.saveMapTileSourceUrl(newUrl!)
            applicationDelegate.appSetting?.saveMapTileRange(startRange, stop: stopRange)
        }
        
        TNMapView.urlTnTileSource   = newUrl
        TNMapView.startMapRange     = startRange
        TNMapView.stopMapRange      = stopRange
        TNMapView.maxZoom           = maxZoom
        //        TNMapView.minZoom           = minZoom
    }
    
    // MARK:- Quick Guide
    func presentQuickGuide() {
        
        let quickGuideViewController = self.storyboard?.instantiateViewControllerWithIdentifier("JTMMainQuickGuideViewController") as? JTMMainQuickGuideViewController
        quickGuideViewController!.maintenanceView = self
        self.presentViewController(quickGuideViewController!, animated: true, completion: nil)
    }
        
    func checkHasSeenQuickGuide() -> Bool {
        
        return !NSUserDefaults.standardUserDefaults().boolForKey(HAS_SEEN_QUICK_GUIDE)
    }

}
