//
//  JTMRegisterView.swift
//  JobThai
//
//  Created by tayathorn on 5/31/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMRegisterView: UIView, UITextFieldDelegate, UITextViewDelegate {

    var view : UIView?
    var delegate : JTMRegisterDelegate?
    
    @IBOutlet weak var emailTextField: JTMTextField!
    @IBOutlet weak var confirmEmailTextField: JTMTextField!
    @IBOutlet weak var passwordTextField: JTMTextField!
    @IBOutlet weak var confirmPasswordTextField: JTMTextField!
    
    @IBOutlet weak var emailHelperLabel: JTMLabel!
    @IBOutlet weak var passwordHelperLabel: JTMLabel!
    
    @IBOutlet weak var conditionTextView: JTMTextView!
    @IBOutlet weak var registerButton: JTMButton!
    
    @IBOutlet weak var bottomLayoutConstraint: NSLayoutConstraint!
    
    @IBAction func onRegisterClick() {
        self.endEditing(true)
        delegate?.onRegisterClick()
    }
    
    
    let CONDITION_LINK_COLOR : UIColor = UIColor(hexString: "#FF5C05")
    let CONDITION_TEXT_COLOR = "#808285"
    
    var textViewFontName : String = "textViewFont"
    var textViewFontSize : CGFloat = 12
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupUI()
        setUILabel()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupUI()
        setUILabel()
    }
    
    func setupUI() {
        view = loadViewFromNib()
        view?.frame = bounds
        view?.autoresizingMask = [.FlexibleWidth, .FlexibleHeight]
        addSubview(view!)
        
        emailHelperLabel.hidden = true
        passwordHelperLabel.hidden = true
        
        emailTextField.delegate = self
        confirmEmailTextField.delegate = self
        passwordTextField.delegate = self
        confirmPasswordTextField.delegate = self
        
        conditionTextView.delegate = self
    }
    
    func loadViewFromNib() -> UIView {
        let bundle = NSBundle(forClass:self.dynamicType)
        let nib = UINib(nibName: "JTMRegisterView", bundle: bundle)
        let view =  nib.instantiateWithOwner(self, options: nil)[0] as! UIView
        return view
    }
    
    func setUILabel() {
        // Textfielf Placeholder
        emailTextField.placeholder = applicationDelegate.getLocalizeStringForKey("EMAIL_PLACEHOLDER")
        confirmEmailTextField.placeholder = applicationDelegate.getLocalizeStringForKey("CONFIRM_EMAIL_PLACEHOLDER")
        passwordTextField.placeholder = applicationDelegate.getLocalizeStringForKey("PASSWORD_PLACEHOLDER")
        confirmPasswordTextField.placeholder = applicationDelegate.getLocalizeStringForKey("CONFIRM_PASSWORD_PLACEHOLDER")
        
        // Helper
        emailHelperLabel.text = applicationDelegate.getLocalizeStringForKey("HELPER_EMAIL")
        passwordHelperLabel.text = applicationDelegate.getLocalizeStringForKey("HELPER_PASSWORD")
        
        //Button
        registerButton.setTitle(applicationDelegate.getLocalizeStringForKey("TEXT_TITLE_REGISTER"), forState: .Normal)
        
        
        // TextView
        textViewFontName = self.conditionTextView.font!.fontName
        textViewFontSize = (self.conditionTextView.font?.pointSize)!
        
        var htmlText = applicationDelegate.getLocalizeStringForKey("REGISTER_TERMS_HTML")
        let lang = (applicationDelegate.appSetting?.currentLanguage == appLanguage.THAI) ? "th" : "en"
        let termLink = "http://test1-v2.jobthai.com/storeresume/detail_terms.php?l=\(lang)"
       
        //  2 Replace String : 1.TEXT_COLOR     2.LINK_CONDITION
        htmlText = (htmlText as NSString).stringByReplacingOccurrencesOfString("@TEXT_COLOR", withString: CONDITION_TEXT_COLOR).stringByReplacingOccurrencesOfString("@LINK_CONDITION", withString: termLink)
     
        conditionTextView.linkTextAttributes = [NSForegroundColorAttributeName: CONDITION_LINK_COLOR]
        conditionTextView.attributedText =  htmlText.utf8Data?.attributedString
        conditionTextView.font = UIFont(name: textViewFontName, size: textViewFontSize)
        conditionTextView.editable = false;
    }
    
    // MARK :- TextField Delegate
    func textFieldDidBeginEditing(textField: UITextField) {
        
        if textField == emailTextField {
            emailHelperLabel.hidden = false
        } else if textField == passwordTextField {
            passwordHelperLabel.hidden = false
        }
        
    }
    
    func textFieldDidEndEditing(textField: UITextField) {
        if textField == emailTextField {
            self.trimSpace(self.emailTextField)
        } else if textField == confirmEmailTextField {
            self.trimSpace(self.confirmEmailTextField)
        }
        
    }
    
    func trimSpace(textField : UITextField) {
        textField.text = textField.text!.stringByTrimmingCharactersInSet(
            NSCharacterSet.whitespaceAndNewlineCharacterSet()
        )
    }
    
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        let nextTag: NSInteger = textField.tag + 1
        
        if let nextResponder: UIResponder! = textField.superview!.viewWithTag(nextTag){
            nextResponder.becomeFirstResponder()
        }else {
            textField.resignFirstResponder()
            self.onRegisterClick()
        }
        
        return false
    }
    
    // MARK :- Textview Delegate
    func textView(textView: UITextView, shouldInteractWithURL URL: NSURL, inRange characterRange: NSRange) -> Bool {
        onTermClick(URL)
        return false
    }
    
    func textViewDidChangeSelection(textView: UITextView) {
        var range = NSRange()
        range.location = 0
        range.length = 0
        textView.selectedRange = range
    }
    
    func onTermClick(URL: NSURL) {
        delegate?.onTermClick(URL)
    }
    
}

protocol JTMRegisterDelegate {
    func onRegisterClick()
    func onTermClick(URL: NSURL)
}


// MARK :- For TextView -> Text HTML

extension NSData {
    var attributedString: NSAttributedString? {
        do {
            return try NSAttributedString(data: self, options:[NSDocumentTypeDocumentAttribute:NSHTMLTextDocumentType, NSCharacterEncodingDocumentAttribute: NSUTF8StringEncoding], documentAttributes: nil)
        } catch let error as NSError {
            print(error.localizedDescription)
        }
        return nil
    }
}
extension String {
    var utf8Data: NSData? {
        return dataUsingEncoding(NSUTF8StringEncoding)
    }
}

class CustomUITextView: UITextView {
    override func canPerformAction(action: Selector, withSender sender: AnyObject?) -> Bool {
        return false
    }
    
//    func text
}