//
//  JTMRegisterViewController.swift
//  JobThai
//
//  Created by tayathorn on 5/31/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import IQKeyboardManagerSwift

class JTMRegisterViewController: JTMBaseViewController, JTMRegisterDelegate {
 
    @IBOutlet weak var registerView: JTMRegisterView!
    
    let FOCUS_MODE = ".FOCUS_MODE"
    let BACK_HOME = ".BACK_HOME"
    let BACK_VIEW_CONTROLLER = ".BACK_VIEW_CONTROLLER"
    
    var parentToBack : UIViewController?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = applicationDelegate.getLocalizeStringForKey("TEXT_TITLE_REGISTER")
        registerView?.delegate = self
        
        //Looks for single or multiple taps. -> Create for dismiss keyboard
        let tap: UITapGestureRecognizer = UITapGestureRecognizer(target: self, action: #selector(DismissKeyboard))
        view.addGestureRecognizer(tap)
        
        // Set distance between keyboard and textField
        IQKeyboardManager.sharedManager().keyboardDistanceFromTextField = 40
    }
    
    override func setControllerFlags() {
        isHasRetryButtonOnAlertConnectionError = false
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_REGISTER)
        applicationDelegate.trackGoogleAnalyticEvent(CAT_REGISTER, action: ACTION_VIEW, label: LABEL_VIEW_REGISTER)
    }
    
    //Calls this function when the tap is recognized.
    func DismissKeyboard(){
        //Causes the view (or one of its embedded text fields) to resign the first responder status.
        view.endEditing(true)
    }
    
    func onRegisterClick() {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_REGISTER, action: ACTION_CLICK, label: LABEL_REGISTER)
        
        if (applicationDelegate.appSetting?.isNetworkReachable)! == false {
            return
        }
        
        checkDataInTextField()
        print("onRegisterClick OUT")
    }
    
    func onTermClick(URL: NSURL) {
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_REGISTER, action: ACTION_CLICK, label: LABEL_TERM_AND_CONDITIONS)
        
        print("shouldInteractWithURL URL : \(URL)")
        
        let webView = self.storyboard?.instantiateViewControllerWithIdentifier("JTMWebViewController") as? JTMWebViewController
        webView?.webTitle =  applicationDelegate.getLocalizeStringForKey("TERM_TEXT_TITLE")
        webView?.urlString = "\(URL)"
        
        self.navigationController?.pushViewController(webView!, animated: true)
        
    }
    
    func checkDataInTextField() -> Bool {
        print("checkDataInTextField")
        var checkError = false
        var titleError = ""
        var messageError = ""
        var labelKey : UITextField?
        
        if (self.registerView.emailTextField.text == "") {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_EMAIL")
            checkError = true
            labelKey = (registerView?.emailTextField)!
            
        } else if (self.registerView.confirmEmailTextField.text == "") {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_CONFIRM_EMAIL")
            checkError = true
            labelKey = (registerView?.confirmEmailTextField)!
            
        } else if (self.registerView.passwordTextField.text == "") {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS")
            checkError = true
            labelKey = (registerView?.passwordTextField)!
            
        } else if (self.registerView.confirmPasswordTextField.text == "") {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_CONFIRM_PASS")
            checkError = true
            labelKey = (registerView?.confirmPasswordTextField)!
            
        } else if !(self.registerView?.emailTextField?.isValidEmailFormat())! {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_EMAIL_FORMAT")
            messageError = applicationDelegate.getLocalizeStringForKey("MESSAGE_EMAIL_FORMAT")
            checkError = true
            labelKey = (registerView?.emailTextField)!
            
        } else if !isEmailMatch() {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_EMAIL_NOT_MATCH")
            checkError = true
            labelKey = (registerView?.confirmEmailTextField)!
            
        } else if !isPasswordMatch() {
            titleError = applicationDelegate.getLocalizeStringForKey("ERROR_PASS_NOT_MATCH")
            checkError = true
            labelKey = (registerView?.confirmPasswordTextField)!
            
        }
        
        
        // Pop Alert Message
        if checkError {
            self.showAlertMessage(titleError, alertMessage: messageError, labelKey: labelKey!, type: self.FOCUS_MODE)
            
        } else {
            callService()
        }
        
        return checkError
    }
    
    // MARK :- Check Error
    func isEmailMatch() -> Bool {
        
        let email = self.registerView?.emailTextField?.text
        let confirmEmail = self.registerView?.confirmEmailTextField?.text
        
        if email == confirmEmail {
            return true
        }
        
        return false
    }
    
    func isPasswordMatch() -> Bool {
        
        let password = self.registerView?.passwordTextField?.text
        let confirmPassword = self.registerView?.confirmPasswordTextField?.text
        
        if password == confirmPassword {
            return true
        }
        
        return false
    }
    
    func callService() {
        
        let email = self.registerView.emailTextField.text
        let confirmEmail = self.registerView.confirmEmailTextField.text
        let password = self.registerView.passwordTextField.text
        let confirmPassword = self.registerView.confirmPasswordTextField.text
        
        var labelKey = (self.registerView?.emailTextField)
        
        let serviceController = JTMServiceController()
        let url = getServiceRegister()
        let params = [
            "email" : email!,
            "confirmEmail" : confirmEmail!,
            "password" : password!,
            "confirmPassword" : confirmPassword!,
            "imei" : applicationDelegate.getDeviceUUID()
        ]
        
        serviceController.requestService(self, urlString: url, params: params) { (response, status) -> Void in
            if (status == .CONNECTION_SUCCESS) {
                let jsonResult = response!.result.value
                let status = jsonResult!["status"] as! String
                
                if status == "error" {
                    let errCode = jsonResult!["errorcode"] as! String
                    let titleMessage = (jsonResult!["titlemsg"] as? String == nil) ? "" : jsonResult!["titlemsg"] as! String
                    let alertMessage = (jsonResult!["errormsg"] as? String == nil) ? "" : jsonResult!["errormsg"] as! String
                    
                    if errCode == INVALID_EMAIL_DOMAIN_EXAMPLE
                        || errCode == INVALID_EMAIL_DOMAIN_JOBTHAI
                        || errCode == EMAIL_ALREADY_IN_SYSTEM {
                        labelKey = (self.registerView?.emailTextField)!
                        
                    } else {
                        labelKey = (self.registerView?.passwordTextField)!
                    }
                    
                    self.showAlertMessage(titleMessage, alertMessage: alertMessage, labelKey: labelKey!, type: self.FOCUS_MODE)
                    
                } else if status == "ok" {
                    // Register Success
                    let titleMessage = jsonResult!["titlemsg"] as! String
                    let alertMessage = jsonResult!["errormsg"] as! String
                    var backType : String = self.BACK_HOME
                    if self.parentToBack != nil {
                        backType = self.BACK_VIEW_CONTROLLER
                    }
                    self.showAlertMessage(titleMessage, alertMessage: alertMessage, labelKey: labelKey!, type: backType)
                }
            }
        }
    }
   
    func showAlertMessage(titleMessage : String, alertMessage : String, labelKey : UITextField, type : String) {
        
        var message : String!
        message = (alertMessage == "") ? nil : alertMessage
        
        self.showAlertWithData(
            titleMessage,
            message: message,
            completionHandler:  { (UIAlertAction) -> Void in
                switch(type) {
                case self.FOCUS_MODE :
                    labelKey.becomeFirstResponder()
                    break
                case self.BACK_HOME :
                    NSNotificationCenter.defaultCenter().postNotificationName(JTM_BACK_TO_HOME_VIEWCONTROLLER, object: nil)
                    break
                case self.BACK_VIEW_CONTROLLER :
                    self.navigationController?.popToViewController(self.parentToBack!, animated: true)
                    break
                default :
                    labelKey.becomeFirstResponder()
                    break
                }
            },
            actionList: nil
        )
        
        
    }
    
}
