//
//  JTMSearchFilterCategoryViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/19/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMSearchFilterCategoryViewController: JTMBaseViewController, UITableViewDataSource, UITableViewDelegate {
    
    let CELL_HEIGHT:CGFloat = 50
    
    @IBOutlet var tableView : UITableView!
    var searchFilterModel = [SearchFilterCategory]()
    var parent: JTMBaseViewController?
    var searchFilterCategoryType: SearchFilterCategoryType?
    var searchFilterTransport : SearchFilterCategoryTransportation?
    
    var selectedFilter: SearchFilterCategory?
    var selectedSubFilter: SearchFilterSubCategory?
    var indexSelector: Int = 0
    var indexSubSelector: Int = -1
    
    let QUICK_SEARCH = "JTMQuickSearchViewController"
    let NEAR_ME_SEARCH = "JTMJobNearMeViewController"
    let TRANSPORT_SEARCH = "JTMTransportationSearchViewController"
    let INDUSTRIAL_SEARCH = "JTMIndustrialSearchViewController"
    
    var saveDelegate : JTMUpdateSaveStateDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        print("JTMSearchFilterCategoryViewController")
        
        self.tableView.tableFooterView = UIView()
        self.tableView.accessibilityIdentifier = ID_SEARCH_FILTER_JOB_CATEGORIES_TABLE_VIEWS
        setNumberIndexSelector()
        self.noInternetView?.hidden = true
    }
    
    func setSearchFilterModel(searchFilterModel: [SearchFilterCategory]) {
        self.searchFilterModel = searchFilterModel
    }
    
    func setTitleName(title: String) {
        self.title = title
    }
    
//    func setNumberIndexSelector(indexSelector: Int) {
//        self.indexSelector = indexSelector
//    }
//    
//    func setNumberIndexSubSelector(indexSubSelector: Int) {
//        self.indexSubSelector = indexSubSelector
//    }
    
    func setSearchFilterTransportModel(searchFilterItem : SearchFilterCategoryTransportation?){
        self.searchFilterTransport = searchFilterItem
    }
    
    func setSelectedFilter(selectedFilter: SearchFilterCategory?) {
        self.selectedFilter = selectedFilter
    }
    
    func setSelectedSubFilter(selectedSubFilter: SearchFilterSubCategory?) {
        self.selectedSubFilter = selectedSubFilter
    }
    
    func setNumberIndexSelector() {
        if let selectedIndex = searchFilterModel.indexOf({$0.code! == selectedFilter?.code}) {
            indexSelector = selectedIndex
            
            if let selectedSubIndex = searchFilterModel[selectedIndex].subCategoryList.indexOf({$0.code! == selectedSubFilter?.code}) {
                indexSubSelector = selectedSubIndex
            }

            dispatch_async(dispatch_get_main_queue(), { () -> Void in
                self.tableView.scrollToRowAtIndexPath(NSIndexPath(forRow: selectedIndex, inSection: 0), atScrollPosition: .Top, animated: false)
            })
        }
    }
    
    func setFilterCategoryType(searchFilterCategoryType: SearchFilterCategoryType) {
        self.searchFilterCategoryType = searchFilterCategoryType
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    

    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return CELL_HEIGHT
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchFilterCategoryViewController")
        
        let searchFilter = searchFilterModel[indexPath.row]
        
        setSelectedBackgroundView(cell!)
        
        if (self.searchFilterCategoryType == .INDUSTRIALESTATE
            && searchFilter.type == "province") {
            cell?.textLabel?.text = "\t"+searchFilter.name!
        }else {
            cell?.textLabel?.text = searchFilter.name
        }
        
        
        
        if cell!.respondsToSelector(Selector("setSeparatorInset:")) {
            cell!.separatorInset = UIEdgeInsetsZero
        }
        if cell!.respondsToSelector(Selector("setLayoutMargins:")) {
            cell!.layoutMargins = UIEdgeInsetsZero
        }
        if cell!.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell!.preservesSuperviewLayoutMargins = false
        }
        
        if(searchFilter.hasSub == true) {
            cell?.accessoryType = .DisclosureIndicator
        } else {
           cell?.accessoryType = .None
        }
        
       
        if(indexSelector == indexPath.row) {
            cell?.textLabel?.textColor = JTM_CORE_ORANGE
//            cell?.textLabel?.textColor = UIColor.blackColor()
        } else {
            cell?.textLabel?.textColor = UIColor.blackColor()
        }
        

        
        return cell!
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchFilterModel.count
    }
    
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
        
        let searchFilter = searchFilterModel[indexPath.row]
        
        print("Select  : \(searchFilter.name!) Has sub : \(searchFilter.hasSub)")
        
        
        // Track Google Analytic Event
        setTrackGoogleAnalytic(searchFilterCategoryType!, name: searchFilter.name!)
        
        if(searchFilter.hasSub == true) {
            
            let searchFilterSubCategory : JTMSearchFilterSubCategoryViewController = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMSearchFilterSubCategoryViewController") as? JTMSearchFilterSubCategoryViewController)!
            
            searchFilterSubCategory.parent = parent
            searchFilterSubCategory.parentBeforeView = self
            searchFilterSubCategory.saveDelegate = saveDelegate
            searchFilterSubCategory.setFilterCategoryType(searchFilterCategoryType!)
            
            searchFilterSubCategory.setNumberIndexSelector(indexPath.row)
            if(indexPath.row == indexSelector) {
                searchFilterSubCategory.setNumberIndexSubSelector(indexSubSelector)
            } else {
                searchFilterSubCategory.setNumberIndexSubSelector(-1)
            }
            
            searchFilterSubCategory.setTitleName(searchFilter.name!)
            searchFilterSubCategory.setSearchFilterSubModel(searchFilter, searchFilterSubModel: searchFilter.subCategoryList)
            
            if(searchFilterCategoryType == .TRANSPORTATION && searchFilterTransport != nil){
                searchFilterSubCategory.setSearchFilterTransportModel(searchFilterTransport)
            }
            
            self.navigationController?.pushViewController(searchFilterSubCategory, animated: true)
        } else {
            if(searchFilterCategoryType == SearchFilterCategoryType.CATEGORY) {
                saveDelegate?.saveStateupdateSelectedJobCategory?(searchFilter)
                
            } else if(searchFilterCategoryType == SearchFilterCategoryType.LOCATION) {
                saveDelegate?.saveStateupdateSelectedJobLocation?(searchFilter)
                
            } else if(searchFilterCategoryType == SearchFilterCategoryType.TRANSPORTATION) {
                saveDelegate?.saveStateupdateSelectedTransportation?(searchFilterTransport)
                saveDelegate?.saveStateupdateSelectedTransportation?(searchFilter)
                
            } else if(searchFilterCategoryType == SearchFilterCategoryType.INDUSTRIALESTATE) {
                saveDelegate?.saveStateupdateSelectedIndustrialEstate?(searchFilter)
            }
            
            if let transportationView = parent as? JTMSearchFilterCategoryTransportationViewController {
                self.navigationController?.popToViewController(transportationView.parent!, animated: true)
            } else {
                self.navigationController?.popViewControllerAnimated(true)
            }
        }
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
    func setTrackGoogleAnalytic(searchType : SearchFilterCategoryType, name : String) {
        
        let trackCat = checkParentTypeForGoogleAnalytic()
        
        switch(searchType) {
            case .CATEGORY :
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_JOB_CATEGORY, label: getLabelNameJobCate(name))
                break
            case .LOCATION :
                print("Location : \(name)")
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_LOCATION, label: getLabelNameLocation(name))
                break
            case .TRANSPORTATION :
                print("[Main] : \(self.title) , [cate] : \(name)")
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_TRANSPORT, label: getLabelRouteWithStationLevel1(self.title!, station : name))
                break
            case .INDUSTRIALESTATE :
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_INDUSTRAIL, label: getLabelProvince(name))
                break
        }
    }
    
    func checkParentTypeForGoogleAnalytic() -> String {
        let parentType = "\((self.parent?.classForCoder)!)"
        print(parentType)
        
        var trackCat = ""
        
        switch(parentType) {
        case QUICK_SEARCH :
            trackCat = CAT_JOB_SEARCH
            break
        case NEAR_ME_SEARCH :
            trackCat = CAT_JOB_NEAR_ME
            break
        case TRANSPORT_SEARCH :
            trackCat = CAT_JOB_SEARCH_BY_TRANSPORTATION
            break
        case INDUSTRIAL_SEARCH :
            trackCat = CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE
            break
        default :
            break
        }
        
        return trackCat
    }
    
}
