//
//  JTMSearchFilterSubCategoryViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/19/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMSearchFilterSubCategoryViewController: JTMBaseViewController, UITableViewDataSource, UITableViewDelegate {
    
    let CELL_HEIGHT:CGFloat = 50
    
    var searchFilterModelTransport : SearchFilterCategoryTransportation?
    var searchFilterModel: SearchFilterCategory?
    var searchFilterSubModel = [SearchFilterSubCategory]()
    
    var searchFilterCategoryType: SearchFilterCategoryType?
    
    var indexSelector: Int =  -1
    var indexSubSelector: Int = -1
    
    var parent: JTMBaseViewController?
    var parentBeforeView : JTMBaseViewController?
    
    let QUICK_SEARCH = "JTMQuickSearchViewController"
    let NEAR_ME_SEARCH = "JTMJobNearMeViewController"
    let TRANSPORT_SEARCH = "JTMTransportationSearchViewController"
    let INDUSTRIAL_SEARCH = "JTMIndustrialSearchViewController"
    
    var saveDelegate : JTMUpdateSaveStateDelegate?
    
    @IBOutlet var tableView : UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.tableView.tableFooterView = UIView()
        self.tableView.accessibilityIdentifier = ID_SEARCH_FILTER_JOB_CATEGORIES_SUB_TABLE_VIEWS
        setNumberIndexSelector()
        self.noInternetView?.hidden = true
    }
    
    func setNumberIndexSelector(){
        if(self.indexSubSelector >= 0){
            dispatch_async(dispatch_get_main_queue(), { () -> Void in
                self.tableView.scrollToRowAtIndexPath(NSIndexPath(forRow: self.indexSubSelector , inSection: 0), atScrollPosition: .Top, animated: false)
            })
        }
    }
    
    func setSearchFilterSubModel(searchFilterModel: SearchFilterCategory, searchFilterSubModel: [SearchFilterSubCategory]) {
        self.searchFilterModel = searchFilterModel
        self.searchFilterSubModel = searchFilterSubModel
    }
    
    func setSearchFilterTransportModel(searchFilterItem : SearchFilterCategoryTransportation?){
        self.searchFilterModelTransport = searchFilterItem
    }
    
    func setTitleName(title: String) {
        self.title = title
    }
    
    func setNumberIndexSelector(indexSelector: Int) {
        self.indexSelector = indexSelector
    }

    func setNumberIndexSubSelector(indexSubSelector: Int) {
        self.indexSubSelector = indexSubSelector
    }

       
    func setFilterCategoryType(searchFilterCategoryType: SearchFilterCategoryType) {
        self.searchFilterCategoryType = searchFilterCategoryType
    }

    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return CELL_HEIGHT
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchFilterSubCategoryViewController")
        
        let searchFilterSub = searchFilterSubModel[indexPath.row]
        
        setSelectedBackgroundView(cell!)
        
        cell?.textLabel?.text = searchFilterSub.name
        
        if cell!.respondsToSelector(Selector("setSeparatorInset:")) {
            cell!.separatorInset = UIEdgeInsetsZero
        }
        if cell!.respondsToSelector(Selector("setLayoutMargins:")) {
            cell!.layoutMargins = UIEdgeInsetsZero
        }
        if cell!.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell!.preservesSuperviewLayoutMargins = false
        }
        
        if(indexSubSelector == indexPath.row) {
            cell?.textLabel?.textColor = JTM_CORE_ORANGE
//            cell?.textLabel?.textColor = UIColor.blackColor()
        } else {
            cell?.textLabel?.textColor = UIColor.blackColor()
        }
        
        return cell!
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchFilterSubModel.count
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
        

        let searchFilterSub = searchFilterSubModel[indexPath.row]
        print("\(searchFilterSub.name) : \(searchFilterSub.code)")
        
        // Track Google Analytic Event
        setTrackGoogleAnalytic(searchFilterCategoryType!, subName: searchFilterSub.name!)
        
        if(searchFilterCategoryType == SearchFilterCategoryType.CATEGORY) {

            // Track Google Analytic Event
//            applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH, action: ACTION_CLICK_JOB_CATEGORY, label: getLabelNameJobCateWithSub(self.title!, subJobCate : searchFilterSub.name!))
            
            saveDelegate?.saveStateupdateSelectedJobCategory?(searchFilterModel)
            saveDelegate?.saveStateupdateSelectedJobCategory?(searchFilterSub)
            
        } else if(searchFilterCategoryType == SearchFilterCategoryType.LOCATION) {
            
            //Delegate Save State
            saveDelegate?.saveStateupdateSelectedJobLocation?(searchFilterModel)
            saveDelegate?.saveStateupdateSelectedJobLocation?(searchFilterSub)
            
            // Track Google Analytic Event
//            applicationDelegate.trackGoogleAnalyticEvent(CAT_JOB_SEARCH, action: ACTION_CLICK_LOCATION, label: getLabelNameLocationWithSub(self.title!, subLocation : searchFilterSub.name!))
            
        } else if(searchFilterCategoryType == SearchFilterCategoryType.TRANSPORTATION) {
            saveDelegate?.saveStateupdateSelectedTransportation?(searchFilterModelTransport)
            saveDelegate?.saveStateupdateSelectedTransportation?(searchFilterModel)
            saveDelegate?.saveStateupdateSelectedTransportation?(searchFilterSub)
    
        } else if(searchFilterCategoryType == SearchFilterCategoryType.INDUSTRIALESTATE) {
            saveDelegate?.saveStateupdateSelectedIndustrialEstate?(searchFilterModel)
            saveDelegate?.saveStateupdateSelectedIndustrialEstate?(searchFilterSub)
            
        }
        
        if let transportationView = parent as? JTMSearchFilterCategoryTransportationViewController {
            self.navigationController?.popToViewController(transportationView.parent!, animated: true)
        } else {
            self.navigationController?.popToViewController(parent!, animated: true)
        }
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
    func setTrackGoogleAnalytic(searchType : SearchFilterCategoryType, subName : String) {
        
        let trackCat = checkParentTypeForGoogleAnalytic()
        
        switch(searchType) {
        case .CATEGORY :
            applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_JOB_CATEGORY, label: getLabelNameJobCateWithSub(self.title!, subJobCate : subName))
            break
        case .LOCATION :
            applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_LOCATION, label: getLabelNameLocationWithSub(self.title!, subLocation : subName))
            break
        case .TRANSPORTATION :
//            applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_TRANSPORT, label: getLabelRouteWithStationLevel1(self.title!, station : subName))
            if (self.parentBeforeView?.isKindOfClass(JTMSearchFilterCategoryTransportationViewController))! {
                print("2 level")
                print("Sub --->>> [Main] : \(self.title) , [sub] : \(subName)")
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_TRANSPORT, label: getLabelRouteWithStationLevel1(self.title!, station : subName))
            } else {
                print("3 level")
                print("Sub --->>> [Main] : \(parentBeforeView?.title) , [cate] : \(self.title) , [sub] : \(subName)")
                applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_TRANSPORT, label: getLabelRouteWithStationLevel2((parentBeforeView?.title)!, line: self.title!, station : subName))
            }
            break
        case .INDUSTRIALESTATE :
            applicationDelegate.trackGoogleAnalyticEvent(trackCat, action: ACTION_CLICK_INDUSTRAIL, label: getLabelProvinceWithIndustrial(self.title!, industrial : subName))
            break
        }
    }
    
    func checkParentTypeForGoogleAnalytic() -> String {
        let parentType = "\((self.parent?.classForCoder)!)"
        print(parentType)
        
        var trackCat = ""
        
        switch(parentType) {
        case QUICK_SEARCH :
            trackCat = CAT_JOB_SEARCH
            break
        case NEAR_ME_SEARCH :
            trackCat = CAT_JOB_NEAR_ME
            break
        case TRANSPORT_SEARCH :
            trackCat = CAT_JOB_SEARCH_BY_TRANSPORTATION
            break
        case "JTMSearchFilterCategoryTransportationViewController" :
            trackCat = CAT_JOB_SEARCH_BY_TRANSPORTATION
            break
        case INDUSTRIAL_SEARCH :
            trackCat = CAT_JOB_SEARCH_BY_INDUSTRIAL_ESTATE
            break
        default :
            break
        }
        
        return trackCat
    }
    
}
