//
//  JTMSearchFilterViewController.swift
//  JobThai
//
//  Created by ForAlL on 2/25/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit

class JTMSearchFilterTypeViewController: JTMBaseViewController, UITableViewDelegate, UITableViewDataSource {
    
    enum searchTypeIndex: Int {
        case QUICK_SEARCH = 0
        case NEAR_ME
        case TRANSPORTATION
        case INDUSTRIALESTATE
    }

    @IBOutlet weak var tableView: UITableView?
    
    var listSearchFilter: NSMutableArray?
    var searchFilterSaveState:  SearchFilterSaveStateModel?
    var quickSearch : JTMQuickSearchViewController?
    var industrialEstate : JTMIndustrialSearchViewController?
    var transportation : JTMTransportationSearchViewController?
    var jobNearMe : JTMJobNearMeViewController?
    var isUpdateJobCategory : Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        searchFilterSaveState = SearchFilterSaveStateModel()
        onLanguageChange()
 
        self.tableView?.registerNib(UINib(nibName: "JTMSearchFilterTypeTableViewCell", bundle: nil), forCellReuseIdentifier: "JTMSearchFilterTypeTableViewCell")
        self.tableView?.accessibilityIdentifier = ID_SEARCH_FILTER_TYPE_TABLE_VIEWS
        loadMenuSearchFilter()
        
        // FOR UI TEST
        self.navigationController?.navigationBar.accessibilityIdentifier = ID_NAV_BAR_QUICK_SEARCH
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        print("SearchFilterType  viewWillAppear")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_MAIN_SEARCH)
    }

    func loadMenuSearchFilter() {
        if let path = NSBundle.mainBundle().pathForResource("ListSearchFilter", ofType: "plist") {
            listSearchFilter = NSMutableArray(contentsOfFile: path)
        }
    }
    
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("Search")
        applicationDelegate.appSetting?.removeAllSaveFilterList()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }

    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("JTMSearchFilterTypeTableViewCell") as! JTMSearchFilterTypeTableViewCell
        
        if cell.respondsToSelector(Selector("setSeparatorInset:")) {
            cell.separatorInset = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setLayoutMargins:")) {
            cell.layoutMargins = UIEdgeInsetsZero
        }
        if cell.respondsToSelector(Selector("setPreservesSuperviewLayoutMargins:")) {
            cell.preservesSuperviewLayoutMargins = false
        }
        
        let searchMenuFilter =  listSearchFilter![indexPath.row] as! NSMutableDictionary
        
        let iconName = searchMenuFilter.objectForKey("icon") as! String
//        let iconNameHighlight = searchMenuFilter.objectForKey("icon_highlight") as! String
        let title = applicationDelegate.getLocalizeStringForKey(searchMenuFilter.objectForKey("title") as! String)
        let subTitle = applicationDelegate.getLocalizeStringForKey(searchMenuFilter.objectForKey("subTitle") as! String)
        
        cell.setMenuSearchFilter(iconName, title: title, subTitle: subTitle)
//        cell.setIconHighlightedImage(iconNameHighlight)
        return cell
    }
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return (self.tableView?.frame.height)! / 4
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        print(indexPath.row)
        print("searchTypeIndex  :: \((searchTypeIndex(rawValue: indexPath.row)!))")
        
        var searchType = ""
        
        switch (searchTypeIndex(rawValue: indexPath.row)!) {
            case .QUICK_SEARCH:
                searchType = SEARCH_TYPE_JOB_SEARCH
                openQuickSearch()
                break
            case .NEAR_ME:
                searchType = SEARCH_TYPE_JOB_NEAR_ME
                openJobNearMe()
                break
            case .TRANSPORTATION:
                searchType = SEARCH_TYPE_JOB_SEARCH_BY_TRANSPORTATION
                openTransportation()
                break
            case .INDUSTRIALESTATE:
                searchType = SEARCH_TYPE_JOB_SEARCH_BY_INDUSTRIAL_ESTATE
                openIndustrialEstate()
                break
        }
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_MAIN_SEARCH, action: ACTION_CLICK, label: getLabelNameSearch(searchType))
       
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return listSearchFilter!.count
    }
    
    func openQuickSearch() {
        print("openQuickSearch")
        
        if  quickSearch == nil {
            quickSearch = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMQuickSearchViewController") as? JTMQuickSearchViewController)!
        }
        quickSearch?.parentController = self
        
        self.navigationController?.pushViewController(quickSearch!, animated: true)
    }
    
    func openIndustrialEstate() {
        print("openIndustrialEstate")
        
        if industrialEstate == nil {
            industrialEstate = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMIndustrialSearchViewController") as? JTMIndustrialSearchViewController)!
        }
        industrialEstate?.parentController = self
        
        self.navigationController?.pushViewController(industrialEstate!, animated: true)
    }
    
    func openTransportation() {
        print("openTransportation")
        
        if transportation == nil {
            transportation = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMTransportationSearchViewController") as? JTMTransportationSearchViewController)!
        }
        transportation?.parentController = self
        
        self.navigationController?.pushViewController(transportation!, animated: true)
    }
    
    func openJobNearMe() {
        print("openJobNearMe")
        
        if jobNearMe == nil {
            jobNearMe = (self.storyboard?.instantiateViewControllerWithIdentifier("JTMJobNearMeViewController") as? JTMJobNearMeViewController)!
        }
        jobNearMe?.parentController = self
        
        self.navigationController?.pushViewController(jobNearMe!, animated: true)
    }

}
