//
//  JTMServiceController.swift
//  JobThai
//
//  Created by kanya on 6/9/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import Alamofire

class JTMServiceController {
    
    let CONNECTION_TIMEOUT : Double = 30
    let appSetting = applicationDelegate.appSetting
    
    func setHttpHeader() -> [String:String] {
        let httpHeader = [
//            "Authorization" : AUTH_CODE,
//            "Content-Type"  : "application/x-www-form-urlencoded",
//            "Accept"        : "application/json",
//            "resumeCode"    : appSetting?.userData == nil ? "" : "\((appSetting?.userData?.resumeCode)!)",
//            "language"      : appSetting?.currentLanguage == .THAI ? "th" : "en"
            
            //NEW VERSION
            "JTM-LANGUAGE"      : appSetting?.currentLanguage == .THAI ? "th" : "en",
            "JTM-DEVICE"        : DEVICE_NAME,
            "JTM-APPVERSION"    : version,
            "JTM-PLATFORM"      : PLATFORM,
            "JTM-DEVICEOS"      : IOS_VERSION,
            "JTM-SERVICEVERSION" : SERVICE_VERSION,
            "JTM-IMEI"          : applicationDelegate.getDeviceUUID(),
            "ResumeCode"        : appSetting?.userData == nil ? "" : "\((appSetting?.userData?.resumeCode)!)",
            "Content-type"      : "application/x-www-form-urlencoded",
        ]
        
        return httpHeader
    }
    
    func encodeSpecialCharacterfoString(text: String) -> String {
        //Parameter encoding for special digit (ex: '+', '@')
        return Alamofire.ParameterEncoding.URL.escape(text)
    }
    
    static func addPercentEncodingToUrl(urlString: String) -> String {
        return urlString.stringByAddingPercentEncodingWithAllowedCharacters(NSCharacterSet.URLQueryAllowedCharacterSet())!
    }
    
    func getHttpBodyForParams(params: [String:AnyObject]) -> NSData {
        var httpBodyList: [String] = []
        
        for (key, value) in params {
            let keyEncode = encodeSpecialCharacterfoString(key)
            let valueEncode = encodeSpecialCharacterfoString("\(value)")
            httpBodyList.append("\(keyEncode)=\(valueEncode)")
        }
        
        let httpBodyString = httpBodyList.joinWithSeparator("&")
        
        return httpBodyString.dataUsingEncoding(NSUTF8StringEncoding)!
    }
    
    func setUserRequestFor(urlString: String, params: [String : AnyObject]?) -> NSURLRequest{
        let userRequest = NSMutableURLRequest(URL: NSURL(string: urlString)!)
        userRequest.HTTPMethod = "POST"
        userRequest.timeoutInterval = CONNECTION_TIMEOUT
        userRequest.allHTTPHeaderFields = setHttpHeader()
        
        if (params != nil) {
            userRequest.HTTPBody = getHttpBodyForParams(params!)
        }
        
        return userRequest
    }
    
    func requestService(controller: JTMBaseViewController,
                        urlString: String,
                        params: [String : AnyObject]?,
                        isCanLoadParallelToOther : Bool = false,
                        isPresentLoadingIndicator : Bool = true,
                        completionHandler: (Response<AnyObject, NSError>?, status: ServiceResponseStatus)->Void) {
            if controller.isLoadingService && !isCanLoadParallelToOther  {
                return
            }
            
            if (appSetting?.isNetworkReachable)! {
                
                if isPresentLoadingIndicator {
                    controller.showLoading()
                }
                
                if !isCanLoadParallelToOther {
                    controller.isLoadingService = true
                }
                
                let userRequest = setUserRequestFor(urlString, params: params)
                Alamofire.request(userRequest).responseJSON { (response) -> Void in
                    if (response.result.isSuccess) {
                        let responseJSON = response.result.value as? NSMutableDictionary
                        
                        if (responseJSON!["status"] as! String == "invalid") {
                            controller.onConnectionError((response.request?.URLString)!)
                            completionHandler(response, status: .CONNECTION_FAIL)
                        } else {
                            //Manage connection success
                            completionHandler(response, status: .CONNECTION_SUCCESS)
                            controller.onConnectionSuccess()
                        }
                    } else {
                        if ((response.result.error?.code)! != NSURLErrorNetworkConnectionLost)
                        && ((response.result.error?.code)! != NSURLErrorNotConnectedToInternet) {
                            controller.onConnectionError((response.request?.URLString)!)
                            completionHandler(response, status: .CONNECTION_FAIL)
                        } else {
                            completionHandler(response, status: .CONNECTION_UNREACHABLE)
                        }
                    }
                    
                    controller.hideLoading()
                    if !isCanLoadParallelToOther {
                        controller.isLoadingService = false
                    }
                }
            } else {
                completionHandler(nil, status: .CONNECTION_UNREACHABLE)
            }
    }
    
    func requestBackgroundService(urlString: String, params: [String : AnyObject]?, completionHandler: (Response<AnyObject, NSError>?, status: ServiceResponseStatus)->Void) {
        if (appSetting?.isNetworkReachable)! {
            let userRequest = setUserRequestFor(urlString, params: params)
            Alamofire.request(userRequest).responseJSON { (response) -> Void in
                
                if (response.result.isSuccess) {
                    let responseJSON = response.result.value as? NSMutableDictionary
         
                    if (responseJSON!["status"] as! String == "invalid") {
                        completionHandler(response, status: .CONNECTION_FAIL)
                    } else {
                        //Manage connection success
                        completionHandler(response, status: .CONNECTION_SUCCESS)
                    }
                } else {
                    if ((response.result.error?.code)! != NSURLErrorNetworkConnectionLost)
                        && ((response.result.error?.code)! != NSURLErrorNotConnectedToInternet) {
                            completionHandler(response, status: .CONNECTION_FAIL)
                    } else {
                        completionHandler(response, status: .CONNECTION_UNREACHABLE)
                    }
                }
            }
        } else {
            completionHandler(nil, status: .CONNECTION_UNREACHABLE)
        }
    }
}
