//
//  JTMSettingViewController.swift
//  JobThai
//
//  Created by ForAlL on 1/12/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import UIKit
import ActionSheetPicker_3_0
import Crashlytics

class JTMSettingViewController: JTMBaseMenuViewController, UITableViewDataSource, UITableViewDelegate {
    
    @IBOutlet weak var settingTableView: UITableView!
    
    var sectionObjectList = NSMutableArray()
    
    let FOOTER_HEIGHT_LARGE : CGFloat = 20
    let FOOTER_HEIGHT_SMALL : CGFloat = 1
    
    let HEADER_HEIGHT_LARGE : CGFloat = 57
    let HEADER_HEIGHT_SMALL : CGFloat = 57
    
    let SETTING_CELL_PADDING : CGFloat = 25
    let CELL_HEIGHT : CGFloat = 37
    
//    let SETTING_CELL_INSET = UIEdgeInsets(top: 0, left: SETTING_CELL_PADDING, bottom: 0, right: 0)
    var alertControllerBanResume : UIAlertController?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.getValueFromPList()
        self.initObserver()
        self.setUpTableView()
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        print("Setting")
        // Track Google Analytic
        applicationDelegate.trackGoogleAnalyticScreen(SCREEN_SETTINGS)
        
        settingTableView.reloadData()
    }
    
    //not need to check login status on viewdidappear
    override func viewDidAppear(animated: Bool) {
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func setControllerFlags() {
        isCanUseAfterLogInOnly = true
    }
    
    //Mark:- UIBarButton
    override func genUISearchMenuButton() -> UIBarButtonItem? {
        return nil
    }
    
    //Mark:- PList
    
    func getValueFromPList() {
        if let path = NSBundle.mainBundle().pathForResource("ListSetting", ofType: "plist") {
            let listSetting = NSMutableArray(contentsOfFile: path)!
            for sectionObject in listSetting {
                sectionObjectList.addObject(sectionObject as! NSMutableDictionary)
            }
        }
    }
    
    //Mark:- TableView
    
    func setUpTableView() {
        self.settingTableView.backgroundColor = UIColor.whiteColor()
        self.settingTableView.tableFooterView = UIView(frame: CGRectZero)
        self.settingTableView.estimatedRowHeight = 44
        self.settingTableView.rowHeight = UITableViewAutomaticDimension
        self.settingTableView.scrollEnabled = false
        
        self.settingTableView.registerNib(UINib(nibName: "JTMSettingLanguageTableViewCell", bundle: nil), forCellReuseIdentifier: "JTMSettingLanguageTableViewCell")
        
        self.settingTableView.accessibilityIdentifier = ID_SETTING_TABLE_VIEWS
    }
    
    //Mark:- Section
    
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        if (applicationDelegate.appSetting?.isLogin == true) {
            return 2
        } else {
            return 1
        }
    }
    
    func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
//        return HEADER_HEIGHT
        if section == 0 {
            return HEADER_HEIGHT_LARGE
        } else {
            return HEADER_HEIGHT_SMALL
        }
    }
    
    func tableView(tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        
        let nib = UINib(nibName: "JTMSettingTableHeaderView", bundle: nil)
        let headerView = nib.instantiateWithOwner(self, options: nil)[0] as? JTMSettingTableHeaderView
        let languageKey = (applicationDelegate.appSetting?.currentLanguage == .THAI) ? "NameTH" : "NameEN"
        
        headerView?.headerTitle.text = self.sectionObjectList[section].objectForKey(languageKey) as? String
        
        return headerView
    }
   
    func tableView(tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        if section == 0 {
            return FOOTER_HEIGHT_LARGE
            
        } else {
            return FOOTER_HEIGHT_SMALL
        }
    }
    
    func tableView(tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        
        var footerHeight : CGFloat!
        var footerPadding : CGFloat!
        
        footerHeight = (section == 0) ? FOOTER_HEIGHT_LARGE : FOOTER_HEIGHT_SMALL
        footerPadding = (section == 0) ? 0 : SETTING_CELL_PADDING
        
        let footerView = UIView(frame: CGRectMake(0, 0, tableView.frame.width, footerHeight-1))
        footerView.backgroundColor = UIColor.whiteColor()
        
        let bottomBorder = UIView(frame: CGRectMake(footerPadding, footerHeight-1, tableView.frame.width, 0.70))
        bottomBorder.backgroundColor = JTM_LIGHT_GRAY
        
        footerView.addSubview(bottomBorder)
        
        return footerView
    
    }
    
    //Mark:- Row
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        switch section {
        case settingSection.ACCOUNT_SETTING.rawValue:
            return 2
        default:
            return 1
        }
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        let nib = UINib(nibName: "JTMSettingTableViewCell", bundle: nil)
        let cell = nib.instantiateWithOwner(self, options: nil)[0] as? JTMSettingTableViewCell
        
        setSelectedBackgroundView(cell!)
        
        let rowItem = sectionObjectList[indexPath.section].objectForKey("List")
        let languageKey = (applicationDelegate.appSetting?.currentLanguage == .THAI) ? "NameTH" : "NameEN"
        
        cell?.settingListLabel.text = rowItem![indexPath.row].objectForKey(languageKey) as? String
        
        return cell!
    }
    
    
    func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        
        return CELL_HEIGHT
    }
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        switch indexPath.section {
        case settingSection.ACCOUNT_SETTING.rawValue:
            if (applicationDelegate.appSetting?.userData?.isBan == true) {
                presentBanUser()
            } else {
                let rowItem = sectionObjectList[indexPath.section].objectForKey("List")
                let identifier = rowItem![indexPath.row].objectForKey("Destination") as! String
                let destinationViewController = self.storyboard?.instantiateViewControllerWithIdentifier(identifier)
                self.navigationController?.pushViewController(destinationViewController!, animated: true)
            }
            break
        default:
            self.showLanguagePickerView()
            break
        }
        
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func setSelectedBackgroundView(cell: UITableViewCell) {
        let backgroundView = UIView(frame: cell.frame)
        backgroundView.backgroundColor = JTM_HIGHLIGHT_COLOR
        
        cell.selectedBackgroundView = backgroundView
    }
    
    //Mark:- Log In Change
    func initObserver() {
        NSNotificationCenter.defaultCenter().addObserver(self, selector: #selector(loginChange(_:)), name: JTM_LOGIN_STATE_DID_CHANGE, object: nil)
    }
    
    func loginChange(notification:NSNotification) {
        self.settingTableView.reloadData()
    }
    
    //Mark:- Language Setting 
    func changeApplicationLanguage(languageRawValue:Int) {
        print(languageRawValue)
        
        var lang : String?
        lang = (languageRawValue == 0) ? LABEL_TH : LABEL_EN  // 0 : TH   1 : EN
        
        // Track Google Analytic Event
        applicationDelegate.trackGoogleAnalyticEvent(CAT_LANGUAGE, action: ACTION_CLICK, label: lang!)
        
        let currentLanguage = applicationDelegate.appSetting?.currentLanguage
        
        if (currentLanguage?.rawValue != languageRawValue) {
            applicationDelegate.setApplicationLanguage(appLanguage(rawValue: languageRawValue)!)
        }
    }
    
    func showLanguagePickerView() {
        let languageList = sectionObjectList[0].objectForKey("Value") as! [String]
        let currentLanguage = applicationDelegate.appSetting?.currentLanguage.rawValue
        
        let actionPicker = ActionSheetStringPicker(title: "",
                            rows: languageList,
                            initialSelection: currentLanguage!,
                            doneBlock: { (picker, row, value) -> Void in
                                self.changeApplicationLanguage(row)
                            }, cancelBlock: { (picker) -> Void in
                                
                            }, origin: self.view)

        actionPicker.setCancelButton(UIBarButtonItem(title: applicationDelegate.getLocalizeStringForKey("Cancel"), style: UIBarButtonItemStyle.Plain, target: self, action: nil))
        actionPicker.setDoneButton(UIBarButtonItem(title: applicationDelegate.getLocalizeStringForKey("Done"), style: UIBarButtonItemStyle.Done, target: self, action: nil))
        actionPicker.tapDismissAction = TapAction.Cancel
        
        actionPicker.showActionSheetPicker()
    }
    
    override func onLanguageChange() {
        super.onLanguageChange()
        self.title = applicationDelegate.getLocalizeStringForKey("NAV_TITLE_SETTING")
        self.settingTableView.reloadData()
        alertControllerBanResume = createBanUserAlertController()
    }
    
    //MARK: Band User Alert View
    func createBanUserAlertController() -> UIAlertController {
        let alertTitle : String = applicationDelegate.getLocalizeStringForKey("BAN_RESUME_ALERT_VIEW_TITLE")
        let alertMessage : String = applicationDelegate.getLocalizeStringForKey("BAN_RESUME_ALERT_VIEW_MESSAGE")
        let buttonTitle : String = applicationDelegate.getLocalizeStringForKey("OK")
        
        let alertView : UIAlertController = UIAlertController(title: alertTitle, message: alertMessage, preferredStyle: .Alert)
        let alertAction : UIAlertAction = UIAlertAction(title: buttonTitle, style: .Cancel, handler: nil)
        
        alertView.addAction(alertAction)
        return alertView
    }
    
    func presentBanUser() {
        if (alertControllerBanResume == nil) {
            alertControllerBanResume = createBanUserAlertController()
        }
        self.presentViewController(alertControllerBanResume!, animated: true, completion: nil)
    }
}
