//
//  JTMFavoriteJobTests.swift
//  JobThai
//
//  Created by kanya on 5/18/16.
//  Copyright © 2016 THiNKNET. All rights reserved.
//

import XCTest

class JTMFavoriteJobTests: XCTestCase {
    
    let SLEEP_FOR_UI_UPDATE : UInt32 = 3
    let SLEEP_FOR_LOADING_SERVICE : UInt32 = 8
    
    let app = XCUIApplication()
    let leftNavigationBarButton = XCUIApplication().navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic slide menu"]
    
    override func setUp() {
        super.setUp()
        app.launch()
        
        //Make sure user are not logged in
        sleep(SLEEP_FOR_UI_UPDATE)
        logOutUser()
    }
    
    func testSelectOnSideMenuWillGoToFavoriteJob() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        XCTAssert(app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW].exists)
    }
    
    func testSelectAvailableJobsWillGoToJobDetail() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let favoriteJobCollectionView = app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW]
        let numberOfCell = favoriteJobCollectionView.cells.count
        var isFoundAvailableJob = false
        
        for index in 0...numberOfCell {
            let cell = favoriteJobCollectionView.cells.elementBoundByIndex(index)
            if (cell.images[ID_FAVORITE_AVAILABLE_JOB_IMAGE].exists) {
                isFoundAvailableJob = true
                cell.tap()
                sleep(SLEEP_FOR_LOADING_SERVICE)
                XCTAssert(app.otherElements[ID_JOB_DETAIL_INFO_VIEW].exists)
                break
            }
        }
        
        if (!isFoundAvailableJob) {
            XCTFail("Not found available job")
        }
        
        app.navigationBars[ID_MAIN_NAVIGATION_BAR].buttons["ic back"].tap()
    }
    
    func testSelectClosedJobsWillAlert() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let favoriteJobCollectionView = app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW]
        var isFoundClosedJob = false
        let closedJobCell = favoriteJobCollectionView.cells.containingType(.Image, identifier: ID_FAVORITE_CLOSED_JOB_IMAGE)
        
        while favoriteJobCollectionView.cells.element.exists {
            if (closedJobCell.element.exists) {
                isFoundClosedJob = true
                closedJobCell.elementBoundByIndex(0).tap()
                sleep(SLEEP_FOR_UI_UPDATE)
                XCTAssert(app.alerts["alert"].exists)
                break
            }
            
            favoriteJobCollectionView.swipeUp()
        }
        
        if (!isFoundClosedJob) {
            XCTFail("Not found closed job")
        }
    }
    
    func testCollectionViewCanDoLazyLoad() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let favoriteJobCollectionView = app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW]
        var isFinishedLoadMore = false
        
        while (favoriteJobCollectionView.cells.element.exists) {
            let cell = favoriteJobCollectionView.cells[ID_FAVORITE_LOAD_MORE]
            if (cell.exists) {
                let loadMoreIndex = favoriteJobCollectionView.indexOfAccessibilityElement(cell)
                sleep(SLEEP_FOR_LOADING_SERVICE)
                let afterReloadCell = favoriteJobCollectionView.cells.elementBoundByIndex(UInt(loadMoreIndex))
                XCTAssertNotEqual(afterReloadCell, cell, "Still Load More")
                isFinishedLoadMore = true
                break
            } else {
                favoriteJobCollectionView.swipeUp()
            }
        }
        
        if (!isFinishedLoadMore) {
            XCTFail("No Load More")
        }
    }
    
    func testCollectionViewBackToTopShowHide() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let favoriteJobCollectionView = app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW]
        var isFoundBackToTop = false
        
        while (favoriteJobCollectionView.cells.element.exists) {
            if (app.buttons[ID_FAVORITE_BACK_TO_TOP].hittable) {
                isFoundBackToTop = true
                let backToTopButton = app.buttons[ID_FAVORITE_BACK_TO_TOP]
                backToTopButton.tap()
                sleep(SLEEP_FOR_UI_UPDATE)
                XCTAssertTrue(favoriteJobCollectionView.cells.elementBoundByIndex(0).hittable, "Not Scroll To Top")
                break
            }
            
            favoriteJobCollectionView.swipeUp()
        }
        
        if (!isFoundBackToTop) {
            XCTFail("No Back To Top")
        }
    }
    
    func testShowPresentSearch() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithManyFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        let searchBarButton = app.buttons[ID_NAV_BAR_QUICK_SEARCH_BUTTON]
        searchBarButton.tap()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        let searchFilterView = app.tables[ID_SEARCH_FILTER_TYPE_TABLE_VIEWS]
        XCTAssertTrue(searchFilterView.exists, "Not show present search")
        
        let cancelButton = app.buttons[ID_NAV_BAR_QUICK_SEARCH_CANCEL]
        cancelButton.tap()
    }
    
    func testShowNoResult() {
        //Log in as jtmtester+05@gmail.com []
        leftNavigationBarButton.tap()
        logInUserWithNoFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        let noresultView = app.otherElements[ID_FAVORITE_NO_RESULT]
        XCTAssertTrue(noresultView.hittable, "Not no result view")
    }
    
    func testAddAndRemoveFavoriteJob() {
        //Log in as vaiohkae@gmail.com [5387740]
        leftNavigationBarButton.tap()
        logInUserWithNoFavoriteJobs()
        
        //Go to favorite jobs
        sleep(SLEEP_FOR_UI_UPDATE)
        leftNavigationBarButton.tap()
        openFavoriteJobsController()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let favoriteCollectionView = app.collectionViews[ID_FAVORITE_JOB_COLLECTIONVIEW]
        let countCellBeforeAddFavorite = favoriteCollectionView.cells.count
        let searchBarButton = app.buttons[ID_NAV_BAR_QUICK_SEARCH_BUTTON]
        searchBarButton.tap()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        let searchFilterTableView = app.tables[ID_SEARCH_FILTER_TYPE_TABLE_VIEWS]
        searchFilterTableView.cells.elementBoundByIndex(0).tap()
        
        sleep(SLEEP_FOR_UI_UPDATE)
        let searchFilterButton = app.buttons[ID_SEARCH_FILTER_QUICK_SEARCH_BUTTON]
        searchFilterButton.tap()

        sleep(SLEEP_FOR_LOADING_SERVICE)
        let searchResultList = app.collectionViews[ID_SEARCH_RESULT_LIST_COLLECTION_VIEWS]
        searchResultList.cells.elementBoundByIndex(0).tap()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let addFavoriteButton = app.otherElements[ID_JOB_DETAIL_ADD_FAV_BUTTON]
        addFavoriteButton.tap()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        app.buttons[ID_NAV_BAR_QUICK_SEARCH_CANCEL].tap()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let countCellAfterAddFavorite = favoriteCollectionView.cells.count
        XCTAssertGreaterThan(countCellAfterAddFavorite, countCellBeforeAddFavorite, "Number of cell after add favorite should greater than before")
        let favoriteCell = favoriteCollectionView.cells.elementBoundByIndex(0)
        favoriteCell.otherElements[ID_REMOVE_FAVORITE_BUTTON].tap()
        let okButton = app.alerts["alert"].collectionViews.buttons.elementBoundByIndex(1)
        okButton.tap()
        
        sleep(SLEEP_FOR_LOADING_SERVICE)
        let noresultView = app.otherElements[ID_FAVORITE_NO_RESULT]
        XCTAssertTrue(noresultView.hittable, "Not no result view")
    }

    override func tearDown() {
        super.tearDown()
        logOutUser()
    }
    
    //MARK:- Tap favorite jobs cell
    func openFavoriteJobsController() {
        let sideMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        let favoriteButton = sideMenu.cells.staticTexts["Favorite Jobs"]
        favoriteButton.tap()
    }
    
    //MARK:- Log in/out
    func logOutUser() {
        leftNavigationBarButton.tap()
        let sideMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        let numberOfCell = sideMenu.cells.count
        
        if (numberOfCell == 8) {
            let logOutCell = sideMenu.cells.staticTexts["Log Out"]
            logOutCell.tap()
            
            let alertLogOut = app.alerts[ID_ALERT_LOGOUT]
            let logOutButton = alertLogOut.collectionViews.buttons.elementBoundByIndex(1)
            logOutButton.tap()
        } else {
            leftNavigationBarButton.tap()
        }
    }
    
    func logInUserWithManyFavoriteJobs() {
        logInUser("vaiohkae@gmail.com", password: "testuser")
    }
    
    func logInUserWithNoFavoriteJobs() {
        logInUser("jtmtester@gmail.com", password: "testuser")
    }
    
    func logInUser(userName: String, password: String) {
        let sideMenu = app.tables[ID_MAIN_MENU_TABLE_VIEWS]
        let logInCell = sideMenu.cells.staticTexts["Log In"]
        logInCell.tap()
        
        let userNameTextField = app.textFields["usernameTextField"]
        userNameTextField.tap()
        userNameTextField.clearText()
        userNameTextField.typeText(userName)
        
        let passwordTextField = app.secureTextFields["passwordTextField"]
        passwordTextField.tap()
        passwordTextField.typeText(password)
        
        app.buttons["loginButton"].tap()
    }
}
